@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: main.c,v 1.23 2002/01/11 19:06:06 miguelfreitas Exp $
+ * $Id: main.c,v 1.25 2002/02/06 11:15:04 f1rmb Exp $
  *
  * xine main for aalib
  *
@@ -264,8 +264,8 @@ int main(int argc, char *argv[]) {
   /*
    * Check xine library version 
    */
-  if(!xine_check_version(0, 9, 4)) {
-    fprintf(stderr, "require xine library version 0.9.4, found %d.%d.%d.\n", 
+  if(!xine_check_version(0, 9, 9)) {
+    fprintf(stderr, "require xine library version 0.9.9, found %d.%d.%d.\n", 
 	    xine_get_major_version(), xine_get_minor_version(),
 	    xine_get_sub_version());
     goto failure;
@@ -364,7 +364,7 @@ int main(int argc, char *argv[]) {
       
   }
 
-  aaxine.config = config_file_init (configfile);
+  aaxine.config = xine_config_file_init (configfile);
 
   /*
    * Initialize AALib
