@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: actions.c,v 1.49 2002/01/07 00:58:31 miguelfreitas Exp $
+ * $Id: actions.c,v 1.59 2002/04/21 22:17:11 f1rmb Exp $
  *
  * implementation of all the various actions for the gui (play, stop, open, pause...)
  *
@@ -45,6 +45,7 @@
 #include "mrl_browser.h"
 #include "setup.h"
 #include "viewlog.h"
+#include "errors.h"
 
 extern gGui_t          *gGui;
 extern _panel_t        *panel;
@@ -63,8 +64,6 @@ void gui_exit (xitk_widget_t *w, void *data) {
      gui_toggle_fullscreen(NULL,NULL);
 #endif
    
-  printf("xine-panel: EXIT\n");
-
   destroy_mrl_browser();
   control_exit(NULL, NULL);
   pl_exit(NULL, NULL);
@@ -81,15 +80,23 @@ void gui_exit (xitk_widget_t *w, void *data) {
 #endif
 
   xitk_stop();
+  /* 
+   * This prevent xine waiting till the end of time for an
+   * XEvent when lirc (and futur other control ways) is used to quit .
+   */
+  gui_send_expose_to_window(gGui->video_window);
   xitk_skin_unload_config(gGui->skin_config);
 }
 
 void gui_play (xitk_widget_t *w, void *data) {
 
-  fprintf(stderr, "xine-panel: PLAY\n");
-
   if (xine_get_status (gGui->xine) != XINE_PLAY) {
-    video_window_hide_logo();
+
+    if (!strncmp (gGui->filename, "xine-ui version", 15)) {
+      xine_error ("No MRL (input stream) specified");
+      return;
+    }
+
     if(!xine_play (gGui->xine, gGui->filename, 0, 0 ))
       gui_handle_xine_error();
   } else {
@@ -103,7 +110,6 @@ void gui_stop (xitk_widget_t *w, void *data) {
 
   gGui->ignore_status = 1;
   xine_stop (gGui->xine);
-  video_window_show_logo();
   gGui->ignore_status = 0; 
   panel_reset_slider ();
   panel_check_pause();
@@ -307,12 +313,13 @@ void gui_change_speed_playback(xitk_widget_t *w, void *data) {
 
 void gui_set_current_position (int pos) {
 
-  if (!xine_is_stream_seekable (gGui->xine)) 
-    return;
+  if((xine_is_stream_seekable (gGui->xine)) == 0)
+     return;
 
   gGui->ignore_status = 1;
   if(!xine_play (gGui->xine, gGui->filename, pos, 0))
     gui_handle_xine_error();
+  
   gGui->ignore_status = 0;
   panel_check_pause();
 }
@@ -370,8 +377,8 @@ void gui_nextprev(xitk_widget_t *w, void *data) {
       gui_set_current_mrl(gGui->playlist[gGui->playlist_cur]);
       if(!xine_play (gGui->xine, gGui->filename, 0, 0 ))
 	gui_handle_xine_error();
+
     } else {
-      video_window_show_logo();
       gGui->playlist_cur = 0;
     }
     gGui->ignore_status = 0;
@@ -441,7 +448,7 @@ void gui_notify_demux_branched () {
 void gui_control_show(xitk_widget_t *w, void *data) {
 
   if(control_is_running() && !control_is_visible())
-    control_toggle_panel_visibility(NULL, NULL);
+    control_toggle_visibility(NULL, NULL);
   else if(!control_is_running())
     control_panel();
   else
@@ -451,7 +458,7 @@ void gui_control_show(xitk_widget_t *w, void *data) {
 void gui_setup_show(xitk_widget_t *w, void *data) {
 
   if (setup_is_running() && !setup_is_visible())
-    setup_toggle_panel_visibility(NULL, NULL);
+    setup_toggle_visibility(NULL, NULL);
   else if(!setup_is_running())
     setup_panel();
   else
@@ -461,7 +468,7 @@ void gui_setup_show(xitk_widget_t *w, void *data) {
 void gui_viewlog_show(xitk_widget_t *w, void *data) {
 
   if (viewlog_is_running() && !viewlog_is_visible())
-    viewlog_toggle_panel_visibility(NULL, NULL);
+    viewlog_toggle_visibility(NULL, NULL);
   else if(!viewlog_is_running())
     viewlog_window();
   else
@@ -525,13 +532,11 @@ void gui_decrease_audio_volume(void) {
   }
 }
 
-void gui_change_zoom(int zoom_dx, int zoom_dy) {
-  
-  gGui->vo_driver->set_property (gGui->vo_driver, VO_PROP_ZOOM_X,
-				 gGui->vo_driver->get_property (gGui->vo_driver, VO_PROP_ZOOM_X) + zoom_dx);
-  gGui->vo_driver->set_property (gGui->vo_driver, VO_PROP_ZOOM_Y,
-				 gGui->vo_driver->get_property (gGui->vo_driver, VO_PROP_ZOOM_Y) + zoom_dy);
-  
+void gui_change_zoom(int zoom_d) {
+
+  gGui->vo_driver->set_property (gGui->vo_driver, VO_PROP_ZOOM_FACTOR,
+				 gGui->vo_driver->get_property (gGui->vo_driver, VO_PROP_ZOOM_FACTOR) + zoom_d);
+
   if (panel_is_visible())  {
     XRaiseWindow (gGui->display, gGui->panel_window);
     XSetTransientForHint (gGui->display, 
@@ -563,3 +568,28 @@ void gui_toggle_tvmode(void) {
   				 gGui->vo_driver->get_property (gGui->vo_driver, VO_PROP_ASPECT_RATIO));
   				 
 }
+
+/*
+ * Send an Expose event to given window.
+ */
+void gui_send_expose_to_window(Window window) {
+  XEvent xev;
+
+  if(window == None)
+    return;
+
+  xev.xany.type          = Expose;
+  xev.xexpose.type       = Expose;
+  xev.xexpose.send_event = True;
+  xev.xexpose.display    = gGui->display;
+  xev.xexpose.window     = window;
+  xev.xexpose.count      = 0;
+  
+  XLockDisplay(gGui->display);
+  if(!XSendEvent(gGui->display, window, False, ExposureMask, &xev)) {
+    fprintf(stderr, "XSendEvent(display, 0x%x ...) failed.\n", (unsigned int) window);
+  }
+  XSync(gGui->display, False);
+  XUnlockDisplay(gGui->display);
+  
+}
