@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: actions.h,v 1.17 2001/12/10 00:16:44 f1rmb Exp $
+ * $Id: actions.h,v 1.20 2002/03/21 00:22:52 f1rmb Exp $
  *
  * all the various actions for the gui (play, stop, open, pause...)
  *
@@ -87,9 +87,11 @@ void gui_increase_audio_volume(void);
 
 void gui_decrease_audio_volume(void);
 
-void gui_change_zoom(int zoom_dx, int zoom_dy);
+void gui_change_zoom(int zoom_d);
 
 void gui_reset_zoom(void);
 
 void gui_toggle_tvmode(void);
+
+void gui_send_expose_to_window(Window window);
 #endif
