@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: control.c,v 1.33 2002/01/15 22:22:35 f1rmb Exp $
+ * $Id: control.c,v 1.44 2002/04/21 22:17:11 f1rmb Exp $
  *
  *
  */
@@ -42,6 +42,11 @@
 #include "event.h"
 #include "actions.h"
 #include "skins.h"
+#include "errors.h"
+
+#ifndef __GNUC__
+#define __FUNCTION__    __func__
+#endif
 
 extern gGui_t          *gGui;
 
@@ -68,6 +73,19 @@ typedef struct {
 
 static _control_t    *control = NULL;
 
+/*
+ *
+ */
+void control_show_tips(int enabled, unsigned long timeout) {
+  
+  if(control) {
+    if(enabled)
+      xitk_set_widgets_tips_timeout(control->widget_list, timeout);
+    else
+      xitk_disable_widgets_tips(control->widget_list);
+  }
+}
+
 /*
  * Get current property 'prop' value from vo_driver.
  */
@@ -249,10 +267,11 @@ void control_raise_window(void) {
     }
   }
 }
+
 /*
  * Hide/show the control panel
  */
-void control_toggle_panel_visibility (xitk_widget_t *w, void *data) {
+void control_toggle_visibility (xitk_widget_t *w, void *data) {
   
   if(control != NULL) {
     if (control->visible && control->running) {
@@ -304,33 +323,52 @@ void control_handle_event(XEvent *event, void *data) {
  * Change the current skin.
  */
 void control_change_skins(void) {
-  
+  ImlibImage   *new_img, *old_img;
+  XSizeHints    hint;
+
   if(control_is_running()) {
     
+    xitk_skin_lock(gGui->skin_config);
+    xitk_hide_widgets(control->widget_list);
+
     XLockDisplay(gGui->display);
     
-    Imlib_destroy_image(gGui->imlib_data, control->bg_image);
-    
-    if(!(control->bg_image = 
-	 Imlib_load_image(gGui->imlib_data,
-			  xitk_skin_get_skin_filename(gGui->skin_config, "CtlBG")))) {
+    if(!(new_img = Imlib_load_image(gGui->imlib_data,
+				    xitk_skin_get_skin_filename(gGui->skin_config, "CtlBG")))) {
       xine_error(_("%s(): couldn't find image for background\n"), __FUNCTION__);
       exit(-1);
     }
     
+    hint.width  = new_img->rgb_width;
+    hint.height = new_img->rgb_height;
+    hint.flags  = PSize;
+    XSetWMNormalHints(gGui->display, control->window, &hint);
+    
     XResizeWindow (gGui->display, control->window,
-		   (unsigned int)control->bg_image->rgb_width,
-		   (unsigned int)control->bg_image->rgb_height);
-    
-    /*
-     * We should here, otherwise new skined window will have wrong size.
-     */
-    XFlush(gGui->display);
-    
-    Imlib_apply_image(gGui->imlib_data, control->bg_image, control->window);
+		   (unsigned int)new_img->rgb_width,
+		   (unsigned int)new_img->rgb_height);
     
     XUnlockDisplay(gGui->display);
     
+    while(!xitk_is_window_size(gGui->display, control->window, 
+			       new_img->rgb_width, new_img->rgb_height)) {
+      xine_usec_sleep(10000);
+    }
+    
+    old_img = control->bg_image;
+    control->bg_image = new_img;
+    
+    XLockDisplay(gGui->display);
+    
+    XSetTransientForHint(gGui->display, control->window, gGui->video_window);
+    
+    Imlib_destroy_image(gGui->imlib_data, old_img);
+    Imlib_apply_image(gGui->imlib_data, new_img, control->window);
+    
+    XUnlockDisplay(gGui->display);
+
+    xitk_skin_unlock(gGui->skin_config);
+    
     xitk_change_skins_widget_list(control->widget_list, gGui->skin_config);
     xitk_paint_widget_list(control->widget_list);
   }
@@ -364,6 +402,7 @@ void control_panel(void) {
   xitk_label_widget_t        lbl;
   xitk_slider_widget_t       sl;
   xitk_combo_widget_t        cmb;
+  xitk_widget_t             *w;
   long                       data[1];
 
   /* This shouldn't be happend */
@@ -413,7 +452,7 @@ void control_panel(void) {
   control->window = XCreateWindow (gGui->display,
 				   gGui->imlib_data->x.root,
 				   hint.x, hint.y, hint.width, hint.height, 0, 
-				   gGui->imlib_data->x.depth, CopyFromParent, 
+				   gGui->imlib_data->x.depth, InputOutput, 
 				   gGui->imlib_data->x.visual,
 				   CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect,
 				   &attr);
@@ -421,9 +460,7 @@ void control_panel(void) {
   XSetStandardProperties(gGui->display, control->window, title, title,
  			 None, NULL, 0, &hint);
   
-  XSelectInput(gGui->display, control->window,
-	       ButtonPressMask | ButtonReleaseMask | KeymapStateMask | PointerMotionMask
-	       | KeyPressMask | ExposureMask | StructureNotifyMask);
+  XSelectInput(gGui->display, control->window, INPUT_MOTION | KeymapStateMask);
   
   /*
    * layer above most other things, like gnome panel
@@ -507,7 +544,8 @@ void control_panel(void) {
     xitk_list_append_content(control->widget_list->l,
 			    (control->hue = xitk_slider_create(gGui->skin_config, &sl)));
     xitk_slider_set_pos(control->widget_list, control->hue, cur);
-
+    xitk_set_widget_tips(control->hue, _("Control HUE value"));
+  
     lbl.skin_element_name = "CtlHueLbl";
     lbl.label             = _("Hue");
     xitk_list_append_content(control->widget_list->l,
@@ -530,6 +568,7 @@ void control_panel(void) {
     xitk_list_append_content(control->widget_list->l,
 	      (control->sat = xitk_slider_create(gGui->skin_config, &sl)));
     xitk_slider_set_pos(control->widget_list, control->sat, cur);
+    xitk_set_widget_tips(control->sat, _("Control SATURATION value"));
 
     lbl.skin_element_name = "CtlSatLbl";
     lbl.label             = _("Sat");
@@ -553,6 +592,7 @@ void control_panel(void) {
     xitk_list_append_content(control->widget_list->l,
 	    (control->bright = xitk_slider_create(gGui->skin_config, &sl)));
     xitk_slider_set_pos(control->widget_list, control->bright, cur);
+    xitk_set_widget_tips(control->bright, _("Control BRIGHTNESS value"));
 
     lbl.skin_element_name = "CtlBrightLbl";
     lbl.label             = _("Brt");
@@ -576,6 +616,7 @@ void control_panel(void) {
     xitk_list_append_content(control->widget_list->l,
 	      (control->contr = xitk_slider_create(gGui->skin_config, &sl)));
     xitk_slider_set_pos(control->widget_list, control->contr, cur);
+    xitk_set_widget_tips(control->contr, _("Control CONTRAST value"));
 
     lbl.skin_element_name = "CtlContLbl";
     lbl.label             = _("Ctr");
@@ -656,10 +697,13 @@ void control_panel(void) {
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   xitk_list_append_content (control->widget_list->l, 
-			  xitk_labelbutton_create (gGui->skin_config, &lb));
+			    (w = xitk_labelbutton_create (gGui->skin_config, &lb)));
+  xitk_set_widget_tips(w, _("Close control window"));
 
   XMapRaised(gGui->display, control->window); 
 
+  XUnlockDisplay (gGui->display);
+
   control->widget_key = 
     xitk_register_event_handler("control", 
 				control->window, 
@@ -670,6 +714,4 @@ void control_panel(void) {
   
   control->visible = 1;
   control->running = 1;
-
-  XUnlockDisplay (gGui->display);
 }
