@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: control.h,v 1.5 2001/10/21 00:28:53 f1rmb Exp $
+ * $Id: control.h,v 1.8 2002/03/21 00:22:52 f1rmb Exp $
  *
  */
 
@@ -29,7 +29,8 @@ void control_change_skins(void);
 void control_exit(xitk_widget_t *, void *);
 int control_is_visible(void);
 int control_is_running(void);
-void control_toggle_panel_visibility(xitk_widget_t *, void *);
+void control_toggle_visibility(xitk_widget_t *, void *);
 void control_raise_window(void);
+void control_show_tips(int enabled, unsigned long timeout);
 
 #endif
