@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,9 +17,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: event.c,v 1.71 2002/01/14 21:44:41 f1rmb Exp $
+ * $Id: event.c,v 1.84 2002/04/27 13:26:38 f1rmb Exp $
  *
- * gui inititalization and top-level event handling stuff
+ * gui initialization and top-level event handling stuff
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -38,6 +38,7 @@
 #include <pthread.h>
 #include <signal.h>
 #include <limits.h>
+#include <zlib.h>
 
 #include "xitk.h"
 
@@ -56,6 +57,8 @@
 #include "xscreensaver-remote.h"
 #include "mrl_browser.h"
 #include "skins.h"
+#include "errors.h"
+#include "network.h"
 
 #ifdef HAVE_LIRC
 extern int no_lirc;
@@ -124,6 +127,13 @@ typedef struct {
 
 static screen_savers_t    ssavers;
 
+/*
+ * Callback for snapshots saving location.
+ */
+static void snapshot_loc_cb(void *data, cfg_entry_t *cfg) {
+  gGui->snapshot_location = cfg->str_value;
+}
+
 static int actions_on_start(action_id_t actions[], action_id_t a) {
   int i = 0;
   while(actions[i] != ACTID_NOKEY) {
@@ -313,32 +323,21 @@ void gui_execute_action_id(action_id_t action) {
   switch(action) {
     
   case ACTID_WINDOWREDUCE:
+    video_window_set_mag (0.8 * video_window_get_mag());
+    break;
   case ACTID_WINDOWENLARGE:
-    if(!video_window_is_fullscreen()) {
-      int x, y;
-      unsigned int w, h, b, d;
-      Window rootwin;
-      
-      XLockDisplay (gGui->display);
-      
-      if(XGetGeometry(gGui->display, 
-		      gGui->video_window, &rootwin, 
-		      &x, &y, &w, &h, &b, &d) != BadDrawable) {
-      }
+    video_window_set_mag (1.2 * video_window_get_mag());
+    break;
 
-      if(action == ACTID_WINDOWREDUCE) {
-	w /= 1.2;
-	h /= 1.2;
-      }
-      else {
-	w *= 1.2;
-	h *= 1.2;
-      }
-      
-      XResizeWindow (gGui->display, gGui->video_window, w, h);
-      XUnlockDisplay(gGui->display);
-      
-    }
+  case ACTID_ZOOM_1_1:
+  case ACTID_WINDOW100:
+    video_window_set_mag (1.0);
+    break;
+  case ACTID_WINDOW200:
+    video_window_set_mag (2.0);
+    break;
+  case ACTID_WINDOW50:
+    video_window_set_mag (0.5);
     break;
 
   case ACTID_SPU_NEXT:
@@ -575,44 +574,12 @@ void gui_execute_action_id(action_id_t action) {
     xine_send_event(gGui->xine, &xine_event);
     break;
     
-  case ACTID_ZOOM_1_1:
-    if(!video_window_is_fullscreen()) {
-      int width, height, ratio_code, format;
-      uint8_t *y, *u, *v;
-      
-      if( xine_get_current_frame (gGui->xine, &width, &height,
-				  &ratio_code, &format,
-				  &y, &u, &v) ) {
-	
-	XLockDisplay (gGui->display);
-	XResizeWindow (gGui->display, gGui->video_window, width, height);
-	XUnlockDisplay(gGui->display);
-      }
-    }
-    break;
-    
   case ACTID_ZOOM_IN:
-    gui_change_zoom(1,1);
+    gui_change_zoom(1);
     break;
 
   case ACTID_ZOOM_OUT:
-    gui_change_zoom(-1,-1);
-    break;
-
-  case ACTID_ZOOM_X_IN:
-    gui_change_zoom(1,0);
-    break;
-
-  case ACTID_ZOOM_X_OUT:
-    gui_change_zoom(-1,0);
-    break;
-
-  case ACTID_ZOOM_Y_IN:
-    gui_change_zoom(0,1);
-    break;
-
-  case ACTID_ZOOM_Y_OUT:
-    gui_change_zoom(0,-1);
+    gui_change_zoom(-1);
     break;
 
   case ACTID_ZOOM_RESET:
@@ -644,41 +611,6 @@ void gui_execute_action_id(action_id_t action) {
   }
 }	    
 
-/*
- * Display an error window error from a xine engine error.
- */
-void gui_handle_xine_error(void) {
-  int err;
-
-  err = xine_get_error(gGui->xine);
-
-  switch(err) {
-
-  case XINE_ERROR_NONE:
-    /* noop */
-    break;
-    
-  case XINE_ERROR_NO_INPUT_PLUGIN:
-    xitk_window_dialog_error(gGui->imlib_data, 
-			     _("- xine engine error -\n\nThere is no available input plugin "
-			       "available to handle '%s'.\n"), gGui->filename);
-    break;
-    
-  case XINE_ERROR_NO_DEMUXER_PLUGIN:
-    xitk_window_dialog_error(gGui->imlib_data, 
-			     _("- xine engine error -\n\nThere is no available demuxer plugin "
-			       "to handle '%s'.\n"), gGui->filename);
-    break;
-
-  default:
-    xitk_window_dialog_error(gGui->imlib_data, 
-			     _("- xine engine error -\n\n!! Unhandled error !!\n"));
-    break;
-  }
-
-
-}
-
 /*
  * top-level event handler
  */
@@ -750,12 +682,12 @@ void gui_status_callback (int nStatus) {
       gui_set_current_mrl(gGui->playlist[gGui->playlist_cur]);
       if(!xine_play (gGui->xine, gGui->filename, 0, 0 ))
 	gui_handle_xine_error();
+
     } else {
       
       if(gGui->actions_on_start[0] == ACTID_QUIT)
 	gui_exit(NULL, NULL);
       
-      video_window_show_logo();
       gGui->playlist_cur--;
     }
   }
@@ -843,12 +775,12 @@ static void gui_find_visual (Visual **visual_return, int *depth_return) {
       visual = vinfo[0].visual;
       XFree(vinfo);
     }
-  }	 
+  }
 
   if (depth == 0) {
     XVisualInfo vinfo;
 
-    XGetWindowAttributes(gGui->display, gGui->imlib_data->x.root, &attribs);
+    XGetWindowAttributes(gGui->display, (DefaultRootWindow(gGui->display)), &attribs);
 
     depth = attribs.depth;
   
@@ -868,17 +800,12 @@ static void gui_find_visual (Visual **visual_return, int *depth_return) {
     *visual_return = visual;
 }
 
-
 /*
  * Initialize the GUI
  */
 void gui_init (int nfiles, char *filenames[]) {
-
   int                   i;
-  XColor                dummy;
   char                 *display_name = ":0.0";
-  char                  buffer[XITK_PATH_MAX + XITK_NAME_MAX + 1]; /* Enought ?? ;-) */
-  ImlibInitParams	imlib_init;
 
   /*
    * init playlist
@@ -923,71 +850,17 @@ void gui_init (int nfiles, char *filenames[]) {
 						   1, "use wm layer property to place window on top", 
 						   NULL, NULL, NULL);
 
+  gGui->snapshot_location = gGui->config->register_string (gGui->config, "gui.snapshotdir", 
+							   (char*) (xine_get_homedir()),
+							   "where snapshots will be saved",
+							   NULL, snapshot_loc_cb, NULL);
   XLockDisplay (gGui->display);
 
   gGui->screen = DefaultScreen(gGui->display);
 
   gui_find_visual(&gGui->visual, &gGui->depth);
 
-  imlib_init.flags = PARAMS_VISUALID;
-  imlib_init.visualid = gGui->visual->visualid;
-  if (gGui->install_colormap && (gGui->visual->class & 1)) {
-      /*
-       * We're using a visual with changable colors / colormaps
-       * (According to the comment in X11/X.h, an odd display class
-       * has changable colors), and the user requested to install a
-       * private colormap for xine.  Allocate a fresh colormap for
-       * Imlib and Xine.
-       */
-      Colormap cm;
-      cm = XCreateColormap(gGui->display, 
-			   RootWindow(gGui->display, gGui->screen),
-			   gGui->visual, AllocNone);
-
-      imlib_init.cmap = cm;
-      imlib_init.flags |= PARAMS_COLORMAP;
-  }
-  gGui->imlib_data = Imlib_init_with_params (gGui->display, &imlib_init);
-  if (gGui->imlib_data == NULL) {
-    fprintf(stderr, _("Unable to initialize Imlib\n"));
-    exit(1);
-  }
-
-  /* 
-   * Create logo image displayed into video window from
-   * the official Xine logo.
-   */
-  sprintf(buffer, "%s/xine_logo.png", XINE_SKINDIR);
-  if((gGui->video_window_logo_image = Imlib_load_image(gGui->imlib_data, buffer)) == NULL) {
-    xine_error(_("Unable to load %s logo\n"), buffer);
-    exit(1);
-  }
-
-  Imlib_render(gGui->imlib_data, gGui->video_window_logo_image,
-	       gGui->video_window_logo_image->rgb_width,
-	       gGui->video_window_logo_image->rgb_height);
-
-  gGui->video_window_logo_pixmap.image = 
-    Imlib_move_image(gGui->imlib_data, gGui->video_window_logo_image);
-
-  gGui->video_window_logo_pixmap.width = 
-    gGui->video_window_logo_image->rgb_width;
-
-  gGui->video_window_logo_pixmap.height = 
-    gGui->video_window_logo_image->rgb_height;
-
-  XAllocNamedColor(gGui->display, 
-		   Imlib_get_colormap(gGui->imlib_data),
-		   "black", &gGui->black, &dummy);
-
-  /*
-   * create an icon pixmap
-   */
-  
-  gGui->icon = XCreateBitmapFromData (gGui->display, 
-				      gGui->imlib_data->x.root,
-				      xine_bits, 40, 40);
-
+  gui_init_imlib (gGui->visual);
 
   XUnlockDisplay (gGui->display);
 
@@ -1005,12 +878,61 @@ void gui_init (int nfiles, char *filenames[]) {
   gGui->running = 1;
 
   video_window_init ();
-  
+
   panel_init ();
 
   disable_screensavers();
 }
 
+
+void gui_init_imlib (Visual *vis) {
+  XColor                dummy;
+  ImlibInitParams	imlib_init;
+
+  /*
+   * This routine isn't re-entrant. I cannot find a Imlib_cleanup either.
+   * However, we have to reinitialize Imlib if we have to change the visual.
+   * This will be a (small) memory leak.
+   */
+  imlib_init.flags = PARAMS_VISUALID;
+  imlib_init.visualid = vis->visualid;
+  if (gGui->install_colormap && (vis->class & 1)) {
+      /*
+       * We're using a visual with changable colors / colormaps
+       * (According to the comment in X11/X.h, an odd display class
+       * has changable colors), and the user requested to install a
+       * private colormap for xine.  Allocate a fresh colormap for
+       * Imlib and Xine.
+       */
+      Colormap cm;
+      cm = XCreateColormap(gGui->display, 
+			   RootWindow(gGui->display, gGui->screen),
+			   vis, AllocNone);
+
+      imlib_init.cmap = cm;
+      imlib_init.flags |= PARAMS_COLORMAP;
+  }
+  gGui->imlib_data = Imlib_init_with_params (gGui->display, &imlib_init);
+  if (gGui->imlib_data == NULL) {
+    fprintf(stderr, _("Unable to initialize Imlib\n"));
+    exit(1);
+  }
+
+  gGui->colormap = Imlib_get_colormap (gGui->imlib_data);
+
+  XAllocNamedColor(gGui->display, gGui->colormap,
+		   "black", &gGui->black, &dummy);
+
+  /*
+   * create an icon pixmap
+   */
+  
+  gGui->icon = XCreateBitmapFromData (gGui->display, 
+				      gGui->imlib_data->x.root,
+				      xine_bits, 40, 40);
+}
+
+
 /*
  *
  */
@@ -1048,6 +970,10 @@ void gui_run (void) {
 
   if(gGui->actions_on_start[0] != ACTID_NOKEY) {
 
+    /* Popup setup window if there is no config file */
+    if(actions_on_start(gGui->actions_on_start, ACTID_SETUP))
+      gui_execute_action_id(ACTID_SETUP);
+    
     /*  The user wants to hide control panel  */
     if(panel_is_visible() && (actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_VISIBLITY)))
       gui_execute_action_id(ACTID_TOGGLE_VISIBLITY);
@@ -1098,10 +1024,12 @@ void gui_run (void) {
 						 gui_dndcallback, 
 						 NULL, NULL);
   
+  start_remote_server();
+
   xitk_run();
 
   gGui->running = 0;
-  
+
   kbindings_save_kbinding(gGui->kbindings);
   kbindings_free_kbinding(&gGui->kbindings);
 
