@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: event.h,v 1.31 2002/01/09 00:36:05 f1rmb Exp $
+ * $Id: event.h,v 1.39 2002/04/27 13:26:38 f1rmb Exp $
  *
  */
 
@@ -32,21 +32,6 @@
 #include "xitk.h"
 #include "i18n.h"
 
-#ifdef __GNUC__
-#define perr(FMT,ARGS...) {fprintf(stderr, FMT, ##ARGS);fflush(stderr);}
-#else	/* C99 version: */
-#define perr(...)	  {fprintf(stderr, __VA_ARGS__);fflush(stderr);}
-#endif
-
-#ifdef __GNUC__
-#define xine_error(FMT,ARGS...) xitk_window_dialog_error(gGui->imlib_data, FMT, ##ARGS)
-#define xine_info(FMT,ARGS...)  xitk_window_dialog_info(gGui->imlib_data, FMT, ##ARGS)
-#else	/* C99 version: */
-#define xine_error(...)	        xitk_window_dialog_error(gGui->imlib_data, __VA_ARGS__)
-#define xine_info(FMT,ARGS...)  xitk_window_dialog_info(gGui->imlib_data,  __VA_ARGS__)
-#endif
-
-
 #define MAX_PLAYLIST_LENGTH  1024
 
 typedef struct {
@@ -73,6 +58,7 @@ typedef struct {
   Visual	      *visual;
   XColor               black;
   Pixmap               icon;
+  Colormap             colormap;
 
   int		       prefered_visual_class;
   VisualID	       prefered_visual_id;
@@ -84,8 +70,6 @@ typedef struct {
   ImlibData           *imlib_data;
 
   Window               video_window; 
-  ImlibImage          *video_window_logo_image;
-  xitk_image_t         video_window_logo_pixmap;
   int                  cursor_visible;
   int                  cursor_grabbed;
 
@@ -122,6 +106,12 @@ typedef struct {
 
   int                  layer_above;
   int                  reparent_hack;
+
+  int                  network;
+  
+  int                  use_root_window;
+
+  char                *snapshot_location;
   
   kbinding_t          *kbindings;
   
@@ -153,6 +143,8 @@ char *gui_get_configfile(void);
 
 void gui_init (int nfiles, char *filenames[]);
 
+void gui_init_imlib (Visual *vis);
+
 void gui_run (void);
 
 void gui_status_callback (int nStatus) ;
@@ -166,8 +158,6 @@ void gui_handle_event (XEvent *event, void *data);
 char *gui_next_mrl_callback (void) ;
 
 void gui_branched_callback (void) ;
-
-void gui_handle_xine_error(void);
 #endif
 
 
