@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: kbindings.c,v 1.15 2002/01/08 20:21:30 heikos Exp $
+ * $Id: kbindings.c,v 1.25 2002/04/15 15:55:47 jkeil Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -34,6 +34,7 @@
 
 #include "event.h"
 #include "kbindings.h"
+#include "errors.h"
 #include "xitk.h"
 
 extern gGui_t                 *gGui;
@@ -122,36 +123,19 @@ typedef struct {
 } kbinding_file_t;
 
 
-#ifndef	HAVE_STRSEP
-static char *
-strsep(char **stringp, char *delim)
-{
-    char *start = *stringp;
-    char *cp;
-    char ch;
-
-    if (start == NULL)
-	return NULL;
-
-    for (cp = start; ch = *cp; cp++) {
-	if (strchr(delim, ch)) {
-	    *cp++ = 0;
-	    *stringp = cp;
-	    return start;
-	}
-    }
-    *stringp = NULL;
-    return start;
-}
-#endif
-
 /* 
  * Default key mapping table.
  */
 static kbinding_entry_t default_binding_table[] = {
-  { "Reduce the output window size.",
+  { "Reduce the output window size by factor 1.2.",
     "WindowReduce",           ACTID_WINDOWREDUCE            , "less",     KEYMOD_NOMOD   },
-  { "Enlarge the output window size.",
+  { "set video output window to 50%",
+    "Window50",               ACTID_WINDOW50                , "1",        KEYMOD_META    },
+  { "set video output window to 100%",
+    "Window100",              ACTID_WINDOW100               , "2",        KEYMOD_META    },
+  { "set video output window to 200%",
+    "Window200",              ACTID_WINDOW200               , "3",        KEYMOD_META    },
+  { "Enlarge the output window size by factor 1.2.",
     "WindowEnlarge",          ACTID_WINDOWENLARGE           , "greater",  KEYMOD_NOMOD   },
   { "Select next sub picture (subtitle) channel.",
     "SpuNext",                ACTID_SPU_NEXT                , "period",   KEYMOD_NOMOD   },
@@ -262,23 +246,15 @@ static kbinding_entry_t default_binding_table[] = {
   { "Next event.",
     "EventNext",              ACTID_EVENT_NEXT              , "KP_Next",  KEYMOD_NOMOD   },
   { "Previous angle event.",
-    "EventAnglePrior",        ACTID_EVENT_ANGLE_PRIOR       , "KP_7",     KEYMOD_NOMOD   },
+    "EventAnglePrior",        ACTID_EVENT_ANGLE_PRIOR       , "KP_End",     KEYMOD_NOMOD   },
   { "Next angle event.",
-    "EventAngleNext",         ACTID_EVENT_ANGLE_NEXT        , "KP_1",     KEYMOD_NOMOD   },
+    "EventAngleNext",         ACTID_EVENT_ANGLE_NEXT        , "KP_Home",     KEYMOD_NOMOD   },
   { "Select event.",
     "EventSelect",            ACTID_EVENT_SELECT            , "KP_Enter", KEYMOD_NOMOD   },
   { "Zoom into video.",
     "ZoomIn",                 ACTID_ZOOM_IN                 , "z",        KEYMOD_NOMOD   },
   { "Zoom out of video.",
     "ZoomOut",                ACTID_ZOOM_OUT                , "Z",        KEYMOD_NOMOD   },
-  { "Zoom into video horizontally, distorting aspect ratio.",
-    "ZoomInX",                ACTID_ZOOM_X_IN               , "z",        KEYMOD_CONTROL },
-  { "Zoom out of video horizontally, distorting aspect ratio.",
-    "ZoomOutX",               ACTID_ZOOM_X_OUT              , "Z",        KEYMOD_CONTROL },
-  { "Zoom into video vertically, distorting aspect ratio.",
-    "ZoomInY",                ACTID_ZOOM_Y_IN               , "z",        KEYMOD_META    },
-  { "Zoom out of video vertically, distorting aspect ratio.",
-    "ZoomOutY",               ACTID_ZOOM_Y_OUT              , "Z",        KEYMOD_META    },
   { "Reset zooming.",
     "ZoomReset",              ACTID_ZOOM_RESET              , "z",        KEYMOD_CONTROL | KEYMOD_META    },
   { "Toggle TV modes on the DXR3",
@@ -465,7 +441,7 @@ static void _kbindings_add_entry(kbinding_t *kbt, user_kbinding_t *ukb) {
       
       modifier = KEYMOD_NOMOD;
 
-      while((p = strsep(&ukb->modifier, ",")) != NULL) {
+      while((p = xine_strsep(&ukb->modifier, ",")) != NULL) {
 	
 	_kbindings_set_pos_to_next_char(&p);
 	
@@ -533,7 +509,7 @@ static void _kbindings_replace_entry(kbinding_t *kbt, user_kbinding_t *ukb) {
 	char *p;
 	int   modifier = KEYMOD_NOMOD;
 	
-	while((p = strsep(&ukb->modifier, ",")) != NULL) {
+	while((p = xine_strsep(&ukb->modifier, ",")) != NULL) {
 
 	  _kbindings_set_pos_to_next_char(&p);
 	  
@@ -978,9 +954,9 @@ void kbindings_handle_kbinding(kbinding_t *kbt, XEvent *event) {
 
     XLockDisplay (gGui->display);
     len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
-    XUnlockDisplay (gGui->display);
-    
     k = kbindings_lookup_binding(kbt, XKeysymToString(mykey), modifier);
+    XUnlockDisplay (gGui->display);
+
     if(k) {
       gui_execute_action_id(k->action_id);
     }
