@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: lang.h,v 1.2 2001/12/14 11:24:17 f1rmb Exp $
+ * $Id: lang.h,v 1.4 2002/03/24 01:33:33 f1rmb Exp $
  *
  */
 
@@ -31,7 +31,9 @@ typedef enum {
   FRENCH,
   GERMAN,
   PORTUGUESE,
-  SPANISH
+  SPANISH,
+  POLISH,
+  UKRAINIAN
 } lang_code_t;
 
 typedef struct {
