@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  *
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: main.c,v 1.74 2002/01/09 08:42:16 f1rmb Exp $
+ * $Id: main.c,v 1.84 2002/04/25 20:03:23 f1rmb Exp $
  *
  * xine main for X11
  *
@@ -45,6 +45,7 @@
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <sys/time.h>
+#include <sys/types.h>
 #include <signal.h>
 
 #include <X11/Xlib.h>
@@ -86,7 +87,7 @@ int       no_lirc;
 #define DISPLAY_KEYMAP          1002
 
 /* options args */
-static const char *short_options = "?hHgfv"
+static const char *short_options = "?hHgfvn"
 #ifdef HAVE_LIRC
  "L"
 #endif
@@ -96,7 +97,7 @@ static const char *short_options = "?hHgfv"
 #ifdef DEBUG
  "d:"
 #endif
- "u:a:V:A:p::s:";
+ "u:a:V:A:p::s:R";
 static struct option long_options[] = {
   {"help"           , no_argument      , 0, 'h'                      },
 #ifdef HAVE_LIRC
@@ -111,12 +112,14 @@ static struct option long_options[] = {
   {"audio-driver"   , required_argument, 0, 'A'                      },
   {"auto-play"      , optional_argument, 0, 'p'                      },
   {"auto-scan"      , required_argument, 0, 's'                      },
-  {"hide-gui"       , no_argument,       0, 'g'                      },
-  {"hide-video"     , no_argument,       0, 'H'                      },
-  {"fullscreen"     , no_argument,       0, 'f'                      },
+  {"hide-gui"       , no_argument      , 0, 'g'                      },
+  {"hide-video"     , no_argument      , 0, 'H'                      },
+  {"fullscreen"     , no_argument      , 0, 'f'                      },
   {"visual"	    , required_argument, 0,  OPTION_VISUAL           },
   {"install"	    , no_argument      , 0,  OPTION_INSTALL_COLORMAP },
   {"keymap"         , optional_argument, 0,  DISPLAY_KEYMAP          },
+  {"network"        , no_argument      , 0, 'n'                      },
+  {"root"           , no_argument      , 0, 'R'                      },
   {"version"        , no_argument      , 0, 'v'                      },
   {0                , no_argument      , 0,  0                       }
 };
@@ -126,7 +129,7 @@ static struct option long_options[] = {
  */
 void show_version(void) {
 
-  printf("This is xine (X11 gui) - a free video player v%s\n(c) 2000, 2001 by G. Bartsch and the xine project team.\n", VERSION);
+  printf("This is xine (X11 gui) - a free video player v%s\n(c) 2000-2002 by G. Bartsch and the xine project team.\n", VERSION);
 }
 
 /*
@@ -200,6 +203,8 @@ void show_usage (void) {
   printf(_("                                 'lirc': display draft of a .lircrc config file.\n"));
   printf(_("                                 'remapped': user remapped keymap table.\n"));
   printf(_("                                 -if no option is given, 'default' is selected.\n"));
+  printf(_("  -n, --network                Enable network remote control server.\n"));
+  printf(_("  -R, --root                   Use root window as video window.\n"));
   printf("\n");
   printf(_("examples for valid MRLs (media resource locator):\n"));
   printf(_("  File:  'path/foo.vob'\n"));
@@ -245,8 +250,8 @@ static void load_video_out_driver(char *video_driver_id) {
 #endif
   }
 
-  vis.calc_dest_size    = video_window_calc_dest_size;
-  vis.request_dest_size = video_window_adapt_size;
+  vis.dest_size_cb      = video_window_dest_size_cb;
+  vis.frame_output_cb   = video_window_frame_output_cb;
   vis.user_data         = NULL;
   
   /*
@@ -257,7 +262,7 @@ static void load_video_out_driver(char *video_driver_id) {
 						  NULL, NULL, NULL);
   if (!video_driver_id) {
     /* video output driver auto-probing */
-    char **driver_ids = xine_list_video_output_plugins (VISUAL_TYPE_X11);
+    char **driver_ids;
     int    i;
     
     /* Try to init video with stored information */
@@ -270,14 +275,14 @@ static void load_video_out_driver(char *video_driver_id) {
 						      VISUAL_TYPE_X11,
 						      (void *) &vis);
       if (gGui->vo_driver) {
-	if(driver_ids)
-	  free(driver_ids);
 	gGui->config->update_string (gGui->config, "video.driver", video_driver_id);
 	return;
       } 
     }
     
     i = 0;
+    driver_ids = xine_list_video_output_plugins (VISUAL_TYPE_X11);
+
     while (driver_ids[i]) {
       video_driver_id = driver_ids[i];
       
@@ -416,14 +421,15 @@ void event_listener (void *user_data, xine_event_t *event) {
 
   case XINE_EVENT_UI_SET_TITLE: 
     {
-
       xine_ui_event_t *uevent = (xine_ui_event_t *)event;
       panel_set_title((char*)(uevent->data));
     }
     break;
+
   case XINE_EVENT_PLAYBACK_FINISHED:
     gui_status_callback (XINE_STOP);
     break;
+
   case XINE_EVENT_NEED_NEXT_MRL:
     {
       xine_next_mrl_event_t *uevent = (xine_next_mrl_event_t *)event;
@@ -432,6 +438,7 @@ void event_listener (void *user_data, xine_event_t *event) {
       uevent->mrl = gui_next_mrl_callback ();
     }
     break;
+
   case XINE_EVENT_BRANCHED:
     gui_branched_callback ();
     break;
@@ -457,8 +464,8 @@ int main(int argc, char *argv[]) {
   sigset_t         vo_mask;
 
   /* Check xine library version */
-  if(!xine_check_version(0, 9, 4)) {
-    fprintf(stderr, _("Require xine library version 0.9.4, found %d.%d.%d.\n"),
+  if(!xine_check_version(0, 9, 9)) {
+    fprintf(stderr, _("Require xine library version 0.9.9, found %d.%d.%d.\n"),
 	    xine_get_major_version(), xine_get_minor_version(),
 	    xine_get_sub_version());
     exit(1);
@@ -481,16 +488,18 @@ int main(int argc, char *argv[]) {
   
   gGui = (gGui_t *) xine_xmalloc(sizeof(gGui_t));
 
-  gGui->debug_level = 0;
-  gGui->autoscan_plugin = NULL;
-  gGui->prefered_visual_class = -1;
-  gGui->prefered_visual_id = None;
-  gGui->install_colormap = 0;
-  gGui->cursor_grabbed = 0;
+  gGui->debug_level            = 0;
+  gGui->autoscan_plugin        = NULL;
+  gGui->prefered_visual_class  = -1;
+  gGui->prefered_visual_id     = None;
+  gGui->install_colormap       = 0;
+  gGui->cursor_grabbed         = 0;
+  gGui->network                = 0;
+  gGui->use_root_window        = 0;
 #ifdef HAVE_XF86VIDMODE
   gGui->XF86VidMode_fullscreen = 0;
 #endif
-  gGui->actions_on_start[aos] = ACTID_NOKEY;
+  gGui->actions_on_start[aos]  = ACTID_NOKEY;
 
   /*
    * initialize CORBA server
@@ -625,6 +634,14 @@ int main(int argc, char *argv[]) {
       exit(1);
       break;
 
+    case 'n': /* Enable remote control server */
+      gGui->network = 1;
+      break;
+      
+    case 'R': /* Use root window for video output */
+      gGui->use_root_window = 1;
+      break;
+
     case 'v': /* Display version and exit*/
       show_version();
       exit(1);
@@ -652,15 +669,20 @@ int main(int argc, char *argv[]) {
    */
   {
     char *cfgfile = ".xine/config";
+    struct stat st;
 
     if (!(gGui->configfile = getenv ("XINERC"))) {
       gGui->configfile = (char *) xine_xmalloc((strlen((xine_get_homedir())) + strlen(cfgfile))+2);
       sprintf (gGui->configfile, "%s/%s", (xine_get_homedir()), cfgfile);
     }
-      
+
+    /* Popup setup window if there is no config file */
+    if(stat(gGui->configfile, &st) < 0)
+      gGui->actions_on_start[aos++] = ACTID_SETUP;
+
   }
 
-  gGui->config = config_file_init (gGui->configfile);
+  gGui->config = xine_config_file_init (gGui->configfile);
 
   /*
    * init gui
@@ -674,6 +696,7 @@ int main(int argc, char *argv[]) {
 
   /* Video out plugin */
   load_video_out_driver(video_driver_id);
+  video_window_select_visual ();
 
   /* Audio out plugin */
   audio_driver = load_audio_out_driver(audio_driver_id);
