@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mrl_browser.c,v 1.16 2001/12/15 13:20:08 f1rmb Exp $
+ * $Id: mrl_browser.c,v 1.22 2002/03/22 15:10:03 guenter Exp $
  *
  * MRL Browser
  *
@@ -37,20 +37,44 @@
 
 #include "event.h"
 #include "actions.h"
+#include "videowin.h"
 #include "mrl_browser.h"
+#include "errors.h"
 
 #define MAX_LIST 9
 
-extern gGui_t       *gGui;
+extern gGui_t            *gGui;
 static xitk_widget_t     *mrlb = NULL;
 
+/*
+ *
+ */
+void mrl_browser_show_tips(int enabled, unsigned long timeout) {
+  
+  if(mrlb)
+    xitk_mrlbrowser_set_tips_timeout(mrlb, enabled, timeout);
+
+}
+
+/*
+ *
+ */
+void set_mrl_browser_transient(void) {
+
+  if(mrlb) {
+    xitk_mrlbrowser_set_transient(mrlb, gGui->video_window);
+  }
+}
+
 /*
  *
  */
 void mrl_browser_change_skins(void) {
 
-  if(mrlb)
+  if(mrlb) {
     xitk_mrlbrowser_change_skins(mrlb, gGui->skin_config);
+    set_mrl_browser_transient();
+  }
 }
 
 /*
@@ -75,16 +99,6 @@ int mrl_browser_is_running(void) {
   return 0;
 }
 
-/*
- *
- */
-void set_mrl_browser_transient(void) {
-
-  if(mrlb) {
-    xitk_mrlbrowser_set_transient(mrlb, gGui->video_window);
-  }
-}
-
 /*
  *
  */
@@ -110,10 +124,10 @@ void hide_mrl_browser(void) {
 /*
  *
  */
-void mrl_browser_toggle_visibility(void) {
-
+void mrl_browser_toggle_visibility(xitk_widget_t *w, void *data) {
+  
   if(mrlb) {
-
+    
     if(mrl_browser_is_visible())
       hide_mrl_browser();
     else
@@ -240,9 +254,6 @@ void mrl_browser(xitk_mrl_callback_t add_cb, xitk_mrl_callback_t add_and_play_cb
  *
  */
 static void mrl_handle_selection(xitk_widget_t *w, void *data) {
-#if 0
-  perr(" +++ Selection called = %d = '%s'\n", 
-#endif
 }
 
 /*
