@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: panel.c,v 1.68 2002/01/15 22:22:35 f1rmb Exp $
+ * $Id: panel.c,v 1.85 2002/04/27 13:26:38 f1rmb Exp $
  *
  * xine panel related stuff
  *
@@ -32,6 +32,7 @@
 #include <pthread.h>
 
 #include <xine/xineutils.h>
+#include <zlib.h>
 
 #include "xitk.h"
 
@@ -43,8 +44,13 @@
 #include "panel.h"
 #include "mrl_browser.h"
 #include "snapshot.h"
+#include "errors.h"
 #include "utils.h"
 
+#ifndef __GNUC__
+#define __FUNCTION__    __func__
+#endif
+
 extern gGui_t     *gGui;
 
 _panel_t          *panel;
@@ -58,6 +64,10 @@ void panel_show_tips(void) {
     xitk_set_widgets_tips_timeout(panel->widget_list, panel->tips.timeout);
   else
     xitk_disable_widgets_tips(panel->widget_list);
+
+  playlist_show_tips(panel->tips.enable, panel->tips.timeout);
+  control_show_tips(panel->tips.enable, panel->tips.timeout);
+  mrl_browser_show_tips(panel->tips.enable, panel->tips.timeout);
 }
 
 /*
@@ -109,32 +119,52 @@ static void panel_store_new_position(int x, int y, int w, int h) {
  * Change the current skin.
  */
 void panel_change_skins(void) {
+  ImlibImage   *new_img, *old_img;
+  XSizeHints    hint;
+  
+  xitk_skin_lock(gGui->skin_config);
+  xitk_hide_widgets(panel->widget_list);
 
   XLockDisplay(gGui->display);
-
-  Imlib_destroy_image(gGui->imlib_data, panel->bg_image);
-    
-  if(!(panel->bg_image = 
-       Imlib_load_image(gGui->imlib_data,
-			xitk_skin_get_skin_filename(gGui->skin_config, "BackGround")))) {
+  
+  if(!(new_img = Imlib_load_image(gGui->imlib_data,
+				  xitk_skin_get_skin_filename(gGui->skin_config, "BackGround")))) {
     xine_error("%s(): couldn't find image for background\n", __FUNCTION__);
     exit(-1);
   }
-
+  
+  hint.width  = new_img->rgb_width;
+  hint.height = new_img->rgb_height;
+  hint.flags  = PSize;
+  XSetWMNormalHints(gGui->display, gGui->panel_window, &hint);
+  
   XResizeWindow (gGui->display, gGui->panel_window,
-		 (unsigned int)panel->bg_image->rgb_width,
-		 (unsigned int)panel->bg_image->rgb_height);
+		 (unsigned int)new_img->rgb_width,
+		 (unsigned int)new_img->rgb_height);
+  
+  XUnlockDisplay(gGui->display);
 
-  /*
-   * We should here, otherwise new skined window will have wrong size.
-   */
-  XFlush(gGui->display);
+  while(!xitk_is_window_size(gGui->display, gGui->panel_window, 
+			     new_img->rgb_width, new_img->rgb_height)) {
+    xine_usec_sleep(10000);
+  }
+  
+  old_img = panel->bg_image;
+  panel->bg_image = new_img;
 
-  Imlib_apply_image(gGui->imlib_data, panel->bg_image, gGui->panel_window);
+  XLockDisplay(gGui->display);
+
+  XSetTransientForHint(gGui->display, gGui->panel_window, gGui->video_window);
+
+  Imlib_destroy_image(gGui->imlib_data, old_img);
+  Imlib_apply_image(gGui->imlib_data, new_img, gGui->panel_window);
   
   XUnlockDisplay(gGui->display);
   
+  xitk_skin_unlock(gGui->skin_config);
+  
   xitk_change_skins_widget_list(panel->widget_list, gGui->skin_config);
+
   /*
    * Update position of dynamic buttons.
    */
@@ -154,7 +184,6 @@ void panel_change_skins(void) {
   }
 
   xitk_paint_widget_list(panel->widget_list);
-
 }
 
 /*
@@ -198,6 +227,7 @@ static void *slider_loop(void *dummy) {
           gGui->mixer.volume_level = xine_get_audio_property(gGui->xine, 
                                                              gGui->mixer.volume_mixer);
           xitk_slider_set_pos(panel->widget_list, panel->mixer.slider, gGui->mixer.volume_level);
+	  panel_check_mute();
         }
 	
 	if(status != XINE_STOP)
@@ -240,12 +270,12 @@ void panel_toggle_visibility (xitk_widget_t *w, void *data) {
     
   if(!panel->visible && control_is_visible()) {}
   else {
-    control_toggle_panel_visibility(NULL, NULL);
+    control_toggle_visibility(NULL, NULL);
   }
 
   if(!panel->visible && mrl_browser_is_visible()) {}
   else {
-    mrl_browser_toggle_visibility();
+    mrl_browser_toggle_visibility(NULL, NULL);
   }
 
 
@@ -254,9 +284,14 @@ void panel_toggle_visibility (xitk_widget_t *w, void *data) {
     XLockDisplay(gGui->display);
     
     if (video_window_is_visible ()) {
-      panel->visible = 0;
-      XUnmapWindow (gGui->display, gGui->panel_window);
-      xitk_hide_widgets(panel->widget_list);
+      if(gGui->use_root_window) { /* Using root window */
+	XIconifyWindow(gGui->display, gGui->panel_window, gGui->screen);
+      }
+      else {
+	panel->visible = 0;
+	XUnmapWindow (gGui->display, gGui->panel_window);
+	xitk_hide_widgets(panel->widget_list);
+      }
     }
 
     if(gGui->cursor_grabbed)
@@ -369,7 +404,7 @@ void panel_toggle_audio_mute(xitk_widget_t *w, void *data, int state) {
  *  to snapshot current frame.
  */
 void panel_snapshot(xitk_widget_t *w, void *data) {
-  create_snapshot(gGui);
+  create_snapshot();
 }
 
 /*
@@ -442,7 +477,10 @@ void panel_add_autoplay_buttons(void) {
 	      xitk_labelbutton_create (gGui->skin_config, &lb)));
     xitk_set_widget_tips(panel->autoplay_plugins[i], 
 			 xine_get_input_plugin_description(gGui->xine, autoplay_plugins[i]));
-    
+
+    if(!panel->tips.enable)
+      xitk_disable_widget_tips(panel->autoplay_plugins[i]);
+
     (void) xitk_set_widget_pos(panel->autoplay_plugins[i], x, y);
 
     x -= xitk_get_widget_width(panel->autoplay_plugins[i]) + 1;
@@ -487,6 +525,11 @@ void panel_add_mixer_control(void) {
   if(gGui->mixer.caps & AO_CAP_MUTE_VOL)
     xitk_set_widget_tips(panel->mixer.mute, _("Mute toggle"));
 
+  if(!panel->tips.enable) {
+    xitk_disable_widget_tips(panel->mixer.slider);
+    xitk_disable_widget_tips(panel->mixer.mute);
+  }
+
 }
 
 /*
@@ -564,17 +607,16 @@ void panel_init (void) {
 				      hint.x, hint.y,
 				      hint.width, hint.height, 0, 
 				      gGui->imlib_data->x.depth,
-				      CopyFromParent,
+				      InputOutput,
 				      gGui->imlib_data->x.visual,
 				      CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect,
 				      &attr);
-  
+
   XSetStandardProperties(gGui->display, gGui->panel_window, title, title,
 			 None, NULL, 0, &hint);
 
-  XSelectInput(gGui->display, gGui->panel_window,
-	       ButtonPressMask | ButtonReleaseMask | PointerMotionMask
-	       | KeyPressMask | KeymapStateMask | ExposureMask | StructureNotifyMask);
+  XSelectInput(gGui->display, gGui->panel_window, INPUT_MOTION | 
+	       /*SubstructureRedirectMask | */ KeymapStateMask);
 
   /*
    * wm, no border please
@@ -781,6 +823,7 @@ void panel_init (void) {
   sl.motion_userdata   = NULL;
   xitk_list_append_content (panel->widget_list->l, 
 			   (panel->slider_play = xitk_slider_create(gGui->skin_config, &sl)));
+  xitk_set_widget_tips(panel->slider_play, _("Stream playback position slider"));
 
   /* Mixer volume slider */
   sl.skin_element_name = "SliderVol";
@@ -862,9 +905,14 @@ void panel_init (void) {
 						"gui panel visibility",
 						NULL, NULL, NULL);
   
+  if(gGui->use_root_window && (!panel->visible))
+    panel->visible = 1;
+
   if (panel->visible)
     XMapRaised(gGui->display, gGui->panel_window); 
   
+  XUnlockDisplay (gGui->display);
+
   panel->widget_key = xitk_register_event_handler("panel", 
 						  gGui->panel_window, 
 						  panel_handle_event,
@@ -876,8 +924,6 @@ void panel_init (void) {
   gGui->cursor_visible = 1;
   video_window_set_cursor_visibility(gGui->cursor_visible);
   
-  XUnlockDisplay (gGui->display);
-
   /*
    * Grag Atoms from a mapped window, panel window could be not already mapped.
    */
@@ -899,5 +945,5 @@ void panel_init (void) {
 }
 
 void panel_set_title(char *title) {
-  xitk_label_change_label (panel->widget_list, panel->title_label, title);
+  xitk_label_change_label(panel->widget_list, panel->title_label, title);
 }
