@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: playlist.c,v 1.38 2002/01/15 22:22:35 f1rmb Exp $
+ * $Id: playlist.c,v 1.49 2002/04/21 22:17:12 f1rmb Exp $
  *
  * Playlist Editor
  *
@@ -38,10 +38,15 @@
 #include "event.h"
 #include "actions.h"
 #include "mrl_browser.h"
+#include "errors.h"
 
 #include <xine.h>
 #include <xine/xineutils.h>
 
+#ifndef __GNUC__
+#define __FUNCTION__    __func__
+#endif
+
 #define MAX_LIST 9
 
 extern gGui_t          *gGui;
@@ -69,6 +74,19 @@ static _playlist_t   *playlist;
 
 void playlist_handle_event(XEvent *event, void *data);
 
+/*
+ *
+ */
+void playlist_show_tips(int enabled, unsigned long timeout) {
+  
+  if(playlist) {
+    if(enabled)
+      xitk_set_widgets_tips_timeout(playlist->widget_list, timeout);
+    else
+      xitk_disable_widgets_tips(playlist->widget_list);
+  }
+}
+
 /*
  *
  */
@@ -576,33 +594,52 @@ static void pl_add_input(xitk_widget_t *w, void *data, char *filename) {
  * Change the current skin.
  */
 void playlist_change_skins(void) {
+  ImlibImage   *new_img, *old_img;
+  XSizeHints    hint;
 
   if(pl_is_running()) {
     
+    xitk_skin_lock(gGui->skin_config);
+    xitk_hide_widgets(playlist->widget_list);
+
     XLockDisplay(gGui->display);
     
-    Imlib_destroy_image(gGui->imlib_data, playlist->bg_image);
-    
-    if(!(playlist->bg_image = 
-	 Imlib_load_image(gGui->imlib_data,
-			  xitk_skin_get_skin_filename(gGui->skin_config, "PlBG")))) {
+    if(!(new_img = Imlib_load_image(gGui->imlib_data,
+				    xitk_skin_get_skin_filename(gGui->skin_config, "PlBG")))) {
       xine_error("%s(): couldn't find image for background\n", __FUNCTION__);
       exit(-1);
     }
     
+    hint.width  = new_img->rgb_width;
+    hint.height = new_img->rgb_height;
+    hint.flags  = PSize;
+    XSetWMNormalHints(gGui->display, playlist->window, &hint);
+
     XResizeWindow (gGui->display, playlist->window,
-		   (unsigned int)playlist->bg_image->rgb_width,
-		   (unsigned int)playlist->bg_image->rgb_height);
-    
-    /*
-     * We should here, otherwise new skined window will have wrong size.
-     */
-    XFlush(gGui->display);
-    
-    Imlib_apply_image(gGui->imlib_data, playlist->bg_image, playlist->window);
+		   (unsigned int)new_img->rgb_width,
+		   (unsigned int)new_img->rgb_height);
     
     XUnlockDisplay(gGui->display);
     
+    while(!xitk_is_window_size(gGui->display, playlist->window, 
+			       new_img->rgb_width, new_img->rgb_height)) {
+      xine_usec_sleep(10000);
+    }
+    
+    old_img = playlist->bg_image;
+    playlist->bg_image = new_img;
+
+    XLockDisplay(gGui->display);
+    
+    XSetTransientForHint(gGui->display, playlist->window, gGui->video_window);
+    
+    Imlib_destroy_image(gGui->imlib_data, old_img);
+    Imlib_apply_image(gGui->imlib_data, new_img, playlist->window);
+    
+    XUnlockDisplay(gGui->display);
+    
+    xitk_skin_unlock(gGui->skin_config);
+    
     xitk_change_skins_widget_list(playlist->widget_list, gGui->skin_config);
     
     {
@@ -637,11 +674,12 @@ void playlist_editor(void) {
   Atom                       prop, XA_WIN_LAYER;
   MWMHints                   mwmhints;
   XClassHint                *xclasshint;
-  xitk_browser_widget_t       br;
-  xitk_labelbutton_widget_t   lb;
-  xitk_label_widget_t         lbl;
-  xitk_inputtext_widget_t     inp;
-  xitk_button_widget_t        b;
+  xitk_browser_widget_t      br;
+  xitk_labelbutton_widget_t  lb;
+  xitk_label_widget_t        lbl;
+  xitk_inputtext_widget_t    inp;
+  xitk_button_widget_t       b;
+  xitk_widget_t             *w;
   long data[1];
 
   /* This shouldn't be happend */
@@ -692,7 +730,7 @@ void playlist_editor(void) {
 				    gGui->imlib_data->x.root, 
 				    hint.x, hint.y, hint.width, 
 				    hint.height, 0, 
-				    gGui->imlib_data->x.depth, CopyFromParent, 
+				    gGui->imlib_data->x.depth, InputOutput, 
 				    gGui->imlib_data->x.visual,
 				    CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect,
 				    &attr);
@@ -700,9 +738,7 @@ void playlist_editor(void) {
   XSetStandardProperties(gGui->display, playlist->window, title, title,
 			 None, NULL, 0, &hint);
   
-  XSelectInput(gGui->display, playlist->window, 
-	       ButtonPressMask | ButtonReleaseMask | KeymapStateMask | PointerMotionMask
-	       | KeyPressMask | ExposureMask | StructureNotifyMask);
+  XSelectInput(gGui->display, playlist->window, INPUT_MOTION | KeymapStateMask);
   
   /*
    * layer above most other things, like gnome panel
@@ -770,31 +806,37 @@ void playlist_editor(void) {
   b.callback          = pl_move_updown;
   b.userdata          = (void *)MOVEUP;
   xitk_list_append_content (playlist->widget_list->l, 
-			    xitk_button_create (gGui->skin_config, &b));
+			    (w = xitk_button_create (gGui->skin_config, &b)));
+  xitk_set_widget_tips(w, _("Move up selected MRL"));
+
 
   b.skin_element_name = "PlMoveDn";
   b.callback          = pl_move_updown;
   b.userdata          = (void *)MOVEDN;
   xitk_list_append_content (playlist->widget_list->l, 
-			   xitk_button_create (gGui->skin_config, &b));
+			    (w = xitk_button_create (gGui->skin_config, &b)));
+  xitk_set_widget_tips(w, _("Move down selected MRL"));
 
   b.skin_element_name = "PlPlay";
   b.callback          = pl_play;
   b.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l, 
-			   xitk_button_create (gGui->skin_config, &b));
-
+			    (w = xitk_button_create (gGui->skin_config, &b)));
+  xitk_set_widget_tips(w, _("Start playback of selected MRL"));
+  
   b.skin_element_name = "PlDelete";
   b.callback          = pl_delete;
   b.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l, 
-			   xitk_button_create (gGui->skin_config, &b));
+			    (w = xitk_button_create (gGui->skin_config, &b)));
+  xitk_set_widget_tips(w, _("Delete selected MRL from playlist"));
 
   b.skin_element_name = "PlDeleteAll";
   b.callback          = pl_delete_all;
   b.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l, 
-			   xitk_button_create (gGui->skin_config, &b));
+			    (w = xitk_button_create (gGui->skin_config, &b)));
+  xitk_set_widget_tips(w, _("Delete all entries in playlist"));
 
   lb.skin_element_name = "PlAdd";
   lb.button_type       = CLICK_BUTTON;
@@ -803,7 +845,9 @@ void playlist_editor(void) {
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l, 
-                           xitk_labelbutton_create (gGui->skin_config, &lb));
+			    (w = xitk_labelbutton_create (gGui->skin_config, &lb)));
+  xitk_set_widget_tips(w, _("Add one or more entries in playlist"));
+    
 
   lb.skin_element_name = "PlLoad";
   lb.button_type       = CLICK_BUTTON;
@@ -812,7 +856,8 @@ void playlist_editor(void) {
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l, 
-			   xitk_labelbutton_create (gGui->skin_config, &lb));
+			    (w = xitk_labelbutton_create (gGui->skin_config, &lb)));
+  xitk_set_widget_tips(w, _("Load saved playlist"));
 
   lb.skin_element_name = "PlSave";
   lb.button_type       = CLICK_BUTTON;
@@ -821,7 +866,8 @@ void playlist_editor(void) {
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l, 
-			   xitk_labelbutton_create (gGui->skin_config, &lb));
+			    (w = xitk_labelbutton_create (gGui->skin_config, &lb)));
+  xitk_set_widget_tips(w, _("Save playlist"));
 
   lb.skin_element_name = "PlDismiss";
   lb.button_type       = CLICK_BUTTON;
@@ -830,7 +876,8 @@ void playlist_editor(void) {
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l, 
-			   xitk_labelbutton_create (gGui->skin_config, &lb));
+			    (w = xitk_labelbutton_create (gGui->skin_config, &lb)));
+  xitk_set_widget_tips(w, _("Close playlist window"));
 
   br.arrow_up.skin_element_name    = "PlUp";
   br.slider.skin_element_name      = "SliderPl";
@@ -859,6 +906,7 @@ void playlist_editor(void) {
   inp.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l,
 			   (playlist->winput = xitk_inputtext_create (gGui->skin_config, &inp)));
+  xitk_set_widget_tips(playlist->winput, _("Direct MRL entry"));
 
   {
     int x, y;
@@ -879,9 +927,11 @@ void playlist_editor(void) {
       xitk_list_append_content (playlist->widget_list->l,
 	       (playlist->autoplay_plugins[i] = 
 		xitk_labelbutton_create (gGui->skin_config, &lb)));
-
+      xitk_set_widget_tips(playlist->autoplay_plugins[i], 
+			   xine_get_input_plugin_description(gGui->xine, autoplay_plugins[i]));
+      
       (void) xitk_set_widget_pos(playlist->autoplay_plugins[i], x, y);
-
+      
       y += xitk_get_widget_height(playlist->autoplay_plugins[i]) + 1;
       i++;
     }
@@ -894,6 +944,8 @@ void playlist_editor(void) {
 
   XMapRaised(gGui->display, playlist->window); 
   
+  XUnlockDisplay (gGui->display);
+
   playlist->widget_key = 
     xitk_register_event_handler("playlist", 
 				playlist->window, 
@@ -904,6 +956,4 @@ void playlist_editor(void) {
 
   playlist->visible = 1;
   playlist->running = 1;
-   
-  XUnlockDisplay (gGui->display);
 }
