@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: setup.c,v 1.25 2001/12/21 04:33:28 f1rmb Exp $
+ * $Id: setup.c,v 1.29 2002/04/12 21:38:51 f1rmb Exp $
  *
  *
  */
@@ -320,7 +320,7 @@ void setup_raise_window(void) {
 /*
  * Hide/show the setup panel
  */
-void setup_toggle_panel_visibility (xitk_widget_t *w, void *data) {
+void setup_toggle_visibility (xitk_widget_t *w, void *data) {
   
   if(setup != NULL) {
     if (setup->visible && setup->running) {
@@ -790,7 +790,7 @@ static widget_triplet_t *setup_list_browser(int x, int y, char **content, int le
 /*
  *
  */
-static void setup_section_widgets (int s) {
+static void setup_section_widgets(int s) {
   int                  x = ((WINDOW_WIDTH>>1) - (FRAME_WIDTH>>1) - 10);
   int                  y = 70;
   cfg_entry_t         *entry;
@@ -866,6 +866,7 @@ static void setup_section_widgets (int s) {
       
       entry = entry->next;
     }
+
   }
 }
 
@@ -939,18 +940,39 @@ static void setup_sections (void) {
 	  break;
 	}
       }
-      
-      if (!found) {
 
-	setup->sections[setup->num_sections] = xine_xmalloc (len+1);
-	strncpy (setup->sections[setup->num_sections], entry->key, len);
-	setup->sections[setup->num_sections][len] = 0;
-	setup->num_sections++;
+      /* 
+       * Only add a tab for non-empty sections (we don't need to handle
+       * sinek/gnome-xine/etc... specific entries)
+       */
+      if (!found) {
+	cfg_entry_t *check_entry = gGui->config->first;
+	int          num_entries = 0;
+	
+	while (check_entry) {
+	  
+	  if(!strncmp(check_entry->key, entry->key, len) && check_entry->description) {
+	    switch(check_entry->type) {
+	    default:
+	      num_entries++;
+	      break;
+	    }
+	  }
+	  check_entry = check_entry->next;
+	}
+	
+	if(num_entries) {
+	  setup->sections[setup->num_sections] = xine_xmalloc (len + 1);
+	  strncpy (setup->sections[setup->num_sections], entry->key, len);
+	  setup->sections[setup->num_sections][len] = 0;
+	  setup->num_sections++;
+	}
       }
     }      
-
+    
     entry = entry->next;
   }
+
   setup->sections[setup->num_sections] = strdup(_("Help"));
   setup->num_sections++;
   setup->sections[setup->num_sections] = strdup(_("README"));
@@ -1176,8 +1198,11 @@ void setup_panel(void) {
 					  (WINDOW_WIDTH>>1) - 50, WINDOW_HEIGHT - 40,
 					  100, 23,
 					  "Black", "Black", "White", fontname));
-
   
+  XMapRaised(gGui->display, xitk_window_get_window(setup->xwin));
+
+  XUnlockDisplay (gGui->display);
+
   setup->kreg = xitk_register_event_handler("setup", 
 					   (xitk_window_get_window(setup->xwin)),
 					   setup_handle_event,
@@ -1186,10 +1211,8 @@ void setup_panel(void) {
 					   setup->widget_list,
 					   NULL);
 
-  XMapRaised(gGui->display, xitk_window_get_window(setup->xwin));
 
   setup->visible = 1;
   setup->running = 1;
 
-  XUnlockDisplay (gGui->display);
 }
