@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: snapshot.c,v 1.12 2001/12/16 19:10:23 hrm Exp $
+ * $Id: snapshot.c,v 1.15 2002/04/27 13:26:40 f1rmb Exp $
  *
  * Image snapshot related stuff
  *
@@ -58,15 +58,18 @@
 #endif
 #include <pthread.h>
 
+#ifdef HAVE_PNG_H
+#include <png.h>
+#elif HAVE_LIBPNG_PNG_H
+#include <libpng/png.h>
+#endif
+
+#include <xine.h>
+#include <xine/xineutils.h>
+
 #include "xitk.h"
 #include "event.h"
-#include <xine/xineutils.h>
-
-#include "xine.h"
-#include <xine/xineutils.h>
-
-#include "png.h"
-#include "pngconf.h"
+#include "errors.h"
 
 #define PIXSZ 3
 #define BIT_DEPTH 8
@@ -694,18 +697,17 @@ static void rgb_free( struct prvt_image_s *image )
 static int prvt_image_alloc( struct prvt_image_s **image )
 {
   char *filename;
-
-  *image = (struct prvt_image_s*)malloc( sizeof( struct prvt_image_s ) );
-
-  if (*image == NULL) return( 0 );
-    
-  memset( *image, 0, sizeof( struct prvt_image_s ) );
-
-  filename = (char *) alloca(strlen(xine_get_homedir()) + 10);
-  sprintf(filename, "%s/%s", xine_get_homedir(), "xinesnap");
-
+  
+  *image = (struct prvt_image_s*) xine_xmalloc( sizeof( struct prvt_image_s ) );
+  
+  if (*image == NULL) 
+    return 0;
+  
+  filename = (char *) alloca(strlen(gGui->snapshot_location) + 10);
+  sprintf(filename, "%s/%s", gGui->snapshot_location, "xinesnap");
+  
   (*image)->file_name = snap_filename( filename, "png" );
-
+  
   return( 1 );
 }
 
@@ -828,7 +830,7 @@ static void write_row_callback( png_structp png_ptr, png_uint_32 row, int pass)
  *  External function
  */
 
-void create_snapshot ( gGui_t *gGui )
+void create_snapshot (void)
 {
   int err = 0;
   struct prvt_image_s *image;
