@@ -1,7 +1,7 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: videowin.c,v 1.61 2002/01/03 22:45:01 jcdutton Exp $
+ * $Id: videowin.c,v 1.85 2002/04/27 13:26:42 f1rmb Exp $
  *
  * video window handling functions
 */
@@ -48,6 +48,7 @@
 #include "videowin.h"
 #include "panel.h"
 #include "actions.h"
+#include "errors.h"
 
 extern gGui_t *gGui;
 
@@ -58,9 +59,17 @@ typedef struct {
   Visual	*visual;	  /* Visual for video window               */
   Colormap	 colormap;	  /* Colormap for video window		   */
   XClassHint    *xclasshint;
+  XClassHint    *xclasshint_fullscreen;
   GC             gc;
+
   int            video_width;     /* size of currently displayed video     */
   int            video_height;
+  int            win_width;       /* size of non-fullscreen window         */
+  int            win_height;
+  int            output_width;    /* output video window width/height      */
+  int            output_height;
+  float          mag;
+
   int            fullscreen_mode; /* are we currently in fullscreen mode?  */
   int            fullscreen_req;  /* ==1 => video_window will 
 				   * switch to fullscreen mode             */
@@ -89,6 +98,7 @@ typedef struct {
   XF86VidModeModeInfo** XF86_modelines;
   int                   XF86_modelines_count;
 #endif
+
 } gVw_t;
 
 static gVw_t    *gVw;
@@ -98,196 +108,45 @@ extern int XShmGetEventBase(Display *);
 #endif
 
 static void video_window_handle_event (XEvent *event, void *data);
+static void video_window_adapt_size (void);
 
 /*
- * Will called by toolkit on every move/resize event.
+ * Let the video driver override the selected visual
  */
-void video_window_change_sizepos(int x, int y, int w, int h) {
-  gVw->xwin = x;
-  gVw->ywin = y;
-  /*  printf("video window change: %d %d %d %d\n", x, y, w, h); */
-}
+void video_window_select_visual (void) {
+  XVisualInfo *vinfo = (XVisualInfo *) -1;
 
-/*
- *
- */
-void video_window_draw_logo(void) {
-  ImlibImage *resized_image;
-  int xwin, ywin, tmp;
-  unsigned int wwin, hwin, bwin, dwin;
-  double ratio = 1;
-  Window rootwin;
-  
-  if(video_window_is_visible()) {
-    
-    XLockDisplay (gGui->display);
-    
-    /* XClearWindow (gGui->display, gGui->video_window);  */
-    
-    if(XGetGeometry(gGui->display, gGui->video_window, &rootwin, 
-		    &xwin, &ywin, &wwin, &hwin, &bwin, &dwin) != BadDrawable) {
-      
-      tmp = (wwin / 100) * 86;
-      ratio = (tmp < gGui->video_window_logo_pixmap.width && tmp >= 1) ? 
-	((double)tmp / (double)gGui->video_window_logo_pixmap.width):1;
-    }
-    
-    resized_image = Imlib_clone_image(gGui->imlib_data, 
-				      gGui->video_window_logo_image);
-    Imlib_render (gGui->imlib_data, resized_image, 
-		  (int)gGui->video_window_logo_pixmap.width * ratio, 
-		  (int)gGui->video_window_logo_pixmap.height * ratio);
-    
-    XCopyArea (gGui->display, resized_image->pixmap, gGui->video_window, 
-	       gVw->gc, 0, 0,
-	       resized_image->width, resized_image->height, 
-	       (wwin - resized_image->width) / 2, 
-	       (hwin - resized_image->height) / 2);
-    
-    
-    XFlush(gGui->display);
-    
-    Imlib_destroy_image(gGui->imlib_data, resized_image);
-    
-    XUnlockDisplay (gGui->display);
-  }
-}
-
-/*
- * Hide the logo in video output window.
- */
-void video_window_hide_logo(void) {
-
-  if(video_window_is_visible()) {
-    XLockDisplay (gGui->display);
-#if 0
-    XUnmapWindow (gGui->display, gGui->video_window);
-    XMapWindow (gGui->display, gGui->video_window);
-#else
-    /*
-     * The Unmap/Map trick(?) has the undesirable side-effect, that
-     * all of the video_windows's transient windows are unmapped, too
-     * - messing up their "visible" state.
-     *
-     * Remove the logo by clearing the window.
-     */
-    XClearWindow (gGui->display, gGui->video_window);
-
-    /* Force ConfigureNotify event */
-    /* 
-     *   no longer needed, video_out_xv repaints it's colorkey now on
-     * GUI_DATA_EX_LOGO_VISIBILITY 
-     *
-    {
-      Window        rootwin;
-      int           xwin, ywin;
-      unsigned int  wwin, hwin, bwin, dwin;
-      
-      if(XGetGeometry(gGui->display, gGui->video_window, &rootwin, 
-		      &xwin, &ywin, &wwin, &hwin, &bwin, &dwin) != BadDrawable) {
-	
-	XMoveResizeWindow (gGui->display, gGui->video_window, xwin, ywin, wwin-1, hwin-1);
-	XMoveResizeWindow (gGui->display, gGui->video_window, xwin, ywin, wwin, hwin);
+  XLockDisplay (gGui->display);
+  if (gGui->vo_driver) {
+    gGui->vo_driver->gui_data_exchange (gGui->vo_driver, GUI_SELECT_VISUAL,
+                                        &vinfo);
+    if (vinfo != (XVisualInfo *) -1) {
+      if (! vinfo) {
+        fprintf (stderr, "videowin: output driver cannot select a working visual\n");
+        exit (1);
       }
+      gGui->visual = vinfo->visual;
+      gGui->depth  = vinfo->depth;
     }
-    */
-    
-    gGui->vo_driver->gui_data_exchange(gGui->vo_driver, GUI_DATA_EX_LOGO_VISIBILITY, (int *)0);
-#endif
-    XUnlockDisplay (gGui->display);
-  }
-
-/* 2001-10-07 ehasenle: activated this code for dxr3 overlay */
-    gGui->vo_driver->gui_data_exchange (gGui->vo_driver,
-    GUI_DATA_EX_DRAWABLE_CHANGED, 
-    (void*)gGui->video_window);
-}
-
-/*
- * Show the logo in video output window.
- */
-void video_window_show_logo(void) {
-   
-  if(video_window_is_visible()) {
-    gGui->vo_driver->gui_data_exchange (gGui->vo_driver, GUI_DATA_EX_LOGO_VISIBILITY, (int *)1);
-     
-    XLockDisplay (gGui->display);
-    XClearWindow (gGui->display, gGui->video_window); 
-    video_window_draw_logo();
-    XUnlockDisplay (gGui->display);
-  }
-}
-
-/*
- *
- */
-void video_window_set_fullscreen (int req_fullscreen) {
-
-  x11_rectangle_t area;
-
-  gVw->fullscreen_req = req_fullscreen;
-
-  video_window_adapt_size (NULL, gVw->video_width, gVw->video_height, 
-			   &area.x, &area.y, &area.w, &area.h);
-
-  gGui->vo_driver->gui_data_exchange (gGui->vo_driver, 
-				      GUI_DATA_EX_DEST_POS_SIZE_CHANGED, 
-				      &area);
-
-  /* Tell the video driver if we are in fullscreen mode or not */
-  gGui->vo_driver->gui_data_exchange(gGui->vo_driver, GUI_DATA_EX_FULLSCREEN, (int *)gVw->fullscreen_mode);
-}
-
-/*
- *
- */
-int video_window_is_fullscreen (void) {
-  return gVw->fullscreen_mode;
-}
-
-/*
- *
- */
-void video_window_calc_dest_size (void *this,
-				  int video_width, int video_height,
-				  int *dest_width, int *dest_height)  {
-
-  if (gVw->fullscreen_mode) {
-
-#ifdef HAVE_XINERAMA
-    if (gVw->xinerama) {
-        int i;
-        for (i = 0; i < gVw->xinerama_cnt; i++) {
-            if (gVw->xinerama[i].screen_number == XScreenNumberOfScreen(XDefaultScreenOfDisplay(gGui->display))) {
-                *dest_width = gVw->xinerama[i].width;
-                *dest_height = gVw->xinerama[i].height;
-                break;
-            }
-        }
-    } else {
-        *dest_width  = gVw->fullscreen_width;
-        *dest_height = gVw->fullscreen_height;
+    if (gGui->visual != gVw->visual) {
+      printf ("videowin: output driver overrides selected visual to visual id 0x%lx\n", gGui->visual->visualid);
+      gui_init_imlib (gGui->visual);
+      video_window_adapt_size ();
     }
-#else
-    *dest_width  = gVw->fullscreen_width;
-    *dest_height = gVw->fullscreen_height;
-#endif
-    
-  } else {
-
-    *dest_width  = video_width;
-    *dest_height = video_height;
-
   }
+  XUnlockDisplay (gGui->display);
 }
 
 /*
+ * will modify/create video output window based on
  *
+ * - fullscreen flags
+ * - win_width/win_height
+ *
+ * will set
+ * output_width/output_height
  */
-void video_window_adapt_size (void *this,
-			      int video_width, int video_height, 
-			      int *dest_x, int *dest_y,
-			      int *dest_width, int *dest_height) {
+static void video_window_adapt_size (void) { 
 
   static char          *window_title = "xine video output";
   XSizeHints            hint;
@@ -300,7 +159,7 @@ void video_window_adapt_size (void *this,
   XEvent                xev;
   XGCValues             xgcv;
   Window                old_video_window = None;
-  long data[1];
+  long                  propvalue[1];
 
 /*  printf("window_adapt:vw=%d, vh=%d, dx=%d, dy=%d, dw=%d, dh=%d\n",
  *           video_width,
@@ -313,10 +172,77 @@ void video_window_adapt_size (void *this,
 
   XLockDisplay (gGui->display);
 
-  gVw->video_width = video_width;
-  gVw->video_height = video_height;
-  *dest_x = 0;
-  *dest_y = 0;
+  if(gGui->use_root_window) { /* Using root window, but not really */
+
+    gVw->output_width    = gVw->fullscreen_width;
+    gVw->output_height   = gVw->fullscreen_height;
+      
+    if(gGui->video_window == None) {
+      XGCValues   gcv;
+      
+      gVw->fullscreen_mode = 1;
+      gVw->visual          = gGui->visual;
+      gVw->depth           = gGui->depth;
+      gVw->colormap        = gGui->colormap;
+      
+
+      attr.override_redirect = True;
+      attr.background_pixel  = gGui->black.pixel;
+      
+      gGui->video_window = XCreateWindow(gGui->display, DefaultRootWindow(gGui->display),
+					 0, 0, gVw->fullscreen_width, gVw->fullscreen_height, 
+					 0, CopyFromParent, CopyFromParent, CopyFromParent, 
+					 CWBackPixel | CWOverrideRedirect, &attr);
+/* fprintf (stderr, "***** XCreateWindow 1 visual %p id 0x%x depth %d\n", gVw->visual, gVw->visual->visualid, gVw->depth); */
+      
+      if(gGui->vo_driver)
+	gGui->vo_driver->gui_data_exchange (gGui->vo_driver,
+					    GUI_DATA_EX_DRAWABLE_CHANGED, 
+					    (void*)gGui->video_window);
+      
+      XSetStandardProperties(gGui->display, gGui->video_window, 
+			     window_title, window_title, None, NULL, 0, 0);
+      
+
+      gcv.foreground         = gGui->black.pixel;
+      gcv.background         = gGui->black.pixel;
+      gcv.graphics_exposures = False;
+      gVw->gc = XCreateGC(gGui->display, gGui->video_window, 
+			  GCForeground | GCBackground | GCGraphicsExposures, &gcv);
+      
+      XSetStandardProperties(gGui->display, gGui->video_window, 
+			     window_title, window_title, None, NULL, 0, 0);
+      
+      hint.flags  = USSize | USPosition | PPosition | PSize;
+      hint.x      = 0;
+      hint.y      = 0;
+      hint.width  = gVw->fullscreen_width;
+      hint.height = gVw->fullscreen_height;
+      XSetNormalHints(gGui->display, gGui->video_window, &hint);
+      
+      XClearWindow(gGui->display, gGui->video_window);
+      
+      XMapWindow(gGui->display, gGui->video_window);
+      
+      XLowerWindow(gGui->display, gGui->video_window);
+      
+      gVw->old_widget_key = gVw->widget_key;
+
+      XUnlockDisplay (gGui->display);
+
+      gVw->widget_key = xitk_register_event_handler("video_window", 
+						    gGui->video_window, 
+						    video_window_handle_event,
+						    NULL,
+						    gui_dndcallback,
+						    NULL, NULL);
+      return;
+    }
+    
+    XUnlockDisplay (gGui->display);
+    
+    return;
+  }
 
 #ifdef HAVE_XF86VIDMODE
   /* XF86VidMode Extension
@@ -350,7 +276,7 @@ void video_window_adapt_size (void *this,
 	  gVw->fullscreen_height = gVw->XF86_modelines[search]->vdisplay;
 	  
 	  /*
-	   *just in case the mouse pointer is off the visible area, move it
+	   * just in case the mouse pointer is off the visible area, move it
 	   * to the middle of the video window
 	   */
 	  XWarpPointer(gGui->display, None, gGui->video_window, 0, 0, 0, 0, gVw->fullscreen_width/2, gVw->fullscreen_height/2);
@@ -369,74 +295,65 @@ void video_window_adapt_size (void *this,
 #endif
 
 #ifdef HAVE_XINERAMA
-    if (gVw->xinerama) {
-        int i;
-        int knowLocation = 0;
-
-        /* someday this could also use the centre of the window as the
-         * test point I guess.  Right now it's the upper-left.
-         */
-        if (gGui->video_window != None) {
-            if (gVw->xwin >= 0 && gVw->ywin >= 0 &&
-              gVw->xwin < gVw->desktopWidth && gVw->ywin < gVw->desktopHeight) {
-                knowLocation = 1;
-            }
-        }
-
-        for (i = 0; i < gVw->xinerama_cnt; i++) {
-            if (
-                (knowLocation == 1 &&
-                 gVw->xwin >= gVw->xinerama[i].x_org &&
-                 gVw->ywin >= gVw->xinerama[i].y_org &&
-                 gVw->xwin <= gVw->xinerama[i].x_org+gVw->xinerama[i].width &&
-                 gVw->ywin <= gVw->xinerama[i].y_org+gVw->xinerama[i].height) ||
-                (knowLocation == 0 &&
-                 gVw->xinerama[i].screen_number == 
-                   XScreenNumberOfScreen(XDefaultScreenOfDisplay(gGui->display)))) {
-                hint.x = gVw->xinerama[i].x_org;
-                hint.y = gVw->xinerama[i].y_org;
-                if (gVw->fullscreen_req) {
-                    hint.width  = gVw->xinerama[i].width;
-                    hint.height = gVw->xinerama[i].height;
-                    gVw->fullscreen_width = hint.width;
-                    gVw->fullscreen_height = hint.height;
-                } else {
-                    hint.width  = gVw->video_width;
-                    hint.height = gVw->video_height;
-                }
-                *dest_width = hint.width;
-                *dest_height = hint.height;
-                break;
-            }
-        }
-    } else {
-        hint.x = 0;
-        hint.y = 0;
-        if (gVw->fullscreen_req) {
-            hint.width  = gVw->fullscreen_width;
-            hint.height = gVw->fullscreen_height;
-        } else {
-            hint.width  = gVw->video_width;
-            hint.height = gVw->video_height;
-        }
-        *dest_width  = hint.width;
-        *dest_height = hint.height;
+  if (gVw->xinerama) {
+    int i;
+    int knowLocation = 0;
+    
+    /* someday this could also use the centre of the window as the
+     * test point I guess.  Right now it's the upper-left.
+     */
+    if (gGui->video_window != None) {
+      if (gVw->xwin >= 0 && gVw->ywin >= 0 &&
+	  gVw->xwin < gVw->desktopWidth && gVw->ywin < gVw->desktopHeight) {
+	knowLocation = 1;
+      }
     }
-#else
+    
+    for (i = 0; i < gVw->xinerama_cnt; i++) {
+      if (
+	  (knowLocation == 1 &&
+	   gVw->xwin >= gVw->xinerama[i].x_org &&
+	   gVw->ywin >= gVw->xinerama[i].y_org &&
+	   gVw->xwin <= gVw->xinerama[i].x_org+gVw->xinerama[i].width &&
+	   gVw->ywin <= gVw->xinerama[i].y_org+gVw->xinerama[i].height) ||
+	  (knowLocation == 0 &&
+	   gVw->xinerama[i].screen_number == 
+	   XScreenNumberOfScreen(XDefaultScreenOfDisplay(gGui->display)))) {
+	hint.x = gVw->xinerama[i].x_org;
+	hint.y = gVw->xinerama[i].y_org;
+	if (gVw->fullscreen_req) {
+	  hint.width  = gVw->xinerama[i].width;
+	  hint.height = gVw->xinerama[i].height;
+	  gVw->fullscreen_width = hint.width;
+	  gVw->fullscreen_height = hint.height;
+	} else {
+	  hint.width  = gVw->video_width;
+	  hint.height = gVw->video_height;
+	}
+	break;
+      }
+    }
+  } else {
     hint.x = 0;
-    hint.y = 0;   /* for now -- could change later */
+    hint.y = 0;
+    if (gVw->fullscreen_req) {
+      hint.width  = gVw->fullscreen_width;
+      hint.height = gVw->fullscreen_height;
+    } else {
+      hint.width  = gVw->win_width;
+      hint.height = gVw->win_height;
+    }
+  }
+#else
+  hint.x = 0;
+  hint.y = 0;   /* for now -- could change later */
 #endif
-
+  
   if (gVw->fullscreen_req) {
 
-#ifndef HAVE_XINERAMA
-    *dest_width  = gVw->fullscreen_width;
-    *dest_height = gVw->fullscreen_height;
-#endif
-
     if (gGui->video_window) {
 
-      if (gVw->fullscreen_mode) {
+      if (gVw->fullscreen_mode && gGui->visual == gVw->visual) {
 #ifdef HAVE_XF86VIDMODE
 	/*
 	 * resizing the video window may be necessary if the modeline has
@@ -444,8 +361,11 @@ void video_window_adapt_size (void *this,
 	 */
 	XResizeWindow (gGui->display, gGui->video_window,
 		       gVw->fullscreen_width, gVw->fullscreen_height);
+	gVw->output_width    = gVw->fullscreen_width;
+	gVw->output_height   = gVw->fullscreen_height;
 #endif
 	XUnlockDisplay (gGui->display);
+	
 	return;
       }
 
@@ -454,6 +374,9 @@ void video_window_adapt_size (void *this,
     }
 
     gVw->fullscreen_mode = 1;
+    gVw->visual   = gGui->visual;
+    gVw->depth    = gGui->depth;
+    gVw->colormap = gGui->colormap;
 
     /*
      * open fullscreen window
@@ -465,23 +388,23 @@ void video_window_adapt_size (void *this,
 
     gGui->video_window = 
       XCreateWindow (gGui->display, gGui->imlib_data->x.root, 
-		     hint.x, hint.y, gVw->fullscreen_width, 
-		     gVw->fullscreen_height, 
-		     0, gVw->depth, CopyFromParent, 
+		     hint.x, hint.y, gVw->fullscreen_width, gVw->fullscreen_height, 
+		     0, gVw->depth, InputOutput/*CopyFromParent*/, 
 		     gVw->visual,
 		     CWBackPixel  | CWBorderPixel | CWColormap, &attr);
+/* fprintf (stderr, "***** XCreateWindow 2 visual %p id 0x%x depth %d\n", gVw->visual, gVw->visual->visualid, gVw->depth); */
     
     if(gGui->vo_driver)
       gGui->vo_driver->gui_data_exchange (gGui->vo_driver,
 					  GUI_DATA_EX_DRAWABLE_CHANGED, 
 					  (void*)gGui->video_window);
     
-    if (gVw->xclasshint != NULL)
-      XSetClassHint(gGui->display, gGui->video_window, gVw->xclasshint);
+    if (gVw->xclasshint_fullscreen != NULL)
+      XSetClassHint(gGui->display, gGui->video_window, gVw->xclasshint_fullscreen);
 
 #ifndef HAVE_XINERAMA
-    hint.x = 0;
-    hint.y = 0;
+    hint.x      = 0;
+    hint.y      = 0;
     hint.width  = gVw->fullscreen_width;
     hint.height = gVw->fullscreen_height;
 #endif
@@ -495,6 +418,8 @@ void video_window_adapt_size (void *this,
         
     XSetWMHints(gGui->display, gGui->video_window, gVw->wm_hint);
 
+    gVw->output_width    = hint.width;
+    gVw->output_height   = hint.height;
     
     /*
      * layer above most other things, like gnome panel
@@ -505,9 +430,9 @@ void video_window_adapt_size (void *this,
       if( XA_WIN_LAYER == None )
 	XA_WIN_LAYER = XInternAtom(gGui->display, "_WIN_LAYER", False);
       
-      data[0] = 10;
+      propvalue[0] = 10;
       XChangeProperty(gGui->display, gGui->video_window, XA_WIN_LAYER,
-		      XA_CARDINAL, 32, PropModeReplace, (unsigned char *)data,
+		      XA_CARDINAL, 32, PropModeReplace, (unsigned char *)propvalue,
 		      1);
     }
 
@@ -528,19 +453,21 @@ void video_window_adapt_size (void *this,
   } else {
 
 #ifndef HAVE_XINERAMA
-    *dest_width  = gVw->video_width;
-    *dest_height = gVw->video_height;
-
-    hint.x = 0;
-    hint.y = 0;
-    hint.width  = gVw->video_width;
-    hint.height = gVw->video_height;
+    hint.x           = 0;
+    hint.y           = 0;
+    hint.width       = gVw->win_width;
+    hint.height      = gVw->win_height;
 #endif
-    hint.flags  = PPosition | PSize;
+
+    hint.flags       = PPosition | PSize;
+
+
+    gVw->output_width  = hint.width;
+    gVw->output_height = hint.height;
 
     if (gGui->video_window) {
 
-      if (gVw->fullscreen_mode) {
+      if (gVw->fullscreen_mode || gVw->visual != gGui->visual) {
 #ifdef HAVE_XF86VIDMODE
 	/*
 	 * toggling from fullscreen to window mode - time to switch back to
@@ -562,12 +489,12 @@ void video_window_adapt_size (void *this,
       }
       else {
 	
-	XResizeWindow (gGui->display, gGui->video_window, 
-		       gVw->video_width, gVw->video_height);
-
 	/* Update window size hints with the new size */
 	XSetNormalHints(gGui->display, gGui->video_window, &hint);
 
+	XResizeWindow (gGui->display, gGui->video_window, 
+		       gVw->win_width, gVw->win_height);
+
 	XUnlockDisplay (gGui->display);
 	
 	return;
@@ -576,16 +503,21 @@ void video_window_adapt_size (void *this,
     }
 
     gVw->fullscreen_mode = 0;
+    gVw->visual   = gGui->visual;
+    gVw->depth    = gGui->depth;
+    gVw->colormap = gGui->colormap;
 
     attr.background_pixel  = gGui->black.pixel;
     attr.border_pixel      = gGui->black.pixel;
     attr.colormap	   = gVw->colormap;
     
-    gGui->video_window = 
+/* fprintf (stderr, "***** XCreateWindow 3 visual %p id 0x%x depth %d\n", gVw->visual, gVw->visual->visualid, gVw->depth); */
+    gGui->video_window =
       XCreateWindow(gGui->display, gGui->imlib_data->x.root,
 		    hint.x, hint.y, hint.width, hint.height, 4, 
-		    gVw->depth, CopyFromParent, gVw->visual,
+		    gVw->depth, InputOutput/*CopyFromParent*/, gVw->visual,
 		    CWBackPixel | CWBorderPixel | CWColormap, &attr);
+/*fprintf (stderr, "***** XCreateWindow done\n"); */
     
     if(gGui->vo_driver)
       gGui->vo_driver->gui_data_exchange (gGui->vo_driver,
@@ -605,9 +537,7 @@ void video_window_adapt_size (void *this,
   }
   
   
-  XSelectInput(gGui->display, gGui->video_window, 
-	       StructureNotifyMask | ExposureMask | 
-	       KeyPressMask | ButtonPressMask | PointerMotionMask);
+  XSelectInput(gGui->display, gGui->video_window, INPUT_MOTION | KeymapStateMask);
 
   wm_hint = XAllocWMHints();
   if (wm_hint != NULL) {
@@ -633,7 +563,7 @@ void video_window_adapt_size (void *this,
 	       &xev) ;
   } while (xev.type != MapNotify || xev.xmap.event != gGui->video_window);
 
-  XFlush(gGui->display);
+  XSync(gGui->display, False);
 
   if (gVw->gc != None) XFreeGC(gGui->display, gVw->gc);
   gVw->gc = XCreateGC(gGui->display, gGui->video_window, 0L, &xgcv);
@@ -664,14 +594,76 @@ void video_window_adapt_size (void *this,
   }
 
   gVw->old_widget_key = gVw->widget_key;
+
+  XUnlockDisplay (gGui->display);
+
   gVw->widget_key = xitk_register_event_handler("video_window", 
 						gGui->video_window, 
 						video_window_handle_event,
-						video_window_change_sizepos,
+						NULL,
 						gui_dndcallback,
 						NULL, NULL);
   
-  XUnlockDisplay (gGui->display);
+}
+
+/*
+ *
+ */
+void video_window_dest_size_cb (void *data,
+				int video_width, int video_height,
+				int *dest_width, int *dest_height)  {
+
+  if (gVw->fullscreen_mode) {
+    *dest_width  = gVw->fullscreen_width;
+    *dest_height = gVw->fullscreen_height;
+  } else {
+    *dest_width  = gVw->output_width;
+    *dest_height = gVw->output_height;
+  }
+}
+
+/*
+ *
+ */
+void video_window_frame_output_cb (void *data,
+				   int video_width, int video_height,
+				   int *dest_x, int *dest_y, 
+				   int *dest_width, int *dest_height,
+				   int *win_x, int *win_y) {
+  
+  gVw->video_width  = video_width;
+  gVw->video_height = video_height;
+
+  *dest_x = 0;
+  *dest_y = 0;
+
+  if (gVw->fullscreen_mode) {
+    *dest_width  = gVw->fullscreen_width;
+    *dest_height = gVw->fullscreen_height;
+  } else {
+    *dest_width  = gVw->output_width;
+    *dest_height = gVw->output_height;
+  }
+
+  *win_x = (gVw->xwin < 0) ? 0 : gVw->xwin;
+  *win_y = (gVw->ywin < 0) ? 0 : gVw->ywin;
+
+}
+
+/*
+ *
+ */
+void video_window_set_fullscreen (int req_fullscreen) {
+  gVw->fullscreen_req = req_fullscreen;
+  
+  video_window_adapt_size ();
+}
+
+/*
+ *
+ */
+int video_window_is_fullscreen (void) {
+  return gVw->fullscreen_mode;
 }
 
 /*
@@ -679,12 +671,15 @@ void video_window_adapt_size (void *this,
  */
 void video_window_set_cursor_visibility(int show_cursor) {
 
+  if(gGui->use_root_window)
+    return;
+  
   gVw->cursor_visible = show_cursor;
 
   XLockDisplay (gGui->display);
   XDefineCursor(gGui->display, gGui->video_window, 
 		gVw->cursor[show_cursor]);
-  XFlush(gGui->display);
+  XSync(gGui->display, False);
   XUnlockDisplay (gGui->display);
 }
 
@@ -699,10 +694,12 @@ int video_window_is_cursor_visible(void) {
  * hide/show video window 
  */
 void video_window_set_visibility(int show_window) {
-
   static Atom  XA_WIN_LAYER = None;
   long         data[1];
   
+  if(gGui->use_root_window)
+    return;
+
   gGui->vo_driver->gui_data_exchange (gGui->vo_driver, 
 				      GUI_DATA_EX_VIDEOWIN_VISIBLE, 
 				      (int *)show_window);
@@ -750,40 +747,41 @@ int video_window_is_visible (void) {
 static unsigned char bm_no_data[] = { 0,0,0,0, 0,0,0,0 };
 void video_window_init (void) {
 
-  Pixmap             bm_no;
-  int                x,y,w,h;
+  Pixmap                bm_no;
 #ifdef HAVE_XINERAMA
   int                   screens;
   int                   dummy_a, dummy_b;
   XineramaScreenInfo   *screeninfo = NULL;
 #endif
 #ifdef HAVE_XF86VIDMODE
-  int                 dummy_query_event, dummy_query_error;
+  int                   dummy_query_event, dummy_query_error;
 #endif
 
   gVw = (gVw_t *) xine_xmalloc(sizeof(gVw_t));
 
-  gVw->fullscreen_req                   = 0;
-  gVw->fullscreen_mode                  = 0;
-  gGui->video_window                    = None;
-  gVw->show                             = 1;
-  gVw->widget_key = gVw->old_widget_key = 0;
-  gVw->gc				= None;
+  gVw->fullscreen_req     = 0;
+  gVw->fullscreen_mode    = 0;
+  gGui->video_window      = None;
+  gVw->show               = 1;
+  gVw->widget_key         = 
+    gVw->old_widget_key   = 0;
+  gVw->gc		  = None;
 
   XLockDisplay (gGui->display);
 
+  gVw->depth	          = gGui->depth;
+  gVw->visual		  = gGui->visual;
+  gVw->colormap		  = gGui->colormap;
+  /* Currently, there no plugin loaded so far, but that might change */
+  video_window_select_visual ();
+  gVw->xwin               = -8192;
+  gVw->ywin               = -8192;
+  gVw->desktopWidth       = DisplayWidth(gGui->display, gGui->screen);
+  gVw->desktopHeight      = DisplayHeight(gGui->display, gGui->screen);
 
-  gVw->depth				= gGui->depth;
-  gVw->visual				= gGui->visual;
-  gVw->colormap				= Imlib_get_colormap(gGui->imlib_data);
-  gVw->xwin                             = -8192;
-  gVw->ywin                             = -8192;
-  gVw->desktopWidth                     = DisplayWidth(gGui->display, gGui->screen);
-  gVw->desktopHeight                    = DisplayHeight(gGui->display, gGui->screen);
-  
 #ifdef HAVE_XINERAMA
-  gVw->xinerama				= NULL;
-  gVw->xinerama_cnt			= 0;
+  gVw->xinerama		  = NULL;
+  gVw->xinerama_cnt	  = 0;
   /* Spark
    * some Xinerama stuff
    * I want to figure out what fullscreen means for this setup
@@ -823,6 +821,10 @@ void video_window_init (void) {
     gVw->xclasshint->res_name = "Xine Video Window";
     gVw->xclasshint->res_class = "Xine";
   }
+  if ((gVw->xclasshint_fullscreen = XAllocClassHint()) != NULL) {
+    gVw->xclasshint_fullscreen->res_name = "Xine Video Fullscreen Window";
+    gVw->xclasshint_fullscreen->res_class = "Xine";
+  }
 
   /* 
    * create cursors
@@ -900,9 +902,13 @@ void video_window_init (void) {
 
   XUnlockDisplay (gGui->display);
 
-  video_window_adapt_size (NULL, 768, 480, &x, &y, &w, &h);
-  video_window_draw_logo();
+  gVw->video_width  = 768;
+  gVw->video_height = 480;
+  
+  video_window_set_mag(1.0);
 
+  /* Ensure we load the right logo */
+  video_window_change_skins();
 }
 
 
@@ -968,6 +974,53 @@ static int video_window_translate_point(int gui_x, int gui_y,
   return 1;
 }
 
+/*
+ * Set/Get magnification.
+ */
+void video_window_set_mag(float mag) {
+  
+  gVw->mag = mag;
+  gVw->win_width  = (int) ((float) gVw->video_width) * gVw->mag;
+  gVw->win_height = (int) ((float) gVw->video_height) * gVw->mag;
+  
+  video_window_adapt_size ();
+}
+float video_window_get_mag (void) {
+  
+  /* compute current mag */
+  gVw->mag = (((float) gVw->output_width / (float) gVw->video_width ) + 
+	      ((float) gVw->output_height / (float) gVw->video_height )) * .5;
+  return gVw->mag;
+}
+
+/*
+ * Change displayed logo, if selected skin want to customize it.
+ */
+void video_window_change_skins(void) {
+  cfg_entry_t *cfg_entry;
+  char        *new_vo_logo;
+
+  cfg_entry   = gGui->config->lookup_entry(gGui->config, "video.logo_file");
+  new_vo_logo = xitk_skin_get_skin_filename(gGui->skin_config, "VOLogo");
+  
+  if(new_vo_logo) {
+
+    if(cfg_entry && cfg_entry->str_value) {
+      /* Old and new logo are same, don't reload */
+      if(!strcmp(cfg_entry->str_value, new_vo_logo))
+	return;
+    }
+    
+    gGui->config->update_string(gGui->config, "video.logo_file", new_vo_logo);
+  }
+  else {
+    char  default_logo[2048];
+    
+    memset(&default_logo, 0, sizeof(default_logo));
+    snprintf(default_logo, 2048, "%s/xine_logo.zyuy2", XINE_SKINDIR);
+    gGui->config->update_string(gGui->config, "video.logo_file", default_logo);
+  }
+}
 
 /*
  *
@@ -1044,14 +1097,9 @@ static void video_window_handle_event (XEvent *event, void *data) {
     if (xev->count == 0) {
 
       if(event->xany.window == gGui->video_window) {
-	if(xine_get_status(gGui->xine) == XINE_STOP)
-	  video_window_draw_logo();
-	else {
-
-	  gGui->vo_driver->gui_data_exchange (gGui->vo_driver, 
-					      GUI_DATA_EX_EXPOSE_EVENT, 
-					      event);
-	}
+	gGui->vo_driver->gui_data_exchange (gGui->vo_driver, 
+					    GUI_DATA_EX_EXPOSE_EVENT, 
+					    event);
       }
     }
   }
@@ -1059,19 +1107,14 @@ static void video_window_handle_event (XEvent *event, void *data) {
 
   case ConfigureNotify:
     if(event->xany.window == gGui->video_window) {
-      if(xine_get_status(gGui->xine) != XINE_STOP) {
-	XConfigureEvent *cev = (XConfigureEvent *) event;
-	x11_rectangle_t area;
+      XConfigureEvent *cev = (XConfigureEvent *) event;
+
+      gVw->output_width  = cev->width;
+      gVw->output_height = cev->height;
+
+      gVw->xwin = cev->x;
+      gVw->ywin = cev->y;
 
-	area.x = 0;
-	area.y = 0;
-	area.w = cev->width;
-	area.h = cev->height;
-	
-	gGui->vo_driver->gui_data_exchange (gGui->vo_driver, 
-					    GUI_DATA_EX_DEST_POS_SIZE_CHANGED, 
-					    &area);
-      }
     }
     break;
     
