@@ -1,7 +1,7 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: videowin.h,v 1.10 2001/10/22 00:52:46 guenter Exp $
+ * $Id: videowin.h,v 1.20 2002/04/27 13:26:44 f1rmb Exp $
  *
  * video window handling functions
  */
@@ -27,16 +27,19 @@
 
 void video_window_init (void);
 
-void video_window_adapt_size (void *this,
-			      int video_width, int video_height, 
-			      int *dest_x, int *dest_y,
-			      int *dest_width, int *dest_height);
+void video_window_select_visual (void);
 
-void video_window_calc_dest_size (void *this,
-				  int video_width, int video_height,
-				  int *dest_width, int *dest_height);
+void video_window_frame_output_cb (void *this,
+				   int video_width, int video_height, 
+				   int *dest_x, int *dest_y,
+				   int *dest_width, int *dest_height,
+				   int *win_x, int *win_y);
 
-/* set/get fullscreen mode */
+void video_window_dest_size_cb (void *this,
+				int video_width, int video_height,
+				int *dest_width, int *dest_height);
+
+/* set/check fullscreen mode */
 void video_window_set_fullscreen (int req_fullscreen);
 int video_window_is_fullscreen (void);
 
@@ -49,7 +52,9 @@ int video_window_is_cursor_visible(void);
 void video_window_set_visibility(int show_window);
 int video_window_is_visible (void);
 
-void video_window_show_logo(void);
-void video_window_hide_logo(void);
+void video_window_set_mag (float mag);
+float video_window_get_mag (void);
+
+void video_window_change_skins(void);
 
 #endif
