@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: viewlog.h,v 1.1 2001/12/10 00:16:44 f1rmb Exp $
+ * $Id: viewlog.h,v 1.3 2002/03/21 00:22:52 f1rmb Exp $
  *
  */
 
@@ -28,7 +28,7 @@ void viewlog_window(void);
 void viewlog_exit(xitk_widget_t *, void *);
 int viewlog_is_visible(void);
 int viewlog_is_running(void);
-void viewlog_toggle_panel_visibility(xitk_widget_t *, void *);
+void viewlog_toggle_visibility(xitk_widget_t *, void *);
 void viewlog_raise_window(void);
 
 #endif
