@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: _xitk.h,v 1.26 2002/01/15 22:22:35 f1rmb Exp $
+ * $Id: _xitk.h,v 1.35 2002/04/21 22:17:12 f1rmb Exp $
  *
  *
  */
@@ -30,6 +30,7 @@
 #endif
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <limits.h>
 #include <X11/Xlib.h>
 #include "Imlib-light/Imlib.h"
@@ -66,15 +67,21 @@ typedef void (*xitk_mrl_callback_t)(xitk_widget_t *, void *, mrl_t *);
 #define XITK_PATH_MAX   PATH_MAX
 #endif
 
+#undef  MAX
+#define MAX(a, b)  (((a) > (b)) ? (a) : (b))
+
+#undef  MIN
+#define MIN(a, b)  (((a) < (b)) ? (a) : (b))
+
 #ifdef	__GNUC__
 #define XITK_DIE(FMT, ARGS...) { fprintf(stderr, "XITK DIE: "FMT, ##ARGS); exit(-1); }
 #define XITK_WARNING(FMT, ARGS...) fprintf(stderr, "XITK WARNING: "FMT, ##ARGS)
 #else	/* C99 version: */
-#define XITK_DIE(...) { fprintf(stder, "XITK DIE: "__VA_ARGS__); exit(-1); }
-#define XITK_WARNING(...) fprintf(stder, "XITK WARNING: "__VA_ARGS__)
+#define XITK_DIE(...) { fprintf(stderr, "XITK DIE: "__VA_ARGS__); exit(-1); }
+#define XITK_WARNING(...) fprintf(stderr, "XITK WARNING: "__VA_ARGS__)
 #endif
 
-#define XITK_FREE(X) if((X)) free((X))
+#define XITK_FREE(X) { if((X)) { free((X)); (X) = NULL; }}
 #define XITK_FREE_XITK_IMAGE(D, X) {                                                     \
                                      if(((X)) && ((X)->image)) {                         \
                                        XFreePixmap((D), (X)->image);                     \
@@ -126,6 +133,72 @@ typedef void (*xitk_mrl_callback_t)(xitk_widget_t *, void *, mrl_t *);
   } while (0)
 #endif
 
+#ifdef HAVE_STRPBRK
+#define xitk_strpbrk strpbrk
+#else
+static inline char *_x_strpbrk(const char *s, const char *accept) {
+
+  while(*s != '\0') {
+    const char *a = accept;
+    while(*a != '\0')
+      if(*a++ == *s)
+	return(char *) s;
+    ++s;
+  }
+
+  return NULL;
+}
+#define xitk_strpbrk _x_strpbrk
+#endif
+
+#ifdef HAVE_STRSEP
+#define xitk_strsep strsep
+#else
+static inline char *_x_strsep(char **stringp, const char *delim) {
+  char *begin, *end;
+  
+  begin = *stringp;
+  if(begin == NULL)
+    return NULL;
+  
+  if(delim[0] == '\0' || delim[1] == '\0') {
+    char ch = delim[0];
+    
+    if(ch == '\0')
+      end = NULL;
+    else {
+      if(*begin == ch)
+	end = begin;
+      else if(*begin == '\0')
+	end = NULL;
+      else
+	end = strchr(begin + 1, ch);
+    }
+  }
+  else
+    end = xitk_strpbrk(begin, delim);
+  
+  if(end) {
+    *end++ = '\0';
+    *stringp = end;
+  }
+  else
+    *stringp = NULL;
+  
+  return begin;
+}
+#define xitk_strsep _x_strsep
+#endif
+
+void xitk_strdupa(char *dest, char *src);
+#define xitk_strdupa(d, s) {                                        \
+  (d) = NULL;                                                       \
+  if((s) != NULL) {                                                 \
+    (d) = (char *) alloca(strlen((s)) + 1);                         \
+    strcpy((d), (s));                                               \
+  }                                                                 \
+}
+
 /* Duplicate s to d timeval values */
 #define timercpy(s, d) {                                                      \
       (d)->tv_sec = (s)->tv_sec;                                              \
@@ -143,6 +216,11 @@ typedef void (*xitk_mrl_callback_t)(xitk_widget_t *, void *, mrl_t *);
       (w)->width  = 0;                                                        \
     }
 
+#define INPUT_MOTION (ExposureMask | ButtonPressMask | ButtonReleaseMask |    \
+                      KeyPressMask | KeyReleaseMask | ButtonMotionMask |      \
+                      StructureNotifyMask | PropertyChangeMask |              \
+                      LeaveWindowMask | EnterWindowMask | PointerMotionMask)
+
 typedef struct {
   Window    window;
   char     *name;
@@ -200,6 +278,7 @@ typedef struct {
 
 #define XITK_VSLIDER 1
 #define XITK_HSLIDER 2
+#define XITK_RSLIDER 3
 
 typedef struct {
   int                       magic;
@@ -390,7 +469,7 @@ void xitk_xevent_notify(XEvent *event);
 #endif
 
 const char *xitk_get_homedir(void);
-void xitk_usec_sleep(unsigned usec);
+void xitk_usec_sleep(unsigned long);
 int xitk_system(int dont_run_as_root, char *command);
 
 char *xitk_get_system_font(void);
@@ -401,7 +480,10 @@ int xitk_get_white_color(void);
 int xitk_get_background_color(void);
 int xitk_get_focus_color(void);
 int xitk_get_select_color(void);
+unsigned long xitk_get_timer_label_animation(void);
 
+int xitk_skin_get_visibility(xitk_skin_config_t *, const char *);
+int xitk_skin_get_enability(xitk_skin_config_t *, const char *);
 int xitk_skin_get_coord_x(xitk_skin_config_t *, const char *);
 int xitk_skin_get_coord_y(xitk_skin_config_t *, const char *);
 char *xitk_skin_get_label_color(xitk_skin_config_t *, const char *);
@@ -410,12 +492,16 @@ char *xitk_skin_get_label_color_click(xitk_skin_config_t *, const char *);
 int xitk_skin_get_label_length(xitk_skin_config_t *, const char *);
 int xitk_skin_get_label_animation(xitk_skin_config_t *, const char *);
 char *xitk_skin_get_label_fontname(xitk_skin_config_t *, const char *);
-int xitk_skin_get_label_printable(xitk_skin_config_t *skonfig, const char *str);
-int xitk_skin_get_label_alignment(xitk_skin_config_t *skonfig, const char *str);
-char *xitk_skin_get_label_skinfont_filename(xitk_skin_config_t *skonfig, const char *str);
+int xitk_skin_get_label_printable(xitk_skin_config_t *, const char *);
+int xitk_skin_get_label_staticity(xitk_skin_config_t *, const char *);
+int xitk_skin_get_label_alignment(xitk_skin_config_t *, const char *);
+char *xitk_skin_get_label_skinfont_filename(xitk_skin_config_t *, const char *);
 char *xitk_skin_get_skin_filename(xitk_skin_config_t *, const char *);
-char *xitk_skin_get_slider_skin_filename(xitk_skin_config_t *skonfig, const char *str);
-int xitk_skin_get_slider_type(xitk_skin_config_t *skonfig, const char *str);
+char *xitk_skin_get_slider_skin_filename(xitk_skin_config_t *, const char *);
+int xitk_skin_get_slider_type(xitk_skin_config_t *, const char *);
+int xitk_skin_get_slider_radius(xitk_skin_config_t *, const char *);
+void xitk_skin_lock(xitk_skin_config_t *);
+void xitk_skin_unlock(xitk_skin_config_t *);
 
 typedef struct {
   Display       *display;
