@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: browser.c,v 1.20 2002/01/15 22:22:36 f1rmb Exp $
+ * $Id: browser.c,v 1.25 2002/04/18 21:14:35 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -61,18 +61,15 @@ static void paint(xitk_widget_t *w, Window win, GC gc) {
   if(w->widget_type & WIDGET_TYPE_BROWSER) {
     int i;
 
-    if(w->visible) {
-      int x     = w->x;
-      int y     = w->y;
-      int itemw = xitk_get_widget_width(private_data->item_tree[WBSTART]);
-      int itemh = xitk_get_widget_height(private_data->item_tree[WBSTART]);
-      int iy    = w->y;
-      int bh;
-      
-      if(private_data->skin_element_name) {
-	notify_change_skin(private_data->parent_wlist, w, private_data->skonfig);
-      }
-      else {
+    if((w->visible == 1)) {
+      if(!private_data->skin_element_name) {
+	int x     = w->x;
+	int y     = w->y;
+	int itemw = xitk_get_widget_width(private_data->item_tree[WBSTART]);
+	int itemh = xitk_get_widget_height(private_data->item_tree[WBSTART]);
+	int iy    = w->y;
+	int bh;
+	
 	(void) xitk_set_widget_pos(private_data->item_tree[WBUP], x + itemw, y);
 	bh = xitk_get_widget_height(private_data->item_tree[WBUP]);
 	(void) xitk_set_widget_pos(private_data->item_tree[WSLID], x + itemw, y + bh);
@@ -89,7 +86,6 @@ static void paint(xitk_widget_t *w, Window win, GC gc) {
 	  iy += itemh;
 	}
       }
-      
     }
     else {
       for(i = 0; i < private_data->max_length+WBSTART; i++) {
@@ -104,17 +100,20 @@ static void paint(xitk_widget_t *w, Window win, GC gc) {
  */
 static void notify_change_skin(xitk_widget_list_t *wl, 
 			       xitk_widget_t *w, xitk_skin_config_t *skonfig) {
-  browser_private_data_t *private_data = 
-    (browser_private_data_t *) w->private_data;
-
+  browser_private_data_t *private_data = (browser_private_data_t *) w->private_data;
+  
   if(w->widget_type & WIDGET_TYPE_BROWSER) {
+    xitk_skin_lock(skonfig);
+    
     if(private_data->skin_element_name) {
       int x, y, i = 0;
       
-      x = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
-      y = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
+      x          = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
+      y          = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
+      w->visible = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1 : -1;
+      w->enable  = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
       
-      for(i = WBSTART; i < private_data->max_length+WBSTART; i++) {
+      for(i = WBSTART; i < private_data->max_length + WBSTART; i++) {
 	
 	(void) xitk_set_widget_pos(private_data->item_tree[i], x, y);
 	
@@ -122,6 +121,7 @@ static void notify_change_skin(xitk_widget_list_t *wl,
 	
       }
     }
+    xitk_skin_unlock(skonfig);
   }
 }
 
@@ -488,7 +488,8 @@ static xitk_widget_t *_xitk_browser_create(xitk_skin_config_t *skonfig, xitk_bro
 					   int x, int y, int width, int height,
 					   char *skin_element_name,
 					   xitk_widget_t *mywidget,
-					   browser_private_data_t *_private_data) {
+					   browser_private_data_t *_private_data,
+					   int visible, int enable) {
   browser_private_data_t        *private_data = _private_data;
   xitk_button_widget_t           b;
   xitk_labelbutton_widget_t      lb;
@@ -527,9 +528,9 @@ static xitk_widget_t *_xitk_browser_create(xitk_skin_config_t *skonfig, xitk_bro
   
   mywidget->private_data               = private_data;
 
-  mywidget->enable                     = 1;
+  mywidget->enable                     = enable;
   mywidget->running                    = 1;
-  mywidget->visible                    = 1;
+  mywidget->visible                    = visible;
   mywidget->have_focus                 = FOCUS_LOST;
   
   mywidget->imlibdata                  = private_data->imlibdata;
@@ -631,7 +632,9 @@ xitk_widget_t *xitk_browser_create(xitk_skin_config_t *skonfig, xitk_browser_wid
   }
   
   return _xitk_browser_create(skonfig, br, 
-			      0, 0, 0, 0, br->browser.skin_element_name, mywidget, private_data);
+			      0, 0, 0, 0, br->browser.skin_element_name, mywidget, private_data,
+			      (xitk_skin_get_visibility(skonfig, br->browser.skin_element_name)) ? 1 : -1,
+			      xitk_skin_get_enability(skonfig, br->browser.skin_element_name));
 }
 
 /*
@@ -743,5 +746,5 @@ xitk_widget_t *xitk_noskin_browser_create(xitk_browser_widget_t *br, GC gc, int
   
   return _xitk_browser_create(NULL, br, x, y, 
 			      (itemw + slidw), (itemh * br->browser.max_displayed_entries), 
-			      NULL, mywidget, private_data);
+			      NULL, mywidget, private_data, 1, 1);
 }
