@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: button.c,v 1.14 2002/01/15 22:22:36 f1rmb Exp $
+ * $Id: button.c,v 1.18 2002/03/21 00:22:52 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -72,7 +72,7 @@ static int notify_inside(xitk_widget_t *b, int x, int y) {
     (button_private_data_t *) b->private_data;
 
   if(b->widget_type & WIDGET_TYPE_BUTTON) {
-    if(b->visible) {
+    if((b->visible == 1)) {
       xitk_image_t *skin = private_data->skin;
       
       return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, b, skin->mask, x, y);
@@ -94,7 +94,7 @@ static void paint_button (xitk_widget_t *b, Window win, GC gc) {
   int                 button_width;
   xitk_image_t       *skin;
 
-  if((b->widget_type & WIDGET_TYPE_BUTTON) && b->visible) {
+  if((b->widget_type & WIDGET_TYPE_BUTTON) && (b->visible == 1)) {
 
     skin         = private_data->skin;
     button_width = skin->width / 3;
@@ -144,7 +144,7 @@ static void notify_change_skin(xitk_widget_list_t *wl,
   if(b->widget_type & WIDGET_TYPE_BUTTON) {
     
     if(private_data->skin_element_name) {
-      
+      xitk_skin_lock(skonfig);
       XITK_FREE_XITK_IMAGE(private_data->imlibdata->x.disp, private_data->skin);
       private_data->skin              = xitk_image_load_image(private_data->imlibdata,
 							      (xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name)));
@@ -153,6 +153,10 @@ static void notify_change_skin(xitk_widget_list_t *wl,
       b->width                        = private_data->skin->width/3;
       b->height                       = private_data->skin->height;
       
+      b->visible                      = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1 : -1;
+      b->enable                       = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
+    
+      xitk_skin_unlock(skonfig);
       xitk_set_widget_pos(b, b->x, b->y);
     }
   }
@@ -204,7 +208,8 @@ static int notify_focus_button (xitk_widget_list_t *wl, xitk_widget_t *b, int bE
  */
 static xitk_widget_t *_xitk_button_create (xitk_skin_config_t *skonfig, xitk_button_widget_t *b,
 					   int x, int y, 
-					   char *skin_element_name, xitk_image_t *skin) {
+					   char *skin_element_name, xitk_image_t *skin,
+					   int visible, int enable) {
   xitk_widget_t          *mywidget;
   button_private_data_t  *private_data;
   
@@ -226,9 +231,9 @@ static xitk_widget_t *_xitk_button_create (xitk_skin_config_t *skonfig, xitk_but
   
   mywidget->private_data          = private_data;
 
-  mywidget->enable                = 1;
+  mywidget->enable                = enable;
   mywidget->running               = 1;
-  mywidget->visible               = 1;
+  mywidget->visible               = visible;
   mywidget->have_focus            = FOCUS_LOST; 
   mywidget->imlibdata             = private_data->imlibdata;
   mywidget->x                     = x;
@@ -263,7 +268,9 @@ xitk_widget_t *xitk_button_create (xitk_skin_config_t *skonfig, xitk_button_widg
 			     (xitk_skin_get_coord_y(skonfig, b->skin_element_name)),
 			     b->skin_element_name,
 			     xitk_image_load_image(b->imlibdata,
-						   (xitk_skin_get_skin_filename(skonfig, b->skin_element_name))));
+						   (xitk_skin_get_skin_filename(skonfig, b->skin_element_name))),
+			     (xitk_skin_get_visibility(skonfig, b->skin_element_name)) ? 1 : -1,
+			     xitk_skin_get_enability(skonfig, b->skin_element_name));
 }
 
 /*
@@ -278,5 +285,5 @@ xitk_widget_t *xitk_noskin_button_create (xitk_button_widget_t *b,
   i = xitk_image_create_image(b->imlibdata, width * 3, height);
   draw_bevel_three_state(b->imlibdata, i);
 
-  return _xitk_button_create(NULL, b, x, y, NULL, i);
+  return _xitk_button_create(NULL, b, x, y, NULL, i, 1, 1);
 }
