@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: checkbox.c,v 1.14 2002/01/15 22:22:36 f1rmb Exp $
+ * $Id: checkbox.c,v 1.18 2002/03/21 00:22:52 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -70,7 +70,7 @@ static int notify_inside(xitk_widget_t *c, int x, int y) {
     (checkbox_private_data_t *) c->private_data;
   
   if(c->widget_type & WIDGET_TYPE_CHECKBOX) {
-    if(c->visible) {
+    if((c->visible == 1)) {
       xitk_image_t *skin = private_data->skin;
       
       return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, c, skin->mask, x, y);
@@ -92,7 +92,7 @@ static void paint_checkbox (xitk_widget_t *c, Window win, GC gc) {
   int           checkbox_width;
   xitk_image_t  *skin;
   
-  if ((c->widget_type & WIDGET_TYPE_CHECKBOX) && c->visible) {
+  if ((c->widget_type & WIDGET_TYPE_CHECKBOX) && (c->visible == 1)) {
     
     skin           = private_data->skin;
     checkbox_width = skin->width / 3;
@@ -187,6 +187,7 @@ static void notify_change_skin(xitk_widget_list_t *wl,
   
   if (c->widget_type & WIDGET_TYPE_CHECKBOX) {
     if(private_data->skin_element_name) {
+      xitk_skin_lock(skonfig);
       XITK_FREE_XITK_IMAGE(private_data->imlibdata->x.disp, private_data->skin);
       private_data->skin = xitk_image_load_image(private_data->imlibdata,
 						 xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
@@ -194,7 +195,11 @@ static void notify_change_skin(xitk_widget_list_t *wl,
       c->y               = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
       c->width           = private_data->skin->width/3;
       c->height          = private_data->skin->height;
-      
+      c->visible         = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1: -1;
+      c->enable          = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
+
+      xitk_skin_unlock(skonfig);
+
       xitk_set_widget_pos(c, c->x, c->y);
     }
   }
@@ -247,7 +252,8 @@ void xitk_checkbox_set_state(xitk_widget_t *c, int state, Window win, GC gc) {
  */
 static xitk_widget_t *_xitk_checkbox_create(xitk_skin_config_t *skonfig, 
 					    xitk_checkbox_widget_t *cb, int x, int y, 
-					    char *skin_element_name, xitk_image_t *skin) {
+					    char *skin_element_name, xitk_image_t *skin,
+					    int visible, int enable) {
   xitk_widget_t *mywidget;
   checkbox_private_data_t *private_data;
 
@@ -270,9 +276,9 @@ static xitk_widget_t *_xitk_checkbox_create(xitk_skin_config_t *skonfig,
 
   mywidget->private_data          = private_data;
 
-  mywidget->enable                = 1;
+  mywidget->enable                = enable;
   mywidget->running               = 1;
-  mywidget->visible               = 1;
+  mywidget->visible               = visible;
   mywidget->have_focus            = FOCUS_LOST;
   mywidget->imlibdata             = private_data->imlibdata;
   mywidget->x                     = x;
@@ -307,7 +313,9 @@ xitk_widget_t *xitk_checkbox_create (xitk_skin_config_t *skonfig, xitk_checkbox_
 			       (xitk_skin_get_coord_y(skonfig, cb->skin_element_name)),
 			       cb->skin_element_name,
 			       (xitk_image_load_image(cb->imlibdata,
-						      xitk_skin_get_skin_filename(skonfig, cb->skin_element_name))));
+						      xitk_skin_get_skin_filename(skonfig, cb->skin_element_name))),
+			       (xitk_skin_get_visibility(skonfig, cb->skin_element_name)) ? 1 : -1,
+			       xitk_skin_get_enability(skonfig, cb->skin_element_name));
 
 }
 
@@ -323,5 +331,5 @@ xitk_widget_t *xitk_noskin_checkbox_create(xitk_checkbox_widget_t *cb,
   i = xitk_image_create_image(cb->imlibdata, width * 3, height);
   draw_bevel_three_state(cb->imlibdata, i);
   
-  return _xitk_checkbox_create(NULL, cb, x, y, NULL, i);
+  return _xitk_checkbox_create(NULL, cb, x, y, NULL, i, 1, 1);
 }
