@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: combo.c,v 1.8 2001/12/17 02:03:52 f1rmb Exp $
+ * $Id: combo.c,v 1.13 2002/04/12 21:38:51 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -75,12 +75,12 @@ static void paint(xitk_widget_t *w, Window win, GC gc) {
   combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
 
   if(w->widget_type & WIDGET_TYPE_COMBO) {
-    if(private_data->visible == 1 && w->visible == 0) {
+    if(private_data->visible == 1 && (w->visible < 1)) {
       xitk_checkbox_set_state(private_data->button_widget, 0,
 			      private_data->parent_wlist->win, private_data->parent_wlist->gc);
       _combo_rollunroll(NULL, (void *)w, 0);
     }
-    if(w->visible) {
+    if((w->visible == 1)) {
       int bx, lw;
 
       lw = xitk_get_widget_width(private_data->label_widget);
@@ -170,11 +170,18 @@ static void notify_change_skin(xitk_widget_list_t *wl,
     if(private_data->skin_element_name) {
       int x, y;
 
+      xitk_skin_lock(skonfig);
+
+      c->visible = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1 : -1;
+      c->enable  = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
+
       xitk_set_widget_pos(c, c->x, c->y);
       xitk_get_widget_pos(private_data->label_widget, &x, &y);
       x += xitk_get_widget_width(private_data->label_widget);
       
       (void) xitk_set_widget_pos(private_data->button_widget, x, y);
+
+      xitk_skin_unlock(skonfig);
     }
   }
 }
@@ -193,8 +200,9 @@ static void _combo_rollunroll(xitk_widget_t *w, void *data, int state) {
       xitk_combo_update_pos(combo);
     }
     else {
-      XLOCK(private_data->imlibdata->x.disp);
       private_data->visible = 0;
+
+      XLOCK(private_data->imlibdata->x.disp);
       XUnmapWindow(private_data->imlibdata->x.disp, (xitk_window_get_window(private_data->xwin)));
       XUNLOCK(private_data->imlibdata->x.disp);
     }
@@ -307,7 +315,8 @@ void xitk_combo_update_list(xitk_widget_t *w, char **list, int len) {
 static xitk_widget_t *_xitk_combo_create(xitk_skin_config_t *skonfig,
 					 xitk_combo_widget_t *c, char *skin_element_name,
 					 xitk_widget_t *mywidget, 
-					 combo_private_data_t *private_data) {
+					 combo_private_data_t *private_data,
+					 int visible, int enable) {
   Atom                        XA_WIN_LAYER;
   long                        data[1];
   char                      **entries = c->entries;
@@ -322,8 +331,6 @@ static xitk_widget_t *_xitk_combo_create(xitk_skin_config_t *skonfig,
   itemw = xitk_get_widget_width(private_data->label_widget);
   itemw += xitk_get_widget_width(private_data->button_widget);
 
-  XLOCK(c->imlibdata->x.disp);
-
   private_data->imlibdata                = c->imlibdata;
   private_data->skin_element_name        = (skin_element_name == NULL) ? NULL : strdup(skin_element_name);
   private_data->entries                  = c->entries;
@@ -351,6 +358,8 @@ static xitk_widget_t *_xitk_combo_create(xitk_skin_config_t *skonfig,
   
   private_data->xwin = xitk_window_create_simple_window(c->imlibdata, 0, 0,
 							(itemw + 2), (itemh * 5) + 2);
+  XLOCK(c->imlibdata->x.disp);
+
   if(c->layer_above) {
     XA_WIN_LAYER = XInternAtom(c->imlibdata->x.disp, "_WIN_LAYER", False);
     
@@ -377,6 +386,8 @@ static xitk_widget_t *_xitk_combo_create(xitk_skin_config_t *skonfig,
   private_data->gc = XCreateGC(c->imlibdata->x.disp, 
 			       (xitk_window_get_window(private_data->xwin)), None, None);
 
+  XUNLOCK(c->imlibdata->x.disp);
+
   private_data->widget_list                = xitk_widget_list_new() ;
   private_data->widget_list->l             = xitk_list_new ();
   private_data->widget_list->focusedWidget = NULL;
@@ -417,14 +428,13 @@ static xitk_widget_t *_xitk_combo_create(xitk_skin_config_t *skonfig,
 				private_data->widget_list,
 				(void *) private_data);
 
-  XUNLOCK(c->imlibdata->x.disp);
   private_data->visible                  = 0;
 
   mywidget->private_data                 = private_data;
 
-  mywidget->enable                       = 1;
+  mywidget->enable                       = enable;
   mywidget->running                      = 1;
-  mywidget->visible                      = 1;
+  mywidget->visible                      = visible;
   mywidget->have_focus                   = FOCUS_LOST;
 
 
@@ -466,8 +476,6 @@ xitk_widget_t *xitk_combo_create(xitk_skin_config_t *skonfig, xitk_combo_widget_
 
   private_data = (combo_private_data_t *) xitk_xmalloc (sizeof(combo_private_data_t));
 
-  XLOCK(c->imlibdata->x.disp);
-
   /* Create label and button (skinable) */
   lbl.label             = "";
   lbl.skin_element_name = c->skin_element_name;
@@ -496,9 +504,9 @@ xitk_widget_t *xitk_combo_create(xitk_skin_config_t *skonfig, xitk_combo_widget_
     (void) xitk_set_widget_pos(private_data->button_widget, x, y);
   }
 
-  XUNLOCK(c->imlibdata->x.disp);
-
-  return _xitk_combo_create(skonfig, c, c->skin_element_name, mywidget, private_data);
+  return _xitk_combo_create(skonfig, c, c->skin_element_name, mywidget, private_data,
+			    (xitk_skin_get_visibility(skonfig, c->skin_element_name)) ? 1 : -1,
+			    xitk_skin_get_enability(skonfig, c->skin_element_name));
 }
 
 /*
@@ -521,8 +529,6 @@ xitk_widget_t *xitk_noskin_combo_create(xitk_combo_widget_t *c,
 
   private_data = (combo_private_data_t *) xitk_xmalloc (sizeof(combo_private_data_t));
 
-  XLOCK(c->imlibdata->x.disp);
-
   /* Create label and button (skinable) */
   {
     xitk_font_t    *fs;
@@ -580,7 +586,5 @@ xitk_widget_t *xitk_noskin_combo_create(xitk_combo_widget_t *c,
 
   }
 
-  XUNLOCK(c->imlibdata->x.disp);
-
-  return _xitk_combo_create(NULL, c, NULL, mywidget, private_data);
+  return _xitk_combo_create(NULL, c, NULL, mywidget, private_data, 1, 1);
 }
