@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: dnd.h,v 1.4 2001/11/13 23:21:53 f1rmb Exp $
+ * $Id: dnd.h,v 1.7 2002/04/27 18:42:46 f1rmb Exp $
  *
  */
 
@@ -28,24 +28,35 @@
 
 typedef void (*xitk_dnd_callback_t) (char *filename);
 
+#define MAX_SUPPORTED_TYPE 2
+
 typedef struct {
   Display             *display;
   Window               win;
-  
+
   xitk_dnd_callback_t  callback;
 
+  int                  x;
+  int                  y;
+  Window               dropper_toplevel;
+  Window               dropper_window;
+  Window               dragger_window;
+  Atom                *dragger_typelist;
+  Atom                 desired;
+  Time                 time;
+
   Atom                 _XA_XdndAware;
   Atom                 _XA_XdndEnter;
   Atom                 _XA_XdndLeave;
   Atom                 _XA_XdndDrop;
   Atom                 _XA_XdndPosition;
   Atom                 _XA_XdndStatus;
-  Atom                 _XA_XdndActionCopy;
   Atom                 _XA_XdndSelection;
   Atom                 _XA_XdndFinished;
-  Atom                 _XA_XINE_XDNDEXCHANGE;
+  Atom                 _XA_XdndTypeList;
+  Atom                 _XA_XITK_PROTOCOL_ATOM;
   Atom                 _XA_WM_DELETE_WINDOW;
-  Atom                 atom_support;
+  Atom                 supported[MAX_SUPPORTED_TYPE];
   Atom                 version;
 
 } xitk_dnd_t;
@@ -95,10 +106,11 @@ typedef struct {
 
 void xitk_init_dnd(Display *display, xitk_dnd_t *);
 
-void xitk_make_window_dnd_aware(xitk_dnd_t *, Window);
+int xitk_make_window_dnd_aware(xitk_dnd_t *, Window);
 
-Bool xitk_process_client_dnd_message(xitk_dnd_t *, XEvent *);
+int xitk_process_client_dnd_message(xitk_dnd_t *, XEvent *);
 
 void xitk_set_dnd_callback(xitk_dnd_t *, xitk_dnd_callback_t);
+void xitk_unset_dnd_callback(xitk_dnd_t *xdnd);
 
 #endif
