@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: filebrowser.c,v 1.27 2002/01/15 22:22:36 f1rmb Exp $
+ * $Id: filebrowser.c,v 1.34 2002/04/21 22:17:12 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -43,6 +43,7 @@
 #include "button.h"
 #include "labelbutton.h"
 #include "dnd.h"
+#include "window.h"
 #include "widget.h"
 #include "widget_types.h"
 
@@ -159,8 +160,8 @@ void xitk_filebrowser_hide(xitk_widget_t *w) {
     if(private_data->visible) {
       XLOCK(private_data->imlibdata->x.disp);
       XUnmapWindow(private_data->imlibdata->x.disp, private_data->window);
-      xitk_hide_widgets(private_data->widget_list);
       XUNLOCK(private_data->imlibdata->x.disp);
+      xitk_hide_widgets(private_data->widget_list);
       private_data->visible = 0;
     }
   }
@@ -175,8 +176,8 @@ void xitk_filebrowser_show(xitk_widget_t *w) {
   if(w && (w->widget_type & WIDGET_TYPE_FILEBROWSER)) {
     private_data = (filebrowser_private_data_t *)w->private_data;
 
-    XLOCK(private_data->imlibdata->x.disp);
     xitk_show_widgets(private_data->widget_list);
+    XLOCK(private_data->imlibdata->x.disp);
     XMapRaised(private_data->imlibdata->x.disp, private_data->window); 
     XUNLOCK(private_data->imlibdata->x.disp);
     private_data->visible = 1;
@@ -935,33 +936,52 @@ static void notify_destroy(xitk_widget_t *w, void *data) {
  */
 void xitk_filebrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
   filebrowser_private_data_t *private_data;
+  ImlibImage                 *new_img, *old_img;
+  XSizeHints                  hint;
 
   if(w && (w->widget_type & WIDGET_TYPE_FILEBROWSER)) {
     private_data = (filebrowser_private_data_t *)w->private_data;
     
+    xitk_skin_lock(skonfig);
+    xitk_hide_widgets(private_data->widget_list);
+
     XLOCK(private_data->imlibdata->x.disp);
     
-    Imlib_destroy_image(private_data->imlibdata, private_data->bg_image);
-    
-    if(!(private_data->bg_image = 
-	 Imlib_load_image(private_data->imlibdata,
-			  xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name)))) {
+    if(!(new_img = Imlib_load_image(private_data->imlibdata,
+				    xitk_skin_get_skin_filename(skonfig,
+							private_data->skin_element_name)))) {
       XITK_DIE("%s(): couldn't find image for background\n", __FUNCTION__);
     }
     
+    hint.width  = new_img->rgb_width;
+    hint.height = new_img->rgb_height;
+    hint.flags  = PSize;
+    XSetWMNormalHints(private_data->imlibdata->x.disp, private_data->window, &hint);
+    
     XResizeWindow (private_data->imlibdata->x.disp, private_data->window,
-		   (unsigned int)private_data->bg_image->rgb_width,
-		   (unsigned int)private_data->bg_image->rgb_height);
+		   (unsigned int)new_img->rgb_width,
+		   (unsigned int)new_img->rgb_height);
     
-    /*
-     * We should here, otherwise new skined window will have wrong size.
-     */
-    XFlush(private_data->imlibdata->x.disp);
-    
-    Imlib_apply_image(private_data->imlibdata, private_data->bg_image, private_data->window);
     
     XUNLOCK(private_data->imlibdata->x.disp);
     
+    while(!xitk_is_window_size(private_data->imlibdata->x.disp, private_data->window, 
+			       new_img->rgb_width, new_img->rgb_height)) {
+      xitk_usec_sleep(10000);
+    }
+    
+    old_img = private_data->bg_image;
+    private_data->bg_image = new_img;
+    
+    XLOCK(private_data->imlibdata->x.disp);
+    
+    Imlib_destroy_image(private_data->imlibdata, old_img);
+    Imlib_apply_image(private_data->imlibdata, new_img, private_data->window);
+
+    XUNLOCK(private_data->imlibdata->x.disp);
+    
+    xitk_skin_unlock(skonfig);
+
     xitk_change_skins_widget_list(private_data->widget_list, skonfig);
     
     xitk_paint_widget_list(private_data->widget_list);
@@ -1051,16 +1071,14 @@ xitk_widget_t *xitk_filebrowser_create(xitk_skin_config_t *skonfig, xitk_filebro
 		   hint.x, hint.y, hint.width, 
 		   hint.height, 0, 
 		   fb->imlibdata->x.depth, 
-		   CopyFromParent, 
+		   InputOutput, 
 		   fb->imlibdata->x.visual,
 		   CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect, &attr);
   
   XSetStandardProperties(fb->imlibdata->x.disp, private_data->window, title, title,
 			 None, NULL, 0, &hint);
 
-  XSelectInput(fb->imlibdata->x.disp, private_data->window,
-	       ButtonPressMask | ButtonReleaseMask | KeymapStateMask | PointerMotionMask 
-	       | KeyPressMask | ExposureMask | StructureNotifyMask);
+  XSelectInput(fb->imlibdata->x.disp, private_data->window, INPUT_MOTION | KeymapStateMask);
   
   /*
    * layer above most other things, like gnome panel
@@ -1111,6 +1129,8 @@ xitk_widget_t *xitk_filebrowser_create(xitk_skin_config_t *skonfig, xitk_filebro
   Imlib_apply_image(fb->imlibdata, 
 		    private_data->bg_image, private_data->window);
 
+  XUNLOCK(fb->imlibdata->x.disp);
+
   private_data->widget_list                = xitk_widget_list_new() ;
   private_data->widget_list->l             = xitk_list_new ();
   private_data->widget_list->focusedWidget = NULL;
@@ -1124,7 +1144,6 @@ xitk_widget_t *xitk_filebrowser_create(xitk_skin_config_t *skonfig, xitk_filebro
 
   lbl.imlibdata        = fb->imlibdata;
 
-
   lb.button_type       = CLICK_BUTTON;
   lb.label             = fb->homedir.caption;
   lb.callback          = filebrowser_homedir;
@@ -1230,7 +1249,9 @@ xitk_widget_t *xitk_filebrowser_create(xitk_skin_config_t *skonfig, xitk_filebro
 			   private_data->fc->dir_disp_contents,
 			   private_data->dir_contents_num, 0);
 
+  XLOCK(fb->imlibdata->x.disp);
   XMapRaised(fb->imlibdata->x.disp, private_data->window); 
+  XUNLOCK(fb->imlibdata->x.disp);
 
   private_data->widget_key = 
     xitk_register_event_handler("file browser",
@@ -1241,8 +1262,6 @@ xitk_widget_t *xitk_filebrowser_create(xitk_skin_config_t *skonfig, xitk_filebro
 				private_data->widget_list,
 				(void *) private_data);
 
-  XUNLOCK (fb->imlibdata->x.disp);
-
   return mywidget;
 }
 
