@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: image.c,v 1.21 2001/12/17 22:30:01 f1rmb Exp $
+ * $Id: image.c,v 1.26 2002/04/12 21:38:51 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -47,15 +47,16 @@ unsigned int xitk_get_pixel_color_from_rgb(ImlibData *im, int r, int g, int b) {
   
   xcolor.flags = DoRed | DoBlue | DoGreen;
   
-  XLOCK(im->x.disp);
-  
   xcolor.red   = r<<8;
   xcolor.green = g<<8;
   xcolor.blue  = b<<8;
+
+  XLOCK(im->x.disp);
   XAllocColor(im->x.disp, Imlib_get_colormap(im), &xcolor);
+  XUNLOCK(im->x.disp);
+
   pixcol = xcolor.pixel;
   
-  XUNLOCK(im->x.disp);
 
   return pixcol;
 }
@@ -321,6 +322,8 @@ xitk_image_t *xitk_image_create_image_with_colors_from_string(ImlibData *im,
 
     XLOCK(im->x.disp);
     XSetForeground(im->x.disp, gc, foreground);
+    XUNLOCK(im->x.disp);
+
     j = height;
     for(i = 0; i < numlines; i++, j += (height + 3)) {
       length = xitk_font_get_string_length(fs, lines[i]);
@@ -332,10 +335,12 @@ xitk_image_t *xitk_image_create_image_with_colors_from_string(ImlibData *im,
       else if(align == ALIGN_RIGHT)
 	x = (width - length);
       
+      XLOCK(im->x.disp);
       XDrawString(im->x.disp, image->image, gc, x, (j - descent), lines[i], strlen(lines[i]));
+      XUNLOCK(im->x.disp);
+	
       XITK_FREE(lines[i]);
     }
-    XUNLOCK(im->x.disp);
   }
 
   xitk_font_unload_font(fs);
@@ -1002,6 +1007,103 @@ void draw_tab(ImlibData *im, xitk_image_t *p) {
   XUNLOCK(im->x.disp);
 }
 
+/*
+ *
+ */
+void draw_paddle_rotate(ImlibData *im, xitk_image_t *p) {
+  GC            gc, mgc;
+  XGCValues     gcv;
+  int           w = p->width/3;
+  int           h = p->height;
+  unsigned int  red, yellow, gray;
+
+  assert(im && p);
+  
+  red    = xitk_get_pixel_color_from_rgb(im, 255, 0, 0);
+  yellow = xitk_get_pixel_color_from_rgb(im, 255, 255, 0);
+  gray   = xitk_get_pixel_color_darkgray(im);
+  
+  XLOCK(im->x.disp);
+  
+  gcv.graphics_exposures = False;
+  gc = XCreateGC(im->x.disp, p->image, GCGraphicsExposures, &gcv);
+  
+  gcv.graphics_exposures = False;
+  mgc = XCreateGC(im->x.disp, p->mask, GCGraphicsExposures, &gcv);
+  
+  {
+    int x, i;
+    unsigned int bg_colors[3] = { gray, yellow, red };
+    
+    XSetForeground(im->x.disp, mgc, 0);
+    XFillRectangle(im->x.disp, p->mask, mgc, 0, 0, w * 3 , h);
+    
+    XSetForeground(im->x.disp, mgc, 1);
+
+    for(x = 0, i = 0; i < 3; i++) {
+      XSetForeground(im->x.disp, mgc, 1);
+      XFillArc(im->x.disp, p->mask, mgc, x, 0, w-1, h-1, (0 * 64), (360 * 64));
+      XDrawArc(im->x.disp, p->mask, mgc, x, 0, w-1, h-1, (0 * 64), (360 * 64));
+      
+      XSetForeground(im->x.disp, gc, bg_colors[i]);
+      XFillArc(im->x.disp, p->image, gc, x, 0, w-1, h-1, (0 * 64), (360 * 64));
+      XDrawArc(im->x.disp, p->image, gc, x, 0, w-1, h-1, (0 * 64), (360 * 64));
+      XSetForeground(im->x.disp, gc, xitk_get_pixel_color_black(im));
+      XDrawArc(im->x.disp, p->image, gc, x, 0, w-1, h-1, (0 * 64), (360 * 64));
+
+      x += w;
+    }
+  }
+
+  XFreeGC(im->x.disp, mgc);
+  XFreeGC(im->x.disp, gc);
+  XUNLOCK(im->x.disp);
+}
+
+/*
+ *
+ */
+void draw_rotate_button(ImlibData *im, xitk_image_t *p) {
+  GC            gc, mgc;
+  XGCValues     gcv;
+  int           w = p->width;
+  int           h = p->height;
+  
+  assert(im && p);
+  
+  XLOCK(im->x.disp);
+
+  gcv.graphics_exposures = False;
+  gc = XCreateGC(im->x.disp, p->image, GCGraphicsExposures, &gcv);
+
+  gcv.graphics_exposures = False;
+  mgc = XCreateGC(im->x.disp, p->mask, GCGraphicsExposures, &gcv);
+
+  /* Draw mask */
+  XSetForeground(im->x.disp, mgc, 0);
+  XFillRectangle(im->x.disp, p->mask, mgc, 0, 0, w , h);
+  
+  XSetForeground(im->x.disp, mgc, 1);
+  XFillArc(im->x.disp, p->mask, mgc, 0, 0, w-1, h-1, (0 * 64), (360 * 64));
+  
+  /* */
+  XSetForeground(im->x.disp, gc, xitk_get_pixel_color_gray(im));
+  XFillArc(im->x.disp, p->image, gc, 0, 0, w-1, h-1, (0 * 64), (360 * 64));
+
+  XSetForeground(im->x.disp, gc, xitk_get_pixel_color_white(im));
+  //  XDrawArc(im->x.disp, p->image, gc, 0, 0, w-1, h-1, (30 * 64), (180 * 64));
+  XDrawArc(im->x.disp, p->image, gc, 1, 1, w-2, h-2, (30 * 64), (180 * 64));
+
+  XSetForeground(im->x.disp, gc, xitk_get_pixel_color_darkgray(im));
+  //  XDrawArc(im->x.disp, p->image, gc, 0, 0, w-1, h-1, (210 * 64), (180 * 64));
+  XDrawArc(im->x.disp, p->image, gc, 1, 1, w-3, h-3, (210 * 64), (180 * 64));
+  
+  XFreeGC(im->x.disp, mgc);
+  XFreeGC(im->x.disp, gc);
+  XUNLOCK(im->x.disp);
+}
+
+
 /*
  *
  */
@@ -1024,14 +1126,12 @@ xitk_image_t *xitk_image_load_image(ImlibData *im, char *image) {
   }
   
   Imlib_render (im, img, img->rgb_width, img->rgb_height);
-  XUNLOCK(im->x.disp);
   
   i->image  = Imlib_copy_image(im, img);
   i->mask   = Imlib_copy_mask(im, img);
   i->width  = img->rgb_width;
   i->height = img->rgb_height;
   
-  XLOCK(im->x.disp);
   Imlib_destroy_image(im, img);
   XUNLOCK(im->x.disp);
   
@@ -1088,7 +1188,7 @@ static void paint_image (xitk_widget_t *i, Window win, GC gc) {
   image_private_data_t *private_data = 
     (image_private_data_t *) i->private_data;
   
-  if ((i->widget_type & WIDGET_TYPE_IMAGE) && i->visible) {
+  if ((i->widget_type & WIDGET_TYPE_IMAGE) && (i->visible == 1)) {
 
     skin = private_data->skin;
     
@@ -1120,9 +1220,11 @@ static void notify_change_skin(xitk_widget_list_t *wl,
   image_private_data_t *private_data = 
     (image_private_data_t *) i->private_data;
   
-  if ((i->widget_type & WIDGET_TYPE_IMAGE) && i->visible) {
+  if ((i->widget_type & WIDGET_TYPE_IMAGE) && (i->visible == 1)) {
     if(private_data->skin_element_name) {
       
+      xitk_skin_lock(skonfig);
+
       XITK_FREE_XITK_IMAGE(private_data->imlibdata->x.disp, private_data->skin);
       private_data->skin = xitk_image_load_image(private_data->imlibdata,
 						 xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
@@ -1131,6 +1233,8 @@ static void notify_change_skin(xitk_widget_list_t *wl,
       i->y               = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
       i->width           = private_data->skin->width;
       i->height          = private_data->skin->height;
+
+      xitk_skin_unlock(skonfig);
       
       xitk_set_widget_pos(i, i->x, i->y);
     }
