@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: inputtext.c,v 1.20 2002/01/15 22:22:36 f1rmb Exp $
+ * $Id: inputtext.c,v 1.27 2002/04/18 21:14:36 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -86,7 +86,7 @@ static int notify_inside(xitk_widget_t *it, int x, int y) {
     (inputtext_private_data_t *) it->private_data;
   
   if(it->widget_type & WIDGET_TYPE_INPUTTEXT) {
-    if(it->visible) {
+    if((it->visible == 1)) {
       xitk_image_t *skin = private_data->skin;
       
       return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, it, skin->mask, x, y);
@@ -208,10 +208,10 @@ static void inputtext_recalc_offsets(xitk_widget_list_t *wl,
 /*
  * Return a pixmap with text drawed.
  */
-static Pixmap create_labelofinputtext(xitk_widget_t *it, 
-				      Window win, GC gc, Pixmap pix, 
-				      int xsize, int ysize, 
-				      char *label, int state) {
+static void create_labelofinputtext(xitk_widget_t *it, 
+				    Window win, GC gc, Pixmap pix, 
+				    int xsize, int ysize, 
+				    char *label, int state) {
   inputtext_private_data_t *private_data = 
     (inputtext_private_data_t *) it->private_data;
   char               *plabel = label;
@@ -316,6 +316,7 @@ static Pixmap create_labelofinputtext(xitk_widget_t *it,
   XDrawString(private_data->imlibdata->x.disp, pix, gc, 
 	      2, ((ysize+asc+des+yoff)>>1)-des, 
 	      plabel, strlen(plabel));
+  XUNLOCK(private_data->imlibdata->x.disp);
 
   width = xitk_font_get_text_width(fs, plabel,
 				   (private_data->cursor_pos - private_data->disp_offset));
@@ -323,6 +324,7 @@ static Pixmap create_labelofinputtext(xitk_widget_t *it,
   /* Draw cursor pointer */
   if(private_data->cursor_pos >= 0) {
     
+    XLOCK(private_data->imlibdata->x.disp);
     XDrawLine(private_data->imlibdata->x.disp, pix, gc,
 	      width + 1, 2, width + 3, 2);
 
@@ -331,10 +333,8 @@ static Pixmap create_labelofinputtext(xitk_widget_t *it,
 
     XDrawLine(private_data->imlibdata->x.disp, pix, gc, 
 	      width + 1, ysize - 3, width + 3, ysize - 3);
-
+    XUNLOCK(private_data->imlibdata->x.disp);
   }
-  
-  XUNLOCK(private_data->imlibdata->x.disp);
 
   xitk_font_unload_font(fs);
 
@@ -343,7 +343,6 @@ static Pixmap create_labelofinputtext(xitk_widget_t *it,
     XITK_FREE(gColor);
   }
 
-  return pix;
 }
 
 /*
@@ -354,12 +353,13 @@ static void paint_inputtext(xitk_widget_t *it, Window win, GC gc) {
     (inputtext_private_data_t *) it->private_data;
   int                 button_width, state = 0;
   xitk_image_t        *skin;
-  Pixmap              btn, bgtmp;
+  Pixmap              btn;
   GC                  lgc;
   XWindowAttributes   attr;
 
-  if ((it->widget_type & WIDGET_TYPE_INPUTTEXT) && it->visible) {
+  if ((it->widget_type & WIDGET_TYPE_INPUTTEXT) && (it->visible == 1)) {
         
+    XLOCK(private_data->imlibdata->x.disp);
     XGetWindowAttributes(private_data->imlibdata->x.disp, win, &attr);
     
     skin = private_data->skin;
@@ -374,37 +374,33 @@ static void paint_inputtext(xitk_widget_t *it, Window win, GC gc) {
 
     button_width = skin->width/2;
     
-    bgtmp = XCreatePixmap(private_data->imlibdata->x.disp, skin->image,
-			  button_width, skin->height, attr.depth);
-    
-    XLOCK(private_data->imlibdata->x.disp);
+    btn = XCreatePixmap(private_data->imlibdata->x.disp, skin->image,
+			button_width, skin->height, attr.depth);
     
     if(private_data->have_focus) {
       state = FOCUS;
       XCopyArea (private_data->imlibdata->x.disp, skin->image,
-		 bgtmp, gc, button_width, 0,
+		 btn, gc, button_width, 0,
 		 button_width, skin->height, 0, 0);
     }
     else {
       state = NORMAL;
       XCopyArea (private_data->imlibdata->x.disp, skin->image,
-		 bgtmp, gc, 0, 0,
+		 btn, gc, 0, 0,
 		 button_width, skin->height, 0, 0);
     }
     
-
-    btn = create_labelofinputtext(it, win, gc, bgtmp,
-				  button_width, skin->height, 
-				  private_data->text, state);
+    XUNLOCK(private_data->imlibdata->x.disp);
     
+    create_labelofinputtext(it, win, gc, btn,
+			    button_width, skin->height, 
+			    private_data->text, state);
+    
+    XLOCK(private_data->imlibdata->x.disp);
     XCopyArea (private_data->imlibdata->x.disp, btn, win, lgc, 0, 0,
 	       button_width, skin->height, it->x, it->y);
-
-
-    XFreePixmap(private_data->imlibdata->x.disp, bgtmp);
-
+    XFreePixmap(private_data->imlibdata->x.disp, btn);
     XFreeGC(private_data->imlibdata->x.disp, lgc);
-
     XUNLOCK(private_data->imlibdata->x.disp);
   }
 
@@ -500,8 +496,11 @@ static void notify_change_skin(xitk_widget_list_t *wl,
   
   if (it->widget_type & WIDGET_TYPE_INPUTTEXT) {
     if(private_data->skin_element_name) {
+
+      xitk_skin_lock(skonfig);
+
       XITK_FREE(private_data->fontname);
-      private_data->fontname      = xitk_skin_get_label_fontname(skonfig, private_data->skin_element_name);
+      private_data->fontname      = strdup(xitk_skin_get_label_fontname(skonfig, private_data->skin_element_name));
       
       XITK_FREE_XITK_IMAGE(private_data->imlibdata->x.disp, private_data->skin);
       private_data->skin          = xitk_image_load_image(private_data->imlibdata, 
@@ -510,15 +509,19 @@ static void notify_change_skin(xitk_widget_list_t *wl,
       private_data->max_visible   = (private_data->skin->width/2);
       
       XITK_FREE(private_data->normal_color);
-      private_data->normal_color  = xitk_skin_get_label_color(skonfig, private_data->skin_element_name);
+      private_data->normal_color  = strdup(xitk_skin_get_label_color(skonfig, private_data->skin_element_name));
       XITK_FREE(private_data->focused_color);
-      private_data->focused_color = xitk_skin_get_label_color_focus(skonfig, private_data->skin_element_name);
+      private_data->focused_color = strdup(xitk_skin_get_label_color_focus(skonfig, private_data->skin_element_name));
       
       it->x                       = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
       it->y                       = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
       it->width                   = private_data->skin->width/2;
       it->height                  = private_data->skin->height;
-      
+      it->visible                 = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1 : -1;
+      it->enable                  = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
+     
+      xitk_skin_unlock(skonfig);
+
       xitk_set_widget_pos(it, it->x, it->y);
     }
   }
@@ -697,8 +700,7 @@ static void inputtext_exec_return(xitk_widget_list_t *wl, xitk_widget_t *it) {
   
   if(strlen(private_data->text) > 0) {
     if(private_data->callback)
-      private_data->callback(it, 
-			     private_data->userdata, private_data->text);
+      private_data->callback(it, private_data->userdata, private_data->text);
   }
 }
 
@@ -971,7 +973,8 @@ static xitk_widget_t *_xitk_inputtext_create (xitk_skin_config_t *skonfig,
 					      int x, int y, char *skin_element_name,
 					      xitk_image_t *skin,
 					      char *fontname,
-					      char *ncolor, char *fcolor) {
+					      char *ncolor, char *fcolor,
+					      int visible, int enable) {
   xitk_widget_t             *mywidget;
   inputtext_private_data_t  *private_data;
   
@@ -1007,9 +1010,9 @@ static xitk_widget_t *_xitk_inputtext_create (xitk_skin_config_t *skonfig,
 
   mywidget->private_data          = private_data;
 
-  mywidget->enable                = 1;
+  mywidget->enable                = enable;
   mywidget->running               = 1;
-  mywidget->visible               = 1;
+  mywidget->visible               = visible;
   mywidget->have_focus            = FOCUS_LOST;
   mywidget->imlibdata             = private_data->imlibdata;
   mywidget->x                     = x;
@@ -1046,7 +1049,9 @@ xitk_widget_t *xitk_inputtext_create (xitk_skin_config_t *skonfig, xitk_inputtex
 							xitk_skin_get_skin_filename(skonfig, it->skin_element_name))),
 				 (xitk_skin_get_label_fontname(skonfig, it->skin_element_name)),
 				 (xitk_skin_get_label_color(skonfig, it->skin_element_name)),
-				 (xitk_skin_get_label_color_focus(skonfig, it->skin_element_name)));
+				 (xitk_skin_get_label_color_focus(skonfig, it->skin_element_name)),
+				 ((xitk_skin_get_visibility(skonfig, it->skin_element_name)) ? 1 : -1),
+				 (xitk_skin_get_enability(skonfig, it->skin_element_name)));
 }
 
 /*
@@ -1062,6 +1067,5 @@ xitk_widget_t *xitk_noskin_inputtext_create (xitk_inputtext_widget_t *it,
   draw_bevel_two_state(it->imlibdata, i);
   
 
-  return _xitk_inputtext_create(NULL, it, x, y, NULL, i, fontname, ncolor, fcolor);
+  return _xitk_inputtext_create(NULL, it, x, y, NULL, i, fontname, ncolor, fcolor, 1, 1);
 }
-
