@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: label.c,v 1.23 2002/01/15 22:22:36 f1rmb Exp $
+ * $Id: label.c,v 1.29 2002/04/21 22:17:12 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -39,26 +39,97 @@
 #include "utils.h"
 #include "_xitk.h"
 
+static void _create_label_pixmap(xitk_widget_t *l) {
+  label_private_data_t  *private_data = (label_private_data_t *) l->private_data;
+  xitk_image_t          *font         = (xitk_image_t *) private_data->font;
+  int                    pixwidth;
+  char                  *_label;
+  int                    x_dest, i;
+  int                    len;
+  
+  private_data->anim_offset = 0;
+
+  if(private_data->animation) {
+    _label = (char *) alloca((strlen(private_data->label) * 2) + 5 + 1);
+    
+    if((strlen(private_data->label)) > private_data->length)
+      sprintf(_label, "%s *** %s", private_data->label, private_data->label);
+    else
+      sprintf(_label, "%s", private_data->label);
+    
+  }
+  else
+    xitk_strdupa(_label, private_data->label);
+  
+  len = strlen(_label);
+  pixwidth = private_data->char_length * 
+    ((private_data->length * ((len / private_data->length) + 1)) + 5);
+
+  XLOCK (private_data->imlibdata->x.disp);
+  private_data->labelpix = XCreatePixmap(private_data->imlibdata->x.disp, 
+					 private_data->imlibdata->x.base_window, 
+					 (pixwidth) ? pixwidth : 1, private_data->char_height, 
+					 private_data->imlibdata->x.depth);
+  XUNLOCK (private_data->imlibdata->x.disp);
+
+  x_dest = 0;
+    
+  for (i = 0; i < pixwidth; i++) {
+    int c = 0;
+    
+    if ((i < len) && (_label[i] >= 32))
+      c = _label[i] - 32;
+    
+    if (c >= 0) {
+      int px, py;
+      
+      px = (c % 32) * private_data->char_length;
+      py = (c / 32) * private_data->char_height;
+      
+      XLOCK(private_data->imlibdata->x.disp);
+      XCopyArea(private_data->imlibdata->x.disp, font->image, 
+		private_data->labelpix, private_data->gc, px, py,
+		private_data->char_length, private_data->char_height, x_dest, 0);
+      XUNLOCK(private_data->imlibdata->x.disp);
+     
+    }
+    
+    x_dest += private_data->char_length;
+  }
+
+}
+
 /*
  *
  */
 static void notify_destroy(xitk_widget_t *w, void *data) {
-  label_private_data_t *private_data = (label_private_data_t *) w->private_data;
+  label_private_data_t *private_data;
 
   if(w->widget_type & WIDGET_TYPE_LABEL) {
+    private_data = (label_private_data_t *) w->private_data;
 
+    private_data->on_change = 1;
+    
     if(private_data->anim_running) {
       void *dummy;
       
       private_data->anim_running = 0;
       pthread_join(private_data->thread, &dummy);
     }
-
-    XITK_FREE(private_data->animated_label);
+    
+    if (private_data->labelpix != None) {
+      XLOCK (private_data->imlibdata->x.disp);
+      XFreePixmap(private_data->imlibdata->x.disp, private_data->labelpix);
+      private_data->labelpix = None;
+      XUNLOCK (private_data->imlibdata->x.disp);
+    }
+    
     XITK_FREE(private_data->fontname);
     XITK_FREE(private_data->skin_element_name);
+
     if(private_data->font)
       xitk_image_free_image(private_data->imlibdata, &private_data->font);
+
     XITK_FREE(private_data);
   }
 }
@@ -67,10 +138,10 @@ static void notify_destroy(xitk_widget_t *w, void *data) {
  *
  */
 static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
-  label_private_data_t *private_data = 
-    (label_private_data_t *) w->private_data;
+  label_private_data_t *private_data;
   
   if(w->widget_type & WIDGET_TYPE_LABEL) {
+    private_data = (label_private_data_t *) w->private_data;
     if(sk == FOREGROUND_SKIN && private_data->font) {
       return private_data->font;
     }
@@ -97,13 +168,13 @@ char *xitk_label_get_label(xitk_widget_t *w) {
  *
  */
 static void paint_label(xitk_widget_t *l, Window win, GC gc) {
-  label_private_data_t *private_data = 
-    (label_private_data_t *) l->private_data;
-  xitk_image_t          *font = (xitk_image_t *) private_data->font;
-  int                   x_dest, y_dest, nCWidth, nCHeight, len, i;
-  char                 *label_to_display;
+  label_private_data_t  *private_data;
+  xitk_image_t          *font;
 
-  if ((l->widget_type & WIDGET_TYPE_LABEL) && l->visible) {
+  if ((l->widget_type & WIDGET_TYPE_LABEL) && (l->visible == 1)) {
+    
+    private_data = (label_private_data_t *) l->private_data;
+    font = (xitk_image_t *) private_data->font;
 
     /* non skinable widget */
     if(private_data->skin_element_name == NULL) {
@@ -138,52 +209,21 @@ static void paint_label(xitk_widget_t *l, Window win, GC gc) {
       xitk_image_free_image(private_data->imlibdata, &bg);
       
       xitk_font_unload_font(fs);
-
+      
       return;
     }
-
-    pthread_mutex_lock(&private_data->mutex);
-
-    if (private_data->anim_running) {
-      label_to_display = 
-	&private_data->animated_label[private_data->anim_offset];
-      len = private_data->length;
-    } 
     else {
-      label_to_display = private_data->label;
-      len = strlen(label_to_display);
+      int width = private_data->char_length * private_data->length;
+      
+      XLOCK(private_data->imlibdata->x.disp);
+      XCopyArea(private_data->imlibdata->x.disp, private_data->labelpix, win, gc, 
+		private_data->anim_offset, 0, 
+		width, private_data->char_height, 
+		l->x, l->y);
+      XUNLOCK(private_data->imlibdata->x.disp);
+      
     }
-    pthread_mutex_unlock(&private_data->mutex);
-    
-    x_dest = l->x;
-    y_dest = l->y;
-    
-    nCWidth = font->width / 32;
-    nCHeight = font->height / 3;
-    
-    for (i=0; i<private_data->length; i++) {
-      int c=0;
-      
-      if ((i<len) && (label_to_display[i] >= 32))
-	c = label_to_display[i]-32;
-      
-      if (c>=0) {
-	int px, py;
-	
-	px = (c % 32) * nCWidth;
-	py = (c / 32) * nCHeight;
-	
-	XLOCK (private_data->imlibdata->x.disp);
-	XCopyArea (private_data->imlibdata->x.disp, font->image, win, gc, px, py,
-		   nCWidth, nCHeight, x_dest, y_dest);
-	XUNLOCK (private_data->imlibdata->x.disp);
-	
-      }
-      
-      x_dest += nCWidth;
-    }
-
-   }
+  }
 }
 
 /*
@@ -191,27 +231,35 @@ static void paint_label(xitk_widget_t *l, Window win, GC gc) {
  */
 void *xitk_label_animation_loop(void *data) {
   label_private_data_t *private_data = (label_private_data_t *)data;
-  xitk_widget_t             *w = private_data->lWidget;
-
+  xitk_widget_t        *w            = private_data->lWidget;
+  unsigned long         t_anim       = xitk_get_timer_label_animation();
+  
   do {
     
-    if(w->visible) {
-
+    if((w->visible == 1)) {
+      
       private_data->anim_offset++;
-      if (private_data->anim_offset>(strlen(private_data->label) + 4))
-	private_data->anim_offset = 0;
-      
-      paint_label(private_data->lWidget,
-		  private_data->window, private_data->gc);
-      
-      /* We can't wait here, otherwise the rolling effect is really jerky */
-      XLOCK (private_data->imlibdata->x.disp);
-      XSync(private_data->imlibdata->x.disp, False);
-      XUNLOCK (private_data->imlibdata->x.disp);
-      
-    }
 
-    xitk_usec_sleep(400000);
+      if (private_data->anim_offset >
+	  (private_data->char_length * (strlen(private_data->label) + 5)))
+	private_data->anim_offset = 1;
+
+      /* 
+       * Label will change sooner, don't try to paint it till the change,
+       * otherwise a deadlock will happened.
+       */
+      if(!private_data->on_change) {
+	paint_label(private_data->lWidget,
+		    private_data->window, private_data->gc);
+	
+	/* We can't wait here, otherwise the rolling effect is really jerky */
+	XLOCK (private_data->imlibdata->x.disp);
+	XSync(private_data->imlibdata->x.disp, False);
+	XUNLOCK (private_data->imlibdata->x.disp);
+      }
+    }
+    
+    xitk_usec_sleep(t_anim);
     
   } while(w->running && private_data->anim_running);
   
@@ -222,77 +270,62 @@ void *xitk_label_animation_loop(void *data) {
  *
  */
 static void label_setup_label(xitk_widget_t *l, char *label_) {
-  label_private_data_t *private_data = 
-    (label_private_data_t *) l->private_data;
+  label_private_data_t *private_data = (label_private_data_t *) l->private_data;
   int label_len;
-
-  pthread_mutex_lock(&private_data->mutex);
-
+  
+  /* Inform animation thread to not paint the label */
+  private_data->on_change = 1;
+  
   if (private_data->label) {
     XITK_FREE(private_data->label);
-    private_data->label = NULL;
   }
   
-  if (private_data->animated_label) {
-    XITK_FREE(private_data->animated_label);
-    private_data->animated_label = NULL;
+  if (private_data->labelpix != None) {
+    XLOCK (private_data->imlibdata->x.disp);
+    XFreePixmap(private_data->imlibdata->x.disp, private_data->labelpix);
+    XUNLOCK (private_data->imlibdata->x.disp);
+    private_data->labelpix = None;
   }
 
   label_len = strlen(label_);
 
   private_data->label = strdup((label_ != NULL) ? label_ : "");
 
-  /*
-  private_data->label = (char *) xitk_xmalloc(label_len + 1);
-  strncpy (private_data->label, label_, label_len);
-  private_data->label[label_len] = 0;
-  */
+  if(private_data->skin_element_name != NULL)
+    _create_label_pixmap(l);
+  else
+    private_data->anim_offset = 0;
 
   if (private_data->animation) {
-
+    
     if (private_data->anim_running) {
       void *dummy;
-
+      
       private_data->anim_running = 0;
-      /*
-       * xitk_label_animation_loop may be already running due to a call
-       * from gui_dndcallback, inside mrl_add_and_play. paint_label will
-       * try to lock this mutex and will block. If we dont unlock it here,
-       * we will block too waiting for the join... Deadlock!
-       */
-      pthread_mutex_unlock(&private_data->mutex);
       pthread_join (private_data->thread, &dummy);
-      pthread_mutex_lock(&private_data->mutex);
     }
-
+    
     if (label_len > private_data->length) {
       pthread_attr_t       pth_attrs;
       struct sched_param   pth_params;
-
-      private_data->anim_running = 1;
-
-      private_data->animated_label = (char *) 
-	xitk_xmalloc(2 * label_len + 11);
-
-      sprintf(private_data->animated_label, "%s *** %s *** ", label_, label_) ;
       
-      private_data->anim_offset = 0;
-
+      private_data->anim_running = 1;
+      
       pthread_attr_init(&pth_attrs);
-
+      
       /* this won't work on linux, freebsd 5.0 */
       pthread_attr_getschedparam(&pth_attrs, &pth_params);
       pth_params.sched_priority = sched_get_priority_min(SCHED_OTHER);
       pthread_attr_setschedparam(&pth_attrs, &pth_params);
-
+      
       pthread_create(&private_data->thread, &pth_attrs, 
 		     xitk_label_animation_loop, (void *)private_data);
-
+      
       pthread_attr_destroy(&pth_attrs);
     }
   }
   
-  pthread_mutex_unlock(&private_data->mutex);
+  private_data->on_change = 0;
 }
 
 /*
@@ -305,6 +338,9 @@ static void notify_change_skin(xitk_widget_list_t *wl,
   
   if(l->widget_type & WIDGET_TYPE_LABEL) {
     if(private_data->skin_element_name) {
+
+      xitk_skin_lock(skonfig);
+
       XITK_FREE_XITK_IMAGE(private_data->imlibdata->x.disp, private_data->font);
       private_data->font        = xitk_image_load_image(private_data->imlibdata, 
 							xitk_skin_get_label_skinfont_filename(skonfig, private_data->skin_element_name));
@@ -318,7 +354,14 @@ static void notify_change_skin(xitk_widget_list_t *wl,
       l->y                      = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
       l->width                  = private_data->char_length * private_data->length;
       l->height                 = private_data->char_height;
+      l->visible                = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1 : -1;
+      l->enable                 = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
       
+      xitk_skin_unlock(skonfig);
+
+      if(private_data->skin_element_name != NULL)
+	_create_label_pixmap(l);
+
       xitk_set_widget_pos(l, l->x, l->y);
     }
   }
@@ -346,7 +389,8 @@ int xitk_label_change_label (xitk_widget_list_t *wl, xitk_widget_t *l, char *new
  */
 static xitk_widget_t *_xitk_label_create(xitk_skin_config_t *skonfig, xitk_label_widget_t *l,
 					 int x, int y, int width, int height,
-					 char *skin_element_name, char *fontname) {
+					 char *skin_element_name, char *fontname,
+					 int visible, int enable) {
   xitk_widget_t          *mywidget;
   label_private_data_t   *private_data;
   
@@ -381,16 +425,17 @@ static xitk_widget_t *_xitk_label_create(xitk_skin_config_t *skonfig, xitk_label
 
   private_data->label          = NULL;
   private_data->animation      = (skin_element_name == NULL) ? 0 : xitk_skin_get_label_animation(skonfig, private_data->skin_element_name);
-  private_data->animated_label = NULL;
   private_data->anim_running   = 0;
   private_data->window         = l->window;
   private_data->gc             = l->gc;
 
+  private_data->labelpix       = None;
+
   mywidget->private_data       = private_data;
 
-  mywidget->enable             = 1;
+  mywidget->enable             = enable;
   mywidget->running            = 1;
-  mywidget->visible            = 1;
+  mywidget->visible            = visible;
   mywidget->have_focus         = FOCUS_LOST;
   mywidget->imlibdata          = private_data->imlibdata;
   mywidget->x                  = x;
@@ -417,8 +462,6 @@ static xitk_widget_t *_xitk_label_create(xitk_skin_config_t *skonfig, xitk_label
   mywidget->tips_timeout       = 0;
   mywidget->tips_string        = NULL;
 
-  pthread_mutex_init(&private_data->mutex, NULL);
-
   label_setup_label(mywidget, l->label);
 
   return mywidget;
@@ -437,7 +480,9 @@ xitk_widget_t *xitk_label_create(xitk_skin_config_t *skonfig, xitk_label_widget_
 			    (xitk_skin_get_label_length(skonfig, l->skin_element_name)),
 			    -1,
 			    l->skin_element_name, 
-			    NULL);
+			    NULL,
+			    ((xitk_skin_get_visibility(skonfig, l->skin_element_name)) ? 1 : -1),
+			    (xitk_skin_get_enability(skonfig, l->skin_element_name)));
 }
 
 /*
@@ -447,5 +492,5 @@ xitk_widget_t *xitk_noskin_label_create(xitk_label_widget_t *l,
 					int x, int y, int width, int height, char *fontname) {
   XITK_CHECK_CONSTITENCY(l);
 
-  return _xitk_label_create(NULL, l, x, y, width, height, NULL, fontname);
+  return _xitk_label_create(NULL, l, x, y, width, height, NULL, fontname, 1, 1);
 }
