@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: label.h,v 1.10 2001/11/24 00:44:38 f1rmb Exp $
+ * $Id: label.h,v 1.13 2002/04/21 22:17:12 f1rmb Exp $
  *
  */
 
@@ -43,18 +43,20 @@ typedef struct {
   int               char_length; /* length of 1 char */
   int               char_height; /* height of 1 char */
 
+  Pixmap            labelpix;
+
   int               length;      /* length in char */
   xitk_image_t     *font;
   char             *fontname;
   char             *label;
 
   int               animation;
-  char             *animated_label;
   int               anim_running;
   int               anim_offset;
 
+  int               on_change;
+
   pthread_t         thread;
-  pthread_mutex_t   mutex;
 
 } label_private_data_t;
 
