@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: labelbutton.c,v 1.23 2002/01/15 22:22:36 f1rmb Exp $
+ * $Id: labelbutton.c,v 1.30 2002/04/18 23:36:59 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -83,7 +83,7 @@ static int notify_inside(xitk_widget_t *lb, int x, int y) {
     (lbutton_private_data_t *) lb->private_data;
   
   if (lb->widget_type & WIDGET_TYPE_LABELBUTTON) {
-    if(lb->visible) {
+    if((lb->visible == 1)) {
       xitk_image_t *skin = private_data->skin;
       
       return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, lb, skin->mask, x, y);
@@ -98,10 +98,10 @@ static int notify_inside(xitk_widget_t *lb, int x, int y) {
 /*
  * Draw the string in pixmap pix, then return it
  */
-static Pixmap create_labelofbutton(xitk_widget_t *lb, 
-				   Window win, GC gc, Pixmap pix, 
-				   int xsize, int ysize, 
-				   char *label, int state) {
+static void create_labelofbutton(xitk_widget_t *lb, 
+				 Window win, GC gc, Pixmap pix, 
+				 int xsize, int ysize, 
+				 char *label, int state) {
   lbutton_private_data_t  *private_data = 
     (lbutton_private_data_t *) lb->private_data;
   xitk_font_t             *fs = NULL;
@@ -129,8 +129,10 @@ static Pixmap create_labelofbutton(xitk_widget_t *lb,
   /*  Some colors configurations */
   switch(state) {
   case CLICK:
-    xoff = -4;
-    yoff = 1;
+    if(private_data->label_static == 0) {
+      xoff = -4;
+      yoff = 1;
+    }
     if(!strcasecmp(private_data->clickcolor, "Default")) {
       DefaultColor = 255;
     }
@@ -202,7 +204,7 @@ static Pixmap create_labelofbutton(xitk_widget_t *lb,
     XITK_FREE(gColor);
   }
     
-  return pix;
+  //  return pix;
 }
 
 /*
@@ -211,13 +213,13 @@ static Pixmap create_labelofbutton(xitk_widget_t *lb,
 static void paint_labelbutton (xitk_widget_t *lb, Window win, GC gc) {
   lbutton_private_data_t *private_data = 
     (lbutton_private_data_t *) lb->private_data;
-  int          button_width, state = 0;
-  GC           lgc;
-  xitk_image_t *skin;
-  Pixmap       btn, bgtmp;
+  int               button_width, state = 0;
+  GC                lgc;
+  xitk_image_t     *skin;
+  Pixmap            btn;
   XWindowAttributes attr;
 
-  if ((lb->widget_type & WIDGET_TYPE_LABELBUTTON) && lb->visible) {
+  if ((lb->widget_type & WIDGET_TYPE_LABELBUTTON) && (lb->visible == 1)) {
     
     XLOCK(private_data->imlibdata->x.disp);
 
@@ -232,30 +234,30 @@ static void paint_labelbutton (xitk_widget_t *lb, Window win, GC gc) {
       XSetClipOrigin(private_data->imlibdata->x.disp, lgc, lb->x, lb->y);
       XSetClipMask(private_data->imlibdata->x.disp, lgc, skin->mask);
     }
-
+    
     button_width = skin->width / 3;
-    bgtmp = XCreatePixmap(private_data->imlibdata->x.disp, skin->image,
-			  button_width, skin->height, attr.depth);
+    btn = XCreatePixmap(private_data->imlibdata->x.disp, skin->image,
+			button_width, skin->height, attr.depth);
     
     if (private_data->bArmed) {
       if (private_data->bClicked) {
 	state = CLICK;
 	XCopyArea (private_data->imlibdata->x.disp, skin->image, 
-		   bgtmp, gc, 2*button_width, 0,
+		   btn, gc, 2*button_width, 0,
 		   button_width, skin->height, 0, 0);
       }
       else {
 	if(!private_data->bState || private_data->bType == CLICK_BUTTON) {
 	  state = FOCUS;
 	  XCopyArea (private_data->imlibdata->x.disp, skin->image,
-		     bgtmp, gc, button_width, 0,
+		     btn, gc, button_width, 0,
 		     button_width, skin->height, 0, 0);
 	}
 	else {
 	  if(private_data->bType == RADIO_BUTTON) {
 	    state = CLICK;
 	    XCopyArea (private_data->imlibdata->x.disp, skin->image,
-		       bgtmp, gc, 2*button_width, 0,
+		       btn, gc, 2*button_width, 0,
 		       button_width, skin->height, 0, 0);
 	  }
 	}
@@ -265,42 +267,37 @@ static void paint_labelbutton (xitk_widget_t *lb, Window win, GC gc) {
       if(private_data->bState && private_data->bType == RADIO_BUTTON) {
 	if(private_data->bOldState == 1 && private_data->bClicked == 1) {
 	  state = NORMAL;
-	  XCopyArea (private_data->imlibdata->x.disp, skin->image, bgtmp, gc, 0, 0,
+	  XCopyArea (private_data->imlibdata->x.disp, skin->image, btn, gc, 0, 0,
 		     button_width, skin->height, 0, 0);
 	}
 	else {
 	  state = CLICK;
 	  XCopyArea (private_data->imlibdata->x.disp, skin->image, 
-		     bgtmp, gc, 2*button_width, 0,
+		     btn, gc, 2*button_width, 0,
 		     button_width, skin->height, 0, 0);
 	}
       }
       else {
 	state = NORMAL;
-	XCopyArea (private_data->imlibdata->x.disp, skin->image, bgtmp, gc, 0, 0,
+	XCopyArea (private_data->imlibdata->x.disp, skin->image, btn, gc, 0, 0,
 		   button_width, skin->height, 0, 0);
       }
     }
     
     /* Only draw text of label is visible */
-    if(private_data->label_visible) {
-      btn = create_labelofbutton(lb, win, gc, bgtmp,
-				 button_width, skin->height, 
-				 private_data->label, state);
-      XCopyArea (private_data->imlibdata->x.disp, btn, win, lgc, 0, 0,
-		 button_width, skin->height, lb->x, lb->y);
-    }
-    else
-      XCopyArea (private_data->imlibdata->x.disp, bgtmp, win, lgc, 0, 0,
-		 button_width, skin->height, lb->x, lb->y);
-      
-
+    XUNLOCK(private_data->imlibdata->x.disp);
     
-
-    XFreePixmap(private_data->imlibdata->x.disp, bgtmp);
-
+    if(private_data->label_visible) {
+      create_labelofbutton(lb, win, gc, btn,
+			   button_width, skin->height, 
+			   private_data->label, state);
+    }
+    
+    XLOCK(private_data->imlibdata->x.disp);
+    XCopyArea (private_data->imlibdata->x.disp, btn, win, lgc, 0, 0,
+	       button_width, skin->height, lb->x, lb->y);
+    XFreePixmap(private_data->imlibdata->x.disp, btn);
     XFreeGC(private_data->imlibdata->x.disp, lgc);
-
     XUNLOCK(private_data->imlibdata->x.disp);
   }
 
@@ -404,6 +401,9 @@ static void notify_change_skin(xitk_widget_list_t *wl,
   
   if (lb->widget_type & WIDGET_TYPE_LABELBUTTON) {
     if(private_data->skin_element_name) {
+
+      xitk_skin_lock(skonfig);
+
       XITK_FREE_XITK_IMAGE(private_data->imlibdata->x.disp, private_data->skin);
       private_data->skin          = xitk_image_load_image(private_data->imlibdata, 
 		       xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
@@ -418,16 +418,22 @@ static void notify_change_skin(xitk_widget_list_t *wl,
 							   private_data->skin_element_name));
       XITK_FREE(private_data->fontname);
       private_data->fontname      = strdup(xitk_skin_get_label_fontname(skonfig, 
-							private_data->skin_element_name));
+									private_data->skin_element_name));
       private_data->label_visible = xitk_skin_get_label_printable(skonfig, private_data->skin_element_name);
       
+      private_data->label_static  = xitk_skin_get_label_staticity(skonfig, private_data->skin_element_name);
+
       private_data->align         = xitk_skin_get_label_alignment(skonfig, private_data->skin_element_name);
 
       lb->x                    = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
       lb->y                    = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
       lb->width                = private_data->skin->width/3;
       lb->height               = private_data->skin->height;
+      lb->visible              = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1 : -1;
+      lb->enable               = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
       
+      xitk_skin_unlock(skonfig);
+
       xitk_set_widget_pos(lb, lb->x, lb->y);
     }
   }
@@ -511,7 +517,9 @@ static xitk_widget_t *_xitk_labelbutton_create (xitk_skin_config_t *skonfig,
 						int x, int y, 
 						char *skin_element_name, xitk_image_t *skin,
 						char *ncolor, char *fcolor, char *ccolor,
-						char *fontname, int label_visible) {
+						char *fontname, 
+						int label_visible, int label_static,
+						int visible, int enable) {
   xitk_widget_t               *mywidget;
   lbutton_private_data_t *private_data;
   
@@ -534,6 +542,7 @@ static xitk_widget_t *_xitk_labelbutton_create (xitk_skin_config_t *skonfig,
   private_data->userdata          = b->userdata;
   private_data->label             = strdup((b->label)?b->label:"");
   private_data->label_visible     = label_visible;
+  private_data->label_static      = label_static;
 
   private_data->skin              = skin;
   private_data->normcolor         = strdup(ncolor);
@@ -545,9 +554,9 @@ static xitk_widget_t *_xitk_labelbutton_create (xitk_skin_config_t *skonfig,
 
   mywidget->private_data          = private_data;
 
-  mywidget->enable                = 1;
+  mywidget->enable                = enable;
   mywidget->running               = 1;
-  mywidget->visible               = 1;
+  mywidget->visible               = visible;
   mywidget->have_focus            = FOCUS_LOST;
   mywidget->imlibdata             = private_data->imlibdata;
   mywidget->x                     = x;
@@ -586,7 +595,10 @@ xitk_widget_t *xitk_labelbutton_create (xitk_skin_config_t *skonfig, xitk_labelb
 		  (xitk_skin_get_label_color_focus(skonfig, b->skin_element_name)),
 		  (xitk_skin_get_label_color_click(skonfig, b->skin_element_name)),
 		  (xitk_skin_get_label_fontname(skonfig, b->skin_element_name)),
-		  (xitk_skin_get_label_printable(skonfig, b->skin_element_name)));
+		  (xitk_skin_get_label_printable(skonfig, b->skin_element_name)),
+		  (xitk_skin_get_label_staticity(skonfig, b->skin_element_name)),
+		  ((xitk_skin_get_visibility(skonfig, b->skin_element_name)) ? 1 : -1),
+		  (xitk_skin_get_enability(skonfig, b->skin_element_name)));
 }
 
 /*
@@ -603,5 +615,5 @@ xitk_widget_t *xitk_noskin_labelbutton_create (xitk_labelbutton_widget_t *b,
   i = xitk_image_create_image(b->imlibdata, width * 3, height);
   draw_bevel_three_state(b->imlibdata, i);
 
-  return _xitk_labelbutton_create(NULL, b, x, y, NULL, i, ncolor, fcolor, ccolor, fname, 1);
+  return _xitk_labelbutton_create(NULL, b, x, y, NULL, i, ncolor, fcolor, ccolor, fname, 1, 0, 1, 1);
 }
