@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mrlbrowser.c,v 1.24 2002/01/15 22:22:36 f1rmb Exp $
+ * $Id: mrlbrowser.c,v 1.33 2002/04/21 22:17:12 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -46,6 +46,7 @@
 #include "button.h"
 #include "labelbutton.h"
 #include "dnd.h"
+#include "window.h"
 #include "widget.h"
 #include "widget_types.h"
 
@@ -289,6 +290,22 @@ static void mrlbrowser_dumpmrl(xitk_widget_t *w, void *data) {
  *                                 END OF PRIVATES
  *****************************************************************************/
 
+/*
+ * Enable/Disable tips.
+ */
+void xitk_mrlbrowser_set_tips_timeout(xitk_widget_t *w, int enabled, unsigned long timeout) {
+  mrlbrowser_private_data_t *private_data;
+  
+  if(w && (w->widget_type & WIDGET_TYPE_MRLBROWSER)) {
+    private_data = (mrlbrowser_private_data_t *)w->private_data;
+    
+    if(enabled)
+      xitk_set_widgets_tips_timeout(private_data->widget_list, timeout);
+    else
+      xitk_disable_widgets_tips(private_data->widget_list);
+  }
+}
+
 /*
  * Return window id of widget.
  */
@@ -357,10 +374,12 @@ void xitk_mrlbrowser_hide(xitk_widget_t *w) {
     private_data = (mrlbrowser_private_data_t *)w->private_data;
 
     if(private_data->visible) {
+
       XLOCK(private_data->imlibdata->x.disp);
       XUnmapWindow(private_data->imlibdata->x.disp, private_data->window);
-      xitk_hide_widgets(private_data->widget_list);
       XUNLOCK(private_data->imlibdata->x.disp);
+
+      xitk_hide_widgets(private_data->widget_list);
       private_data->visible = 0;
     }
   }
@@ -375,10 +394,12 @@ void xitk_mrlbrowser_show(xitk_widget_t *w) {
   if(w && (w->widget_type & WIDGET_TYPE_MRLBROWSER)) {
     private_data = (mrlbrowser_private_data_t *)w->private_data;
 
-    XLOCK(private_data->imlibdata->x.disp);
     xitk_show_widgets(private_data->widget_list);
+
+    XLOCK(private_data->imlibdata->x.disp);
     XMapRaised(private_data->imlibdata->x.disp, private_data->window); 
     XUNLOCK(private_data->imlibdata->x.disp);
+
     private_data->visible = 1;
   }
 }
@@ -470,34 +491,51 @@ void xitk_mrlbrowser_exit(xitk_widget_t *w, void *data) {
  *
  */
 void xitk_mrlbrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
+  ImlibImage                *new_img, *old_img;
   mrlbrowser_private_data_t *private_data;
-  
+  XSizeHints                 hint;
+
   if(w && (w->widget_type & WIDGET_TYPE_MRLBROWSER)) {
     private_data = (mrlbrowser_private_data_t *)w->private_data;
     
+    xitk_skin_lock(skonfig);
+    xitk_hide_widgets(private_data->widget_list);
+
     XLOCK(private_data->imlibdata->x.disp);
     
-    Imlib_destroy_image(private_data->imlibdata, private_data->bg_image);
-    
-    if(!(private_data->bg_image = 
-	 Imlib_load_image(private_data->imlibdata,
-			  xitk_skin_get_skin_filename(skonfig, 
-						      private_data->skin_element_name)))) {
+    if(!(new_img = Imlib_load_image(private_data->imlibdata,
+				    xitk_skin_get_skin_filename(skonfig, 
+							private_data->skin_element_name)))) {
       XITK_DIE("%s(): couldn't find image for background\n", __FUNCTION__);
     }
     
+    hint.width  = new_img->rgb_width;
+    hint.height = new_img->rgb_height;
+    hint.flags  = PSize;
+    XSetWMNormalHints(private_data->imlibdata->x.disp, private_data->window, &hint);
+
     XResizeWindow (private_data->imlibdata->x.disp, private_data->window,
-		   (unsigned int)private_data->bg_image->rgb_width,
-		   (unsigned int)private_data->bg_image->rgb_height);
-    
-    /*
-     * We should here, otherwise new skined window will have wrong size.
-     */
-    XFlush(private_data->imlibdata->x.disp);
-    
-    Imlib_apply_image(private_data->imlibdata, private_data->bg_image, private_data->window);
-    
+		   (unsigned int)new_img->rgb_width,
+		   (unsigned int)new_img->rgb_height);
+
     XUNLOCK(private_data->imlibdata->x.disp);
+
+    while(!xitk_is_window_size(private_data->imlibdata->x.disp, private_data->window, 
+			       new_img->rgb_width, new_img->rgb_height)) {
+      xitk_usec_sleep(10000);
+    }
+
+    old_img = private_data->bg_image;
+    private_data->bg_image = new_img;
+
+    XLOCK(private_data->imlibdata->x.disp);
+
+    Imlib_destroy_image(private_data->imlibdata, old_img);
+    Imlib_apply_image(private_data->imlibdata, new_img, private_data->window);
+
+    XUNLOCK(private_data->imlibdata->x.disp);
+
+    xitk_skin_unlock(skonfig);
     
     xitk_change_skins_widget_list(private_data->widget_list, skonfig);
 
@@ -727,6 +765,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
   xitk_labelbutton_widget_t   lb;
   xitk_button_widget_t        pb;
   xitk_label_widget_t         lbl;
+  xitk_widget_t              *w;
   long                        data[1];
   
   XITK_CHECK_CONSTITENCY(mb);
@@ -802,7 +841,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
 					  hint.x, hint.y, hint.width, 
 					  hint.height, 0, 
 					  mb->imlibdata->x.depth, 
-					  CopyFromParent, 
+					  InputOutput, 
 					  mb->imlibdata->x.visual,
 					  CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect,
 					  &attr);
@@ -810,9 +849,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
   XSetStandardProperties(mb->imlibdata->x.disp, private_data->window, title, title,
 			 None, NULL, 0, &hint);
 
-  XSelectInput(mb->imlibdata->x.disp, private_data->window,
-	       ButtonPressMask | ButtonReleaseMask | KeymapStateMask | PointerMotionMask 
-	       | KeyPressMask | ExposureMask | StructureNotifyMask);
+  XSelectInput(mb->imlibdata->x.disp, private_data->window, INPUT_MOTION | KeymapStateMask);
   
   /*
    * layer above most other things, like gnome panel
@@ -863,6 +900,8 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
   Imlib_apply_image(mb->imlibdata, 
 		    private_data->bg_image, private_data->window);
 
+  XUNLOCK (mb->imlibdata->x.disp);
+
   private_data->widget_list                = xitk_widget_list_new() ;
   private_data->widget_list->l             = xitk_list_new ();
   private_data->widget_list->focusedWidget = NULL;
@@ -877,13 +916,15 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
   lb.userdata          = (void *)private_data;
   lb.skin_element_name = mb->select.skin_element_name;
   xitk_list_append_content(private_data->widget_list->l,
-			  xitk_labelbutton_create (skonfig, &lb));
-
+			   (w = xitk_labelbutton_create (skonfig, &lb)));
+  xitk_set_widget_tips(w, _("Select current entry"));
+  
   pb.skin_element_name = mb->play.skin_element_name;
   pb.callback          = mrlbrowser_play;
   pb.userdata          = (void *)private_data;
   xitk_list_append_content(private_data->widget_list->l,
-			  xitk_button_create (skonfig, &pb));
+			   (w = xitk_button_create (skonfig, &pb)));
+  xitk_set_widget_tips(w, _("Play selected entry"));
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = mb->dismiss.caption;
@@ -892,7 +933,8 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
   lb.userdata          = (void *)mywidget;
   lb.skin_element_name = mb->dismiss.skin_element_name;
   xitk_list_append_content(private_data->widget_list->l,
-			  xitk_labelbutton_create (skonfig, &lb));
+			   (w = xitk_labelbutton_create (skonfig, &lb)));
+  xitk_set_widget_tips(w, _("Close MRL browser window"));
   
   private_data->add_callback      = mb->select.callback;
   private_data->play_callback     = mb->play.callback;
@@ -947,8 +989,10 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
       lb.userdata       = (void *)private_data;
       lb.skin_element_name = mb->ip_name.button.skin_element_name;
       xitk_list_append_content(private_data->widget_list->l,
-			      (private_data->autodir_plugins[i] = xitk_labelbutton_create (skonfig, &lb)));
-
+			       (private_data->autodir_plugins[i] = xitk_labelbutton_create (skonfig, &lb)));
+      xitk_set_widget_tips(private_data->autodir_plugins[i], 
+			   xine_get_input_plugin_description(mb->xine, mb->ip_availables[i]));
+      
       (void) xitk_set_widget_pos(private_data->autodir_plugins[i], x, y);
 
       y += xitk_get_widget_height(private_data->autodir_plugins[i]) + 1;
@@ -959,7 +1003,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
   }
     
   private_data->visible        = 1;
-
+  
   mywidget->enable             = 1;
   mywidget->running            = 1;
   mywidget->visible            = 1;
@@ -986,7 +1030,9 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
 			   private_data->mc->mrls_disp, 
 			   private_data->mrls_num, 0);
   
+  XLOCK (mb->imlibdata->x.disp);
   XMapRaised(mb->imlibdata->x.disp, private_data->window); 
+  XUNLOCK (mb->imlibdata->x.disp);
 
   private_data->widget_key = 
     xitk_register_event_handler("mrl browser",
@@ -996,8 +1042,6 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
 				mb->dndcallback,
 				private_data->widget_list,
 				(void *) private_data);
-  
-  XUNLOCK (mb->imlibdata->x.disp);
 
   return mywidget;
 }
