@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skin.c,v 1.6 2002/01/15 22:22:36 f1rmb Exp $
+ * $Id: skin.c,v 1.13 2002/04/18 23:37:00 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -27,7 +27,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <assert.h>
+#include <pthread.h>
 #include <errno.h>
 
 #include "skin.h"
@@ -135,6 +137,22 @@ static char *_get_expanded_command(xitk_skin_config_t *skonfig, char *cmd) {
   return ret;
 }
 
+/*
+ * Nullify all entried from s element.
+ */
+static void _nullify_me(xitk_skin_element_t *s) {
+  s->prev        = NULL;
+  s->next        = NULL;
+  s->section     = NULL;
+  s->pixmap      = NULL;
+  s->pixmap_pad  = NULL;
+  s->pixmap_font = NULL;
+  s->color       = NULL;
+  s->color_focus = NULL;
+  s->color_click = NULL;
+  s->font        = NULL;
+}
+
 /*
  * Return position in str of char 'c'. -1 if not found
  */
@@ -169,6 +187,21 @@ static int skin_end_section(xitk_skin_config_t *skonfig) {
   return _is_there_char(skonfig->ln, '}');
 }
 
+static int istriplet(char *c) {
+  int dummy1, dummy2, dummy3;
+
+  if((strncasecmp(c, "#", 1) <= 0) && (strlen(c) >= 7)) {
+
+    if(((isalnum(*(c+1))) && (isalnum(*(c+2))) && (isalnum(*(c+3))) 
+       && (isalnum(*(c+4))) && (isalnum(*(c+5))) && (isalnum(*(c+6)))
+       && ((*(c+7) == '\0') || (*(c+7) == '\n') || (*(c+7) == '\r') || (*(c+7) == ' ')))
+       && (sscanf(c, "#%2x%2x%2x", &dummy1, &dummy2, &dummy3) == 3)) {
+      return 1;
+    }
+  }
+
+  return 0;
+}
 /*
  * Cleanup the EOL ('\n','\r',' ')
  */
@@ -181,10 +214,12 @@ static void skin_clean_eol(xitk_skin_config_t *skonfig) {
 
   if(p) {
     while(*p != '\0') {
-      if(*p == '\n' || *p == '\r') {
+      if(*p == '\n' || *p == '\r' || (*p == '#' && (istriplet(p) == 0)) || *p == ';' 
+	 || (*p == '/' && (*(p+1) == '/' || *(p+1) == '*'))) {
 	*p = '\0';
 	break;
       }
+
       p++;
     }
 
@@ -327,7 +362,11 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
 	  skonfig->celement->pixmap_pad = (char *) xitk_xmalloc(strlen(skonfig->path) + strlen(p) + 2);
 	  sprintf(skonfig->celement->pixmap_pad, "%s/%s", skonfig->path, p);
 	}
-	else if(!strncasecmp(skonfig->ln, "type", 1)) {
+	else if(!strncasecmp(skonfig->ln, "radius", 6)) {
+	  skin_set_pos_to_value(&p);
+	  skonfig->celement->radius = strtol(p, &p, 10);
+	}
+	else if(!strncasecmp(skonfig->ln, "type", 4)) {
 	  skin_set_pos_to_value(&p);
 	  if(!strncasecmp("horizontal", p, strlen(p))) {
 	    skonfig->celement->slider_type = XITK_HSLIDER;
@@ -335,6 +374,9 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
 	  else if(!strncasecmp("vertical", p, strlen(p))) {
 	    skonfig->celement->slider_type = XITK_VSLIDER;
 	  }
+	  else if(!strncasecmp("rotate", p, strlen(p))) {
+	    skonfig->celement->slider_type = XITK_RSLIDER;
+	  }
 	  else
 	    skonfig->celement->slider_type = XITK_HSLIDER;
 	}
@@ -372,6 +414,10 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
 	  skonfig->celement->pixmap_font = (char *) xitk_xmalloc(strlen(skonfig->path) + strlen(p) + 2);
 	  sprintf(skonfig->celement->pixmap_font, "%s/%s", skonfig->path, p);
 	}
+	else if(!strncasecmp(skonfig->ln, "static", 6)) {
+	  skin_set_pos_to_value(&p);
+	  skonfig->celement->staticity = skin_get_bool_value(p);
+	}
 	else if(!strncasecmp(skonfig->ln, "align", 5)) {
 	  skin_set_pos_to_value(&p);
 	  skonfig->celement->align = skin_get_align_value(p);
@@ -418,7 +464,8 @@ static void skin_parse_section(xitk_skin_config_t *skonfig) {
 	  xitk_skin_element_t *s;
 	  
 	  s = (xitk_skin_element_t *) xitk_xmalloc(sizeof(xitk_skin_element_t));
-
+	  _nullify_me(s);
+	  
 	  if(skonfig->celement) {
 	    skonfig->celement->next = s;
 	    s->prev = skonfig->celement;
@@ -430,7 +477,8 @@ static void skin_parse_section(xitk_skin_config_t *skonfig) {
 	  }
 
 	  s->section = strdup(section);
-	  
+	  s->visible = s->enable = 1;
+
 	  skin_get_next_line(skonfig);
 
 	__next_subsection:
@@ -444,15 +492,27 @@ static void skin_parse_section(xitk_skin_config_t *skonfig) {
 	    else
 	     goto  __next_subsection;
 	  }
-	  else
-	    if(!strncasecmp(skonfig->ln, "pixmap", 6)) {
+	  else {
+
+	    if(!strncasecmp(skonfig->ln, "visible", 7)) {
+	      skin_set_pos_to_value(&p);
+	      s->visible = skin_get_bool_value(p);
+	    }
+	    else if(!strncasecmp(skonfig->ln, "pixmap", 6)) {
 	      skin_set_pos_to_value(&p);
 	      s->pixmap = (char *) xitk_xmalloc(strlen(skonfig->path) + strlen(p) + 2);
 	      sprintf(s->pixmap, "%s/%s", skonfig->path, p);
 	    }
-	  
+	    else if(!strncasecmp(skonfig->ln, "enable", 6)) {
+	      skin_set_pos_to_value(&p);
+	      s->enable = skin_get_bool_value(p);
+	    }
+	    
+	  }
+
 	  skin_get_next_line(skonfig);
-	  
+	  p = skonfig->ln;
+
 	  if(skin_end_section(skonfig) >= 0) {
 	    return;
 	  }
@@ -525,6 +585,8 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
 
     while(s) {
       printf("Section '%s'\n", s->section);
+      printf("  enable      = %d\n", s->enable);
+      printf("  visible     = %d\n", s->visible);
       printf("  X           = %d\n", s->x);
       printf("  Y           = %d\n", s->y);
       printf("  pixmap      = '%s'\n", s->pixmap);
@@ -536,6 +598,7 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
 
       printf("  animation   = %d\n", s->animation);
       printf("  print       = %d\n", s->print);
+      printf("  static      = %d\n", s->staticity);
       printf("  length      = %d\n", s->length);
       printf("  color       = '%s'\n", s->color);
       printf("  color focus = '%s'\n", s->color_focus);
@@ -548,12 +611,15 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
     printf("revert order\n");
     s = skonfig->last;
     while(s) {
-      printf("Section '%s'\n", s->section);
+      printf("Section '%s'\n", s->section); 
+      printf("  enable      = %d\n", s->enable);
+      printf("  visible     = %d\n", s->visible);
       printf("  X           = %d\n", s->x);
       printf("  Y           = %d\n", s->y);
       printf("  pixmap      = '%s'\n", s->pixmap);
       printf("  animation   = %d\n", s->animation);
       printf("  print       = %d\n", s->print);
+      printf("  static      = %d\n", s->staticity);
       printf("  length      = %d\n", s->length);
       printf("  color       = '%s'\n", s->color);
       printf("  color focus = '%s'\n", s->color_focus);
@@ -573,8 +639,10 @@ static xitk_skin_element_t *skin_lookup_section(xitk_skin_config_t *skonfig, con
 
   s = skonfig->first;
   while(s) {
+
     if(!strcasecmp(str, s->section))
       return s;
+
     s = s->next;
   }
 
@@ -602,6 +670,8 @@ xitk_skin_config_t *xitk_skin_init_config(void) {
 
   skonfig->ln = skonfig->buf;
   
+  pthread_mutex_init(&skonfig->mutex, NULL);
+
   return skonfig;
 }
 
@@ -616,22 +686,20 @@ void xitk_skin_free_config(xitk_skin_config_t *skonfig) {
   if(skonfig->celement) {
     s = skonfig->last;
     
-    if(s) {
-      while(s) {
-	XITK_FREE(s->section);
-	XITK_FREE(s->pixmap);
-	XITK_FREE(s->pixmap_pad);
-	XITK_FREE(s->pixmap_font);
-	XITK_FREE(s->color);
-	XITK_FREE(s->color_focus);
-	XITK_FREE(s->color_click);
-	XITK_FREE(s->font);
-	
-	c = s;
-	s = s->prev;
-	
-	XITK_FREE(c);
-      }
+    while(s) {
+      XITK_FREE(s->section);
+      XITK_FREE(s->pixmap);
+      XITK_FREE(s->pixmap_pad);
+      XITK_FREE(s->pixmap_font);
+      XITK_FREE(s->color);
+      XITK_FREE(s->color_focus);
+      XITK_FREE(s->color_click);
+      XITK_FREE(s->font);
+      
+      c = s;
+      s = c->prev;
+      
+      XITK_FREE(c);
     }
   }
   
@@ -643,6 +711,9 @@ void xitk_skin_free_config(xitk_skin_config_t *skonfig) {
   XITK_FREE(skonfig->load_command);
   XITK_FREE(skonfig->unload_command);
   XITK_FREE(skonfig->skinfile);
+
+  pthread_mutex_destroy(&skonfig->mutex);
+
   XITK_FREE(skonfig);
 }
 
@@ -734,6 +805,34 @@ int xitk_skin_check_version(xitk_skin_config_t *skonfig, int min_version) {
   return -1;
 }
 
+/*
+ *
+ */
+int xitk_skin_get_visibility(xitk_skin_config_t *skonfig, const char *str) {
+  xitk_skin_element_t *s;
+  
+  assert(skonfig);
+  
+  if((s = skin_lookup_section(skonfig, str)) != NULL)
+    return s->visible;
+
+  return 1;
+}
+
+/*
+ *
+ */
+int xitk_skin_get_enability(xitk_skin_config_t *skonfig, const char *str) {
+  xitk_skin_element_t *s;
+  
+  assert(skonfig);
+  
+  if((s = skin_lookup_section(skonfig, str)) != NULL)
+    return s->enable;
+
+  return 1;
+}
+
 /*
  *
  */
@@ -860,6 +959,20 @@ int xitk_skin_get_label_printable(xitk_skin_config_t *skonfig, const char *str)
   return 1;
 }
 
+/*
+ *
+ */
+int xitk_skin_get_label_staticity(xitk_skin_config_t *skonfig, const char *str) {
+  xitk_skin_element_t *s;
+  
+  assert(skonfig);
+  
+  if((s = skin_lookup_section(skonfig, str)) != NULL)
+    return s->staticity;
+  
+  return 0;
+}
+
 /*
  *
  */
@@ -930,3 +1043,35 @@ int xitk_skin_get_slider_type(xitk_skin_config_t *skonfig, const char *str) {
 
   return 0;
 }
+
+/*
+ *
+ */
+int xitk_skin_get_slider_radius(xitk_skin_config_t *skonfig, const char *str) {
+  xitk_skin_element_t *s;
+  
+  assert(skonfig);
+  
+  if((s = skin_lookup_section(skonfig, str)) != NULL)
+    return s->radius;
+
+  return 0;
+}
+
+/*
+ *
+ */
+void xitk_skin_lock(xitk_skin_config_t *skonfig) {
+  assert(skonfig);
+
+  pthread_mutex_lock(&skonfig->mutex);
+}
+
+/*
+ *
+ */
+void xitk_skin_unlock(xitk_skin_config_t *skonfig) {
+  assert(skonfig);
+  
+  pthread_mutex_unlock(&skonfig->mutex);
+}
