@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: slider.c,v 1.27 2002/01/09 23:44:45 f1rmb Exp $
+ * $Id: slider.c,v 1.35 2002/04/19 22:14:38 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -34,50 +34,133 @@
 #include "widget_types.h"
 #include "_xitk.h"
 
-#ifdef DEBUG_GUI
-#define COMPUTE_COORDS(X,Y)                                                \
-     {                                                                     \
-       if(private_data->sType == XITK_HSLIDER) {                           \
-         private_data->pos = (int) ((X - sl->x) * private_data->ratio);    \
-       }                                                                   \
-       else if(private_data->sType == XITK_VSLIDER) {                      \
-         private_data->pos = (int) ((private_data->bg_skin->height         \
-				- (Y - sl->y))                             \
-                                * private_data->ratio);                    \
-       }                                                                   \
-       else {                                                              \
-         fprintf(stderr, "Unknown slider type (%d)\n",                     \
-                 private_data->sType);                                     \
-       }                                                                   \
-       if(private_data->pos > private_data->max) {                         \
-         fprintf(stderr, "slider POS > MAX!.\n");                          \
-         private_data->pos = private_data->max;                            \
-       }                                                                   \
-       if(private_data->pos < private_data->min) {                         \
-         fprintf(stderr, "slider POS < MIN!.\n");                          \
-         private_data->pos = private_data->min;                            \
-       }                                                                   \
-     }
-#else
-#define COMPUTE_COORDS(X,Y)                                                \
-     {                                                                     \
-       if(private_data->sType == XITK_HSLIDER) {                           \
-         private_data->pos = (int) ((X - sl->x) * private_data->ratio);    \
-       }                                                                   \
-       else if(private_data->sType == XITK_VSLIDER) {                      \
-         private_data->pos = (int) ((private_data->bg_skin->height         \
-				- (Y - sl->y))                             \
-                                * private_data->ratio);                    \
-       }                                                                   \
-       if(private_data->pos > private_data->max) {                         \
-         private_data->pos = private_data->max;                            \
-       }                                                                   \
-       if(private_data->pos < private_data->min) {                         \
-         private_data->pos = private_data->min;                            \
-       }                                                                   \
-     }
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
 #endif
 
+/*
+ *
+ */
+static void slider_update_value(xitk_widget_t *w, float value) {
+  slider_private_data_t *private_data = (slider_private_data_t *) w->private_data;
+  float                  range;
+
+  if(value < private_data->lower)
+    private_data->value = private_data->lower;
+  else if(value > private_data->upper)
+    private_data->value = private_data->upper;
+  else
+    private_data->value = value;
+
+  range = (private_data->upper + -private_data->lower);
+  
+  private_data->percentage = (private_data->value + -private_data->lower) / range;
+  private_data->angle = 7.*M_PI/6. - (private_data->value - private_data->lower) * 
+    4.*M_PI/3. / (private_data->upper - private_data->lower);
+  
+}
+
+/*
+ *
+ */
+static void slider_update_minmax(xitk_widget_t *w, float min, float max) {
+  slider_private_data_t *private_data = (slider_private_data_t *) w->private_data;
+
+  if(min > max) {
+    XITK_WARNING("%s@%d: slider min value > max value !\n", __FILE__, __LINE__);
+    return;
+  }
+  
+  private_data->upper = (min == max) ? max + 1 : max;
+  private_data->lower = min;
+  slider_update_value(w, private_data->value);
+}
+
+/*
+ *
+ */
+static void slider_update(xitk_widget_t *w, int x, int y) {
+  slider_private_data_t *private_data = (slider_private_data_t *) w->private_data;
+  
+  if(private_data->sType == XITK_RSLIDER) {
+    int   xc, yc;
+    float old_value;
+    
+    xc = private_data->bg_skin->width / 2;
+    yc = private_data->bg_skin->height / 2;
+    
+    old_value = private_data->value;
+    private_data->angle = atan2(yc - y, x - xc);
+    
+    if(private_data->angle < -M_PI / 2.)
+      private_data->angle += 2 * M_PI;
+    
+    if(private_data->angle < -M_PI / 6)
+      private_data->angle = -M_PI / 6;
+    
+    if(private_data->angle > 7. * M_PI / 6.)
+      private_data->angle = 7. * M_PI / 6.;
+    
+    private_data->value = private_data->lower + (7. * M_PI / 6 - private_data->angle) *
+      (private_data->upper - private_data->lower) / (4. * M_PI / 3.);
+    
+    if(private_data->value != old_value) {
+      float new_value;
+      
+      new_value = private_data->value;
+      
+      if(new_value < private_data->lower)
+	new_value = private_data->lower;
+      
+      if(new_value > private_data->upper)
+	new_value = private_data->upper;
+      
+      slider_update_value(w, new_value);
+      
+    }
+  }
+  else {
+    float width, height;
+    float old_value, new_value = 0.0;
+    
+    old_value = private_data->value;
+
+    width = (float)private_data->bg_skin->width;
+    height = (float)private_data->bg_skin->height;
+
+    if(x < 0)
+      x = 0;
+    if(x > width)
+      x = width;
+    if(y < 0)
+      y = 0;
+    if(y > height)
+      y = height;
+    
+    if(private_data->sType == XITK_HSLIDER)
+      new_value = (x * .01) / (width * .01);
+    else if(private_data->sType == XITK_VSLIDER)
+      new_value = ((height - y) * .01) / (height * .01);
+    
+    private_data->value = private_data->lower + 
+      (new_value * (private_data->upper - private_data->lower));
+    
+    if(private_data->value != old_value) {
+      float new_value;
+      
+      new_value = private_data->value;
+      
+      if (new_value < private_data->lower)
+	new_value = private_data->lower;
+      
+      if (new_value > private_data->upper)
+	new_value = private_data->upper;
+      
+      slider_update_value(w, new_value);
+    }
+  }
+}
+
 /*
  *
  */
@@ -96,8 +179,7 @@ static void notify_destroy(xitk_widget_t *w, void *data) {
  *
  */
 static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
-  slider_private_data_t *private_data = 
-    (slider_private_data_t *) w->private_data;
+  slider_private_data_t *private_data = (slider_private_data_t *) w->private_data;
   
   if(w->widget_type & WIDGET_TYPE_SLIDER) {
     if(sk == FOREGROUND_SKIN && private_data->paddle_skin) {
@@ -115,11 +197,10 @@ static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
  *
  */
 static int notify_inside(xitk_widget_t *sl, int x, int y) {
-  slider_private_data_t *private_data = 
-    (slider_private_data_t *) sl->private_data;
+  slider_private_data_t *private_data = (slider_private_data_t *) sl->private_data;
 
   if(sl->widget_type & WIDGET_TYPE_SLIDER) {
-    if(sl->visible) {
+    if((sl->visible == 1)) {
       xitk_image_t *skin;
       
       if(private_data->paddle_cover_bg == 1)
@@ -139,111 +220,146 @@ static int notify_inside(xitk_widget_t *sl, int x, int y) {
 /*
  * Draw widget
  */
-static void paint_slider (xitk_widget_t *sl, Window win, GC gc) {
-  float                   tmp = 0L;
+static void paint_slider(xitk_widget_t *sl, Window win, GC gc) {
   int                     button_width, button_height;	
   slider_private_data_t  *private_data = (slider_private_data_t *) sl->private_data;
   GC                      bgc, pgc;
   xitk_image_t           *bg = (xitk_image_t *) private_data->bg_skin;
   xitk_image_t           *paddle = (xitk_image_t *) private_data->paddle_skin;
-  int                     srcx1, srcx2, destx1, srcy1, srcy2, desty1;
   
-  if ((sl->widget_type & WIDGET_TYPE_SLIDER) && sl->visible) {
-    int x=0, y=0;
-
-    if(private_data->pos > private_data->max
-       || private_data->pos < private_data->min)
-      return;
+  if((sl->widget_type & WIDGET_TYPE_SLIDER) && (sl->visible == 1)) {
+    int    x, y, srcx1, srcx2, destx1, srcy1, srcy2, desty1;
+    int    xcenter, ycenter;
+    int    paddle_width;
+    int    paddle_height;
+    double angle;
     
+    x = y = srcx1 = srcx2 = destx1 = srcy1 = srcy2 = desty1 = 0;
+        
     XLOCK (private_data->imlibdata->x.disp);
-    
-    button_width = private_data->button_width;
-    button_height = private_data->paddle_skin->height;
-
-    srcx1 = srcx2 = destx1 = srcy1 = srcy2 = desty1 = 0;
 
     bgc = XCreateGC(private_data->imlibdata->x.disp, bg->image, None, None);
     XCopyGC(private_data->imlibdata->x.disp, gc, (1 << GCLastBit) - 1, bgc);
     pgc = XCreateGC(private_data->imlibdata->x.disp, paddle->image, None, None);
     XCopyGC(private_data->imlibdata->x.disp, gc, (1 << GCLastBit) - 1, pgc);
-    
-    if (bg->mask) {
+      
+    if(bg->mask) {
       XSetClipOrigin(private_data->imlibdata->x.disp, bgc, sl->x, sl->y);
       XSetClipMask(private_data->imlibdata->x.disp, bgc, bg->mask);
     }
     
-    if(private_data->sType == XITK_HSLIDER)
-      tmp = sl->width - button_width;
-    if(private_data->sType == XITK_VSLIDER)
-      tmp = sl->height - button_height;
+    XCopyArea(private_data->imlibdata->x.disp, bg->image, win, bgc, 0, 0,
+	      bg->width, bg->height, sl->x, sl->y);
+      
+    XUNLOCK(private_data->imlibdata->x.disp);
 
-    tmp /= private_data->max;
-    tmp *= private_data->pos;
-    
-    XCopyArea (private_data->imlibdata->x.disp, bg->image, win, bgc, 0, 0,
-	       bg->width, bg->height, sl->x, sl->y);
-    
-    if(private_data->sType == XITK_HSLIDER) {
-      x = rint(sl->x + tmp);
-      y = sl->y;
-    }
-    else if(private_data->sType == XITK_VSLIDER) {
-      x = sl->x;
-      y = rint(sl->y + private_data->bg_skin->height - button_height - tmp);
-    }
+    if(private_data->sType == XITK_RSLIDER) {
+      
+      button_width = private_data->bg_skin->width;
+      button_height = private_data->bg_skin->height;
 
-    if(private_data->paddle_cover_bg == 1) {
-      int pixpos;
+      xcenter       = (bg->width /2) + sl->x;
+      ycenter       = (bg->width /2) + sl->y;
+      paddle_width  = paddle->width / 3;
+      paddle_height = paddle->height;
+      angle         = private_data->angle;
       
-      pixpos = (int)(private_data->pos / private_data->ratio);
+      if(angle < M_PI / -2)
+	angle = angle + M_PI * 2;
       
-      if(private_data->sType == XITK_VSLIDER) {
-	pixpos = button_height - pixpos;
-	srcx1  = 0;
-	srcy1  = pixpos;
-	srcx2  = button_width;
-	srcy2  = paddle->height - pixpos;
-	destx1 = sl->x;
-	desty1 = sl->y + pixpos;
-      }
-      else if(private_data->sType == XITK_HSLIDER) {
-	srcx1  = 0;
-	srcy1  = 0;
-	srcx2  = pixpos;
-	srcy2  = paddle->height;
-	destx1 = sl->x;
-	desty1 = sl->y;
-      }
+      x = (int) (0.5 + xcenter + private_data->radius * cos(angle)) - (paddle_width / 2);
+      y = (int) (0.5 + ycenter - private_data->radius * sin(angle)) - (paddle_height / 2);
 
-                  
-      if(paddle->mask) {
-	XSetClipOrigin(private_data->imlibdata->x.disp, pgc, sl->x, sl->y);
-	XSetClipMask(private_data->imlibdata->x.disp, pgc, paddle->mask);
+      srcx1 = 0;
+      
+      if(private_data->bArmed) {
+	srcx1 += paddle_width;
+	if(private_data->bClicked)
+	  srcx1 += paddle_width;
       }
       
-    }
-    else {
-      
-      if(paddle->mask) {
-	XSetClipOrigin(private_data->imlibdata->x.disp, pgc, x, y);
-	XSetClipMask(private_data->imlibdata->x.disp, pgc, paddle->mask);
-      }
-      
-      srcy1  = 0;
-      srcx2  = button_width;
-      srcy2  = paddle->height;
+      srcy1 = 0;
+      srcx2 = paddle_width;
+      srcy2 = paddle_height;
       destx1 = x;
       desty1 = y;
 
     }
+    else {
+      int   dir_area;
+      float tmp;
+
+      button_width = private_data->button_width;
+      button_height = paddle->height;
+      
+      dir_area = (float)(private_data->sType == XITK_HSLIDER) ? 
+	sl->width - button_width : sl->height - button_height;
+	
+      tmp = (dir_area * .01) * (private_data->percentage * 100.0);
+      
+      if(private_data->sType == XITK_HSLIDER) {
+	x = rint(sl->x + tmp);
+	y = sl->y;
+      }
+      else if(private_data->sType == XITK_VSLIDER) {
+	x = sl->x;
+	y = rint(sl->y + private_data->bg_skin->height - button_height - tmp);
+      }
+      
+      if(private_data->paddle_cover_bg == 1) {
+	int pixpos;
+	int direction;
+	
+	x = sl->x;
+	y = sl->y;
+	
+	direction = (private_data->sType == XITK_HSLIDER) ? 
+	  private_data->bg_skin->width : private_data->bg_skin->height;
+	
+	pixpos = (int)private_data->value / 
+	  ((private_data->upper - private_data->lower) / direction);
+	
+	if(private_data->sType == XITK_VSLIDER) {
+	  pixpos = button_height - pixpos;
+	  srcx1  = 0;
+	  srcy1  = pixpos;
+	  srcx2  = button_width;
+	  srcy2  = paddle->height - pixpos;
+	  destx1 = sl->x;
+	  desty1 = sl->y + pixpos;
+	}
+	else if(private_data->sType == XITK_HSLIDER) {
+	  srcx1  = 0;
+	  srcy1  = 0;
+	  srcx2  = pixpos;
+	  srcy2  = paddle->height;
+	  destx1 = sl->x;
+	  desty1 = sl->y;
+	}
+      }
+      else {
+	srcy1  = 0;
+	srcx2  = button_width;
+	srcy2  = paddle->height;
+	destx1 = x;
+	desty1 = y;
+      }
+      
+      if(private_data->bArmed) {
+	if(private_data->bClicked)
+	  srcx1 = 2*button_width;
+	else
+	  srcx1 = button_width;
+      }
 
-    if(private_data->bArmed) {
-      if(private_data->bClicked)
-	srcx1 = 2*button_width;
-      else
-	srcx1 = button_width;
     }
-
+    
+    XLOCK(private_data->imlibdata->x.disp);
+    if(paddle->mask) {
+      XSetClipOrigin(private_data->imlibdata->x.disp, pgc, x, y);
+      XSetClipMask(private_data->imlibdata->x.disp, pgc, paddle->mask);
+    }
+    
     XCopyArea(private_data->imlibdata->x.disp, paddle->image, win, pgc,
 	      srcx1, srcy1, srcx2, srcy2, destx1, desty1);
     
@@ -251,6 +367,7 @@ static void paint_slider (xitk_widget_t *sl, Window win, GC gc) {
     XFreeGC(private_data->imlibdata->x.disp, bgc);
     XUNLOCK(private_data->imlibdata->x.disp);
   }
+
 }
 
 /*
@@ -258,11 +375,13 @@ static void paint_slider (xitk_widget_t *sl, Window win, GC gc) {
  */
 static void notify_change_skin(xitk_widget_list_t *wl, 
 			       xitk_widget_t *sl, xitk_skin_config_t *skonfig) {
-  slider_private_data_t *private_data = 
-    (slider_private_data_t *) sl->private_data;
+  slider_private_data_t *private_data = (slider_private_data_t *) sl->private_data;
   
   if(sl->widget_type & WIDGET_TYPE_SLIDER) {
     if(private_data->skin_element_name) {
+
+      xitk_skin_lock(skonfig);
+
       XITK_FREE_XITK_IMAGE(private_data->imlibdata->x.disp, private_data->paddle_skin);
       private_data->paddle_skin     = xitk_image_load_image(private_data->imlibdata, xitk_skin_get_slider_skin_filename(skonfig, private_data->skin_element_name));
       private_data->button_width    = private_data->paddle_skin->width / 3;
@@ -280,11 +399,17 @@ static void notify_change_skin(xitk_widget_list_t *wl,
 	  private_data->paddle_cover_bg = 1;
       }
       
+      private_data->radius = xitk_skin_get_slider_radius(skonfig, private_data->skin_element_name);
+
       sl->x       = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
       sl->y       = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
       sl->width   = private_data->bg_skin->width;
       sl->height  = private_data->bg_skin->height;
+      sl->visible = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1 : -1;
+      sl->enable  = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
       
+      xitk_skin_unlock(skonfig);
+
       xitk_set_widget_pos(sl, sl->x, sl->y);
     }
   }
@@ -293,72 +418,55 @@ static void notify_change_skin(xitk_widget_list_t *wl,
 /*
  * Got click
  */
-static int notify_click_slider (xitk_widget_list_t *wl, 
-				xitk_widget_t *sl, int bUp, int x, int y) {
-  slider_private_data_t *private_data = 
-    (slider_private_data_t *) sl->private_data;
-  int retpos;
+static int notify_click_slider(xitk_widget_list_t *wl, 
+			       xitk_widget_t *sl, int bUp, int x, int y) {
+  slider_private_data_t *private_data = (slider_private_data_t *) sl->private_data;
 
-  if (sl->widget_type & WIDGET_TYPE_SLIDER) {
+  if(sl->widget_type & WIDGET_TYPE_SLIDER) {
+    
+    slider_update(sl, (x - sl->x), (y - sl->y));
     
-    COMPUTE_COORDS(x, y);
-
     private_data->bClicked = !bUp;
     
     paint_slider(sl, wl->win, wl->gc);
     
-    if(bUp==0 && private_data->bArmed) {
+    if((bUp == 0) && private_data->bArmed) {
       XEvent sliderevent;
 
       /*
        * Exec motion callback function (if available)
        */
+
       if(private_data->motion_callback) {
-	if(private_data->realmin < 0)
-	  retpos = (private_data->realmin + private_data->pos);
-	else
-	  retpos = private_data->pos;
 	private_data->motion_callback(private_data->sWidget,
 				      private_data->motion_userdata,
-				      retpos);
+				      (int) private_data->value);
       }
 
       /*
        * Loop of death ;-)
        */
       do {
-	/* XLOCK (private_data->imlibdata->x.disp); */
-	XNextEvent (private_data->imlibdata->x.disp, &sliderevent) ;
-	/* XUNLOCK (private_data->imlibdata->x.disp); */
+
+	XNextEvent(private_data->imlibdata->x.disp, &sliderevent) ;
 
 	switch(sliderevent.type) {
 	  
 	case MotionNotify: {
-	  int maxx = (sl->x + sl->width);
-	  int maxy = (sl->y + sl->height);
-	  
-	  while (XCheckMaskEvent (private_data->imlibdata->x.disp, ButtonMotionMask,
-				  &sliderevent));
 
-	  COMPUTE_COORDS(((sliderevent.xbutton.x <= maxx) ? sliderevent.xbutton.x : maxx),
-			 ((sliderevent.xbutton.y <= maxy) ? sliderevent.xbutton.y : maxy));
-	  
+	  while(XCheckMaskEvent (private_data->imlibdata->x.disp, ButtonMotionMask,
+				 &sliderevent));
+
+	  slider_update(sl, (sliderevent.xbutton.x - sl->x), (sliderevent.xbutton.y - sl->y));
+
 	  paint_slider(sl, wl->win, wl->gc);
-	  
-	  /*
-	   * Callback exec on all of motion events
-	   */
-	  if(private_data->realmin < 0) {
-	    retpos = (private_data->realmin + private_data->pos);
-	  }
-	  else
-	    retpos = private_data->pos;
-	  
+
  	  if(private_data->motion_callback) {
 	    private_data->motion_callback(private_data->sWidget,
 					  private_data->motion_userdata,
-					  retpos);
+					  (int) private_data->value);
 	  }
+
 	}
 	break;
 
@@ -367,15 +475,12 @@ static int notify_click_slider (xitk_widget_list_t *wl,
 
 	  paint_slider(sl, wl->win, wl->gc);
 
-	  if(private_data->realmin < 0)
-	    retpos = (private_data->realmin + private_data->pos);
-	  else
-	    retpos = private_data->pos;
 	  if(private_data->callback) {
 	    private_data->callback(private_data->sWidget,
 				   private_data->userdata,
-				   retpos);
+				   (int) private_data->value);
 	  }
+
 	  break;
 
 	default:
@@ -383,9 +488,9 @@ static int notify_click_slider (xitk_widget_list_t *wl,
  	  break;
 	}
 	
-      } while (sliderevent.type != ButtonRelease); 
+      } while(sliderevent.type != ButtonRelease); 
     }
-    /* CHECKME    paint_widget_list (wl); */
+
   }
 
   return 1;
@@ -394,12 +499,11 @@ static int notify_click_slider (xitk_widget_list_t *wl,
 /*
  * Got focus
  */
-static int notify_focus_slider (xitk_widget_list_t *wl,
-			 xitk_widget_t *sl, int bEntered) {
-  slider_private_data_t *private_data = 
-    (slider_private_data_t *) sl->private_data;
+static int notify_focus_slider(xitk_widget_list_t *wl,
+			       xitk_widget_t *sl, int bEntered) {
+  slider_private_data_t *private_data = (slider_private_data_t *) sl->private_data;
 
-  if (sl->widget_type & WIDGET_TYPE_SLIDER)
+  if(sl->widget_type & WIDGET_TYPE_SLIDER)
     private_data->bArmed = bEntered;    
 
   return 1;
@@ -409,13 +513,13 @@ static int notify_focus_slider (xitk_widget_list_t *wl,
  * Increment position
  */
 void xitk_slider_make_step(xitk_widget_list_t *wl, xitk_widget_t *sl) {
-  slider_private_data_t *private_data = 
-    (slider_private_data_t *) sl->private_data;
+  slider_private_data_t *private_data = (slider_private_data_t *) sl->private_data;
 
-  if (sl->widget_type & WIDGET_TYPE_SLIDER
-      && !private_data->bClicked) {
-    if((xitk_slider_get_pos(sl) + private_data->step) <= private_data->max)
+  if (sl->widget_type & WIDGET_TYPE_SLIDER && !private_data->bClicked) {
+    
+    if(((xitk_slider_get_pos(sl)) + private_data->step) <= (xitk_slider_get_max(sl)))
       xitk_slider_set_pos(wl, sl, xitk_slider_get_pos(sl) + private_data->step);
+
   } 
 
 }
@@ -424,13 +528,13 @@ void xitk_slider_make_step(xitk_widget_list_t *wl, xitk_widget_t *sl) {
  * Decrement position
  */
 void xitk_slider_make_backstep(xitk_widget_list_t *wl, xitk_widget_t *sl) {
-  slider_private_data_t *private_data = 
-    (slider_private_data_t *) sl->private_data;
+  slider_private_data_t *private_data = (slider_private_data_t *) sl->private_data;
 
-  if (sl->widget_type & WIDGET_TYPE_SLIDER
-      && !private_data->bClicked) {
-    if((xitk_slider_get_pos(sl) - private_data->step) >= private_data->min)
+  if(sl->widget_type & WIDGET_TYPE_SLIDER && !private_data->bClicked) {
+    
+    if(((xitk_slider_get_pos(sl)) - private_data->step) >= (xitk_slider_get_min(sl)))
       xitk_slider_set_pos(wl, sl, xitk_slider_get_pos(sl) - private_data->step);
+
   } 
 
 }
@@ -439,21 +543,11 @@ void xitk_slider_make_backstep(xitk_widget_list_t *wl, xitk_widget_t *sl) {
  * Set value MIN.
  */
 void xitk_slider_set_min(xitk_widget_t *sl, int min) {
-  slider_private_data_t *private_data =
-    (slider_private_data_t *) sl->private_data;
+  slider_private_data_t *private_data = (slider_private_data_t *) sl->private_data;
   
-  if (sl->widget_type & WIDGET_TYPE_SLIDER) {
-    if(min < private_data->max) {
-      private_data->min = min;
-      if(private_data->sType == XITK_HSLIDER)
-	private_data->ratio = (float) 
-	  (private_data->max-private_data->min)/private_data->bg_skin->width;
-      else if(private_data->sType == XITK_VSLIDER)
-	private_data->ratio = (float)
-	  (private_data->max-private_data->min)/private_data->bg_skin->height;
-      else
-	XITK_WARNING("Unknown slider type (%d)\n", private_data->sType);
-    }
+  if(sl->widget_type & WIDGET_TYPE_SLIDER) {
+    slider_update_minmax(sl, (float)((min == private_data->upper) 
+				     ? min - 1 : min), private_data->upper);
   }
 }
 
@@ -461,11 +555,10 @@ void xitk_slider_set_min(xitk_widget_t *sl, int min) {
  * Return the MIN value
  */
 int xitk_slider_get_min(xitk_widget_t *sl) {
-  slider_private_data_t *private_data = 
-    (slider_private_data_t *) sl->private_data;
+  slider_private_data_t *private_data = (slider_private_data_t *) sl->private_data;
   
-  if (sl->widget_type & WIDGET_TYPE_SLIDER) {
-    return private_data->realmin;
+  if(sl->widget_type & WIDGET_TYPE_SLIDER) {
+    return (int) private_data->lower;
   } 
 
   return -1;
@@ -475,11 +568,10 @@ int xitk_slider_get_min(xitk_widget_t *sl) {
  * Return the MAX value
  */
 int xitk_slider_get_max(xitk_widget_t *sl) {
-  slider_private_data_t *private_data = 
-    (slider_private_data_t *) sl->private_data;
+  slider_private_data_t *private_data = (slider_private_data_t *) sl->private_data;
   
-  if (sl->widget_type & WIDGET_TYPE_SLIDER) {
-    return private_data->max;
+  if(sl->widget_type & WIDGET_TYPE_SLIDER) {
+    return (int) private_data->upper;
   } 
 
   return -1;
@@ -489,19 +581,12 @@ int xitk_slider_get_max(xitk_widget_t *sl) {
  * Set value MAX
  */
 void xitk_slider_set_max(xitk_widget_t *sl, int max) {
-  slider_private_data_t *private_data =
-    (slider_private_data_t *) sl->private_data;
+  slider_private_data_t *private_data = (slider_private_data_t *) sl->private_data;
   
-  if (sl->widget_type & WIDGET_TYPE_SLIDER) {
-    if(max > private_data->min) {
-      private_data->max = max;
-      if(private_data->sType == XITK_HSLIDER)
-	private_data->ratio = (float) 
-	  (private_data->max-private_data->min)/private_data->bg_skin->width;
-      else if(private_data->sType == XITK_VSLIDER)
-	private_data->ratio = (float)
-	  (private_data->max-private_data->min)/private_data->bg_skin->height;
-    }
+  if(sl->widget_type & WIDGET_TYPE_SLIDER) {
+    slider_update_minmax(sl, private_data->lower, (float)((max == private_data->lower) 
+							  ? max + 1 : max));
+
   } 
 }
 
@@ -509,15 +594,10 @@ void xitk_slider_set_max(xitk_widget_t *sl, int max) {
  * Set pos to 0 and redraw the widget.
  */
 void xitk_slider_reset(xitk_widget_list_t *wl, xitk_widget_t *sl) {
-  slider_private_data_t *private_data = 
-    (slider_private_data_t *) sl->private_data;
+  slider_private_data_t *private_data = (slider_private_data_t *) sl->private_data;
   
-  if (sl->widget_type & WIDGET_TYPE_SLIDER) {
-    if(private_data->realmin < 0)
-      private_data->pos = (0 - private_data->realmin);
-    else
-      private_data->pos = 0;
-
+  if(sl->widget_type & WIDGET_TYPE_SLIDER) {
+    slider_update_value(sl, 0.0);
     private_data->bClicked = 0;
     paint_slider(sl, wl->win, wl->gc);
   }
@@ -527,32 +607,23 @@ void xitk_slider_reset(xitk_widget_list_t *wl, xitk_widget_t *sl) {
  * Set pos to max and redraw the widget.
  */
 void xitk_slider_set_to_max(xitk_widget_list_t *wl, xitk_widget_t *sl) {
-  slider_private_data_t *private_data = 
-    (slider_private_data_t *) sl->private_data;
+  slider_private_data_t *private_data = (slider_private_data_t *) sl->private_data;
   
-  if (sl->widget_type & WIDGET_TYPE_SLIDER
-      && !private_data->bClicked) {
-    if(private_data->realmin < 0)
-      private_data->pos = (private_data->max - private_data->realmin);
-    else
-      private_data->pos = private_data->max;
-
+  if(sl->widget_type & WIDGET_TYPE_SLIDER && !private_data->bClicked) {
+    slider_update_value(sl, private_data->upper);
     paint_slider(sl, wl->win, wl->gc);
   }
+
 }
 
 /*
  * Return current position.
  */
 int xitk_slider_get_pos(xitk_widget_t *sl) {
-  slider_private_data_t *private_data = 
-    (slider_private_data_t *) sl->private_data;
+  slider_private_data_t *private_data = (slider_private_data_t *) sl->private_data;
 
-  if (sl->widget_type & WIDGET_TYPE_SLIDER) {
-    if(private_data->realmin< 0)
-      return (private_data->realmin + private_data->pos);
-    else
-      return private_data->pos;
+  if(sl->widget_type & WIDGET_TYPE_SLIDER) {
+    return (int) private_data->value;
   } 
 
   return -1;
@@ -562,32 +633,26 @@ int xitk_slider_get_pos(xitk_widget_t *sl) {
  * Set position.
  */
 void xitk_slider_set_pos(xitk_widget_list_t *wl, xitk_widget_t *sl, int pos) {
-  slider_private_data_t *private_data = 
-    (slider_private_data_t *) sl->private_data;
-
-  if (sl->widget_type & WIDGET_TYPE_SLIDER
-      && !private_data->bClicked) {
-    if (pos >= private_data->realmin && pos <= private_data->max) {
-      if(private_data->realmin < 0) {
-	private_data->pos = pos - private_data->realmin;
-      }
-      else
-	private_data->pos = pos;
-
+  slider_private_data_t *private_data = (slider_private_data_t *) sl->private_data;
+  
+  if(sl->widget_type & WIDGET_TYPE_SLIDER && !private_data->bClicked) {
+    float value = (float) pos;
+    
+    if(value >= private_data->lower && value <= private_data->upper) {
+      slider_update_value(sl, value);
       paint_slider(sl, wl->win, wl->gc);
     }
-    else
-      xitk_slider_reset(wl, sl);
+    
   } 
 }
 
 /*
  * Create the widget
  */
-static xitk_widget_t *_xitk_slider_create (xitk_skin_config_t *skonfig, xitk_slider_widget_t *s,
-					   int x, int y, char *skin_element_name,
-					   xitk_image_t *bg_skin, xitk_image_t *pad_skin,
-					   int stype) {
+static xitk_widget_t *_xitk_slider_create(xitk_skin_config_t *skonfig, xitk_slider_widget_t *s,
+					  int x, int y, char *skin_element_name,
+					  xitk_image_t *bg_skin, xitk_image_t *pad_skin,
+					  int stype, int radius, int visible, int enable) {
   xitk_widget_t           *mywidget;
   slider_private_data_t   *private_data;
 
@@ -602,36 +667,23 @@ static xitk_widget_t *_xitk_slider_create (xitk_skin_config_t *skonfig, xitk_sli
   private_data->sType                    = stype;
   private_data->bClicked                 = 0;
   private_data->bArmed                   = 0;
-  private_data->min                      = s->min;
 
-  if(s->max <= s->min) 
-    private_data->max                    = s->min + 1;
-  else
-    private_data->max                    = s->max;
+  private_data->angle                    = 0.0;
 
-  if(s->min < 0) {
-    private_data->realmin                = s->min;
-    private_data->min                    = 0;
-    private_data->realmax                = s->max;
-    private_data->max                    = s->max - s->min;
-  }
-  else
-    private_data->realmin                = s->min;
+  private_data->upper                    = (float)((s->min == s->max) ? s->max + 1 : s->max);
+  private_data->lower                    = (float)s->min;
+
+  private_data->value                    = 0.0;
+  private_data->percentage               = 0.0;
+  private_data->radius                   = radius;
 
-  private_data->pos                      = 0;
   private_data->step                     = s->step;
+
   private_data->paddle_skin              = pad_skin;
   private_data->button_width             = private_data->paddle_skin->width / 3;
   
   private_data->bg_skin                  = bg_skin;
 
-  if(private_data->sType == XITK_HSLIDER)
-    private_data->ratio                  = (float)(private_data->max - private_data->min)/private_data->bg_skin->width;
-  else if(private_data->sType == XITK_VSLIDER)
-    private_data->ratio                  = (float)(private_data->max - private_data->min)/private_data->bg_skin->height;
-  else
-    XITK_WARNING("Unknown slider type (%d)\n", private_data->sType);
-
   private_data->paddle_cover_bg          = 0;
   if(private_data->sType == XITK_HSLIDER) {
     if(private_data->button_width == private_data->bg_skin->width)
@@ -649,9 +701,9 @@ static xitk_widget_t *_xitk_slider_create (xitk_skin_config_t *skonfig, xitk_sli
 
   mywidget->private_data                 = private_data;
 
-  mywidget->enable                       = 1;
+  mywidget->enable                       = enable;
   mywidget->running                      = 1;
-  mywidget->visible                      = 1;
+  mywidget->visible                      = visible;
   mywidget->have_focus                   = FOCUS_LOST;
   mywidget->imlibdata                    = private_data->imlibdata;
   mywidget->x                            = x;
@@ -677,10 +729,10 @@ static xitk_widget_t *_xitk_slider_create (xitk_skin_config_t *skonfig, xitk_sli
 /*
  * Create the widget
  */
-xitk_widget_t *xitk_slider_create (xitk_skin_config_t *skonfig, xitk_slider_widget_t *s) {
+xitk_widget_t *xitk_slider_create(xitk_skin_config_t *skonfig, xitk_slider_widget_t *s) {
 
   XITK_CHECK_CONSTITENCY(s);
-
+  
   return _xitk_slider_create(skonfig, s,
 			     (xitk_skin_get_coord_x(skonfig, s->skin_element_name)),
 			     (xitk_skin_get_coord_y(skonfig, s->skin_element_name)),
@@ -689,15 +741,19 @@ xitk_widget_t *xitk_slider_create (xitk_skin_config_t *skonfig, xitk_slider_widg
 						    xitk_skin_get_skin_filename(skonfig, s->skin_element_name))),
 			     (xitk_image_load_image(s->imlibdata, 
 						    xitk_skin_get_slider_skin_filename(skonfig, s->skin_element_name))),
-			     (xitk_skin_get_slider_type(skonfig, s->skin_element_name)));
+			     (xitk_skin_get_slider_type(skonfig, s->skin_element_name)),
+			     (xitk_skin_get_slider_radius(skonfig, s->skin_element_name)),
+			     ((xitk_skin_get_visibility(skonfig, s->skin_element_name)) ? 1 : -1),
+			     (xitk_skin_get_enability(skonfig, s->skin_element_name)));
 }
 
 /*
  * Create the widget (not skined).
  */
-xitk_widget_t *xitk_noskin_slider_create (xitk_slider_widget_t *s,
-					  int x, int y, int width, int height, int type) {
+xitk_widget_t *xitk_noskin_slider_create(xitk_slider_widget_t *s,
+					 int x, int y, int width, int height, int type) {
   xitk_image_t   *b, *p;
+  int             radius;
 
   XITK_CHECK_CONSTITENCY(s);
   
@@ -705,17 +761,33 @@ xitk_widget_t *xitk_noskin_slider_create (xitk_slider_widget_t *s,
     p = xitk_image_create_image(s->imlibdata, width * 3, height / 5);
   else if(type == XITK_HSLIDER)
     p = xitk_image_create_image(s->imlibdata, (width / 5) * 3, height);
+  else if(type == XITK_RSLIDER) {
+    int w;
+    
+    w = ((((width + height) >> 1) >> 1) / 10) * 3;
+    p = xitk_image_create_image(s->imlibdata, (w * 3), w);
+  }
   else
     XITK_DIE("Slider type unhandled.\n");
-
+  
   xitk_image_add_mask(s->imlibdata, p);
   if(type == XITK_VSLIDER)
     draw_paddle_three_state_vertical(s->imlibdata, p);
   else if(type == XITK_HSLIDER)
     draw_paddle_three_state_horizontal(s->imlibdata, p);
+  else if(type == XITK_RSLIDER) {
+    draw_paddle_rotate(s->imlibdata, p);
+  }
   
   b = xitk_image_create_image(s->imlibdata, width, height);
-  draw_inner(s->imlibdata, b->image, width, height);
+  xitk_image_add_mask(s->imlibdata, b);
+  if((type == XITK_HSLIDER) || (type == XITK_VSLIDER))
+    draw_inner(s->imlibdata, b->image, width, height);
+  else if(type == XITK_RSLIDER) {
+    draw_rotate_button(s->imlibdata, b);
+  }
+  
+  radius = (b->height >> 1) - (p->height);
 
-  return _xitk_slider_create(NULL, s, x, y, NULL, b, p, type);
+  return _xitk_slider_create(NULL, s, x, y, NULL, b, p, type, radius, 1, 1);
 }
