@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: slider.h,v 1.7 2001/11/13 23:21:53 f1rmb Exp $
+ * $Id: slider.h,v 1.9 2002/03/21 00:22:53 f1rmb Exp $
  *
  */
 
@@ -29,10 +29,12 @@
 #include "widget.h"
 #include "_xitk.h"
 
-/*  To handle a vertical slider */
+/*  vertical slider type */
 #define XITK_VSLIDER 1
-/*  To handle an horizontal slider */
+/*  horizontal slider type */
 #define XITK_HSLIDER 2
+/*  rotate button *slider* type */
+#define XITK_RSLIDER 3
 
 typedef struct {
   ImlibData              *imlibdata;
@@ -42,15 +44,20 @@ typedef struct {
   int                     sType;
   int                     bClicked;
   int                     bArmed;
-  int 	                  pos;
-  int                     max;
-  int                     step;
-  int                     min;
-  int                     realmin;
-  int                     realmax;
 
-  xitk_image_t            *paddle_skin;
-  xitk_image_t            *bg_skin;
+
+  float                   angle;
+  float                   percentage;
+
+  float                   upper;
+  float                   value;
+  float                   lower;
+  int                     step;
+
+  int                     radius;
+
+  xitk_image_t           *paddle_skin;
+  xitk_image_t           *bg_skin;
 
   xitk_state_callback_t   motion_callback;
   void                   *motion_userdata;
