@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: window.c,v 1.10 2001/12/21 04:44:09 f1rmb Exp $
+ * $Id: window.c,v 1.15 2002/04/19 20:59:49 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -70,6 +70,28 @@ static void _xitk_window_destroy_window(xitk_widget_t *, void *);
 
 #define TITLE_BAR_HEIGHT 20
 
+/*
+ * Is window is size match with given args
+ */
+int xitk_is_window_size(Display *display, Window window, int width, int height) {
+  XWindowAttributes  wattr;
+  
+  if((display == NULL) || (window == None))
+    return -1;
+  
+  XLOCK(display);
+  if(!XGetWindowAttributes(display, window, &wattr)) {
+    XITK_WARNING("XGetWindowAttributes() failed.n");
+    return -1;
+  }
+  XUNLOCK(display);
+  
+  if((wattr.width == width) && (wattr.height == height))
+    return 1;
+  
+  return 0;
+}
+
 /*
  * Set/Change window title.
  */
@@ -234,9 +256,7 @@ xitk_window_t *xitk_window_create_window(ImlibData *im, int x, int y, int width,
   
   XSetStandardProperties(im->x.disp, xwin->window, title, title, None, NULL, 0, &hint);
 
-  XSelectInput(im->x.disp, xwin->window,
-	       ButtonPressMask | ButtonReleaseMask | PointerMotionMask 
-	       | KeyPressMask | KeymapStateMask | ExposureMask | StructureNotifyMask);
+  XSelectInput(im->x.disp, xwin->window, INPUT_MOTION | KeymapStateMask);
   
   XA_WIN_LAYER = XInternAtom(im->x.disp, "_WIN_LAYER", False);
   
@@ -290,15 +310,16 @@ xitk_window_t *xitk_window_create_simple_window(ImlibData *im, int x, int y, int
   xwin->height = height;
   
   XLOCK(im->x.disp);
-  
   gcv.graphics_exposures = False;
   gc = XCreateGC(im->x.disp, xwin->window, GCGraphicsExposures, &gcv);
+  XUNLOCK(im->x.disp);
   
   xwin->background = xitk_image_create_pixmap(im, width, height);
   draw_outter(im, xwin->background, width, height);
   xitk_window_apply_background(im, xwin);
-  XFreeGC(im->x.disp, gc);
 
+  XLOCK(im->x.disp);
+  XFreeGC(im->x.disp, gc);
   XUNLOCK(im->x.disp);
 
   xitk_window_move_window(im, xwin, x, y);
@@ -358,26 +379,32 @@ xitk_window_t *xitk_window_create_dialog_window(ImlibData *im, char *title,
   XSetForeground(im->x.disp, gc, colorgray);
   XDrawLine(im->x.disp, bar, gc, width - 1, 0, width - 1, TITLE_BAR_HEIGHT - 1);
   XDrawLine(im->x.disp, bar, gc, 0, TITLE_BAR_HEIGHT - 1, width - 1, TITLE_BAR_HEIGHT - 1);
+  XUNLOCK(im->x.disp);
 
   fs = xitk_font_load_font(im->x.disp, DEFAULT_FONT_12);
   xitk_font_set_font(fs, gc);
   xitk_font_string_extent(fs, title, &lbear, &rbear, &wid, &asc, &des);
 
+  XLOCK(im->x.disp);
   XSetForeground(im->x.disp, gc, colorwhite);
   XDrawString(im->x.disp, bar, gc, 
 	      (width - wid) - TITLE_BAR_HEIGHT, ((TITLE_BAR_HEIGHT+asc+des) >> 1) - des, 
 	      title, strlen(title));
 
+  XUNLOCK(im->x.disp);
+
   xitk_font_unload_font(fs);
 
+  XLOCK(im->x.disp);
   XCopyArea(im->x.disp, bar, pix_bg, gc, 0, 0, width, TITLE_BAR_HEIGHT, 0, 0);
+  XUNLOCK(im->x.disp);
   
   xitk_window_change_background(im, xwin, pix_bg, width, height);
   
+  XUNLOCK(im->x.disp);
   XFreePixmap(im->x.disp, bar);
   XFreePixmap(im->x.disp, pix_bg);
   XFreeGC(im->x.disp, gc);
-
   XUNLOCK(im->x.disp);
 
   return xwin;
@@ -447,8 +474,11 @@ int xitk_window_change_background(ImlibData *im,
 
   XLOCK(im->x.disp);
   XFreePixmap(im->x.disp, w->background);
+  XUNLOCK(im->x.disp);
+
   w->background = xitk_image_create_pixmap(im, width, height);
   
+  XLOCK(im->x.disp);
   if(XGetGeometry(im->x.disp, w->window, &rootwin, 
 		  &xwin, &ywin, &wwin, &hwin, &bwin, &dwin) != BadDrawable) {
     
@@ -460,9 +490,12 @@ int xitk_window_change_background(ImlibData *im,
   gcv.graphics_exposures = False;
   gc = XCreateGC(im->x.disp, bg, GCGraphicsExposures, &gcv);
   XCopyArea(im->x.disp, bg, w->background, gc, 0, 0, width, height, 0, 0);
-  xitk_window_apply_background(im, w);
-  XFreeGC(im->x.disp, gc);
+  XUNLOCK(im->x.disp);
 
+  xitk_window_apply_background(im, w);
+
+  XLOCK(im->x.disp);
+  XFreeGC(im->x.disp, gc);
   XUNLOCK(im->x.disp);
   
   return 1;
@@ -523,8 +556,9 @@ void xitk_window_destroy_window(ImlibData *im, xitk_window_t *w) {
   w->width = -1;
   w->height = -1;
   XDestroyWindow(im->x.disp, w->window);
-  XITK_FREE(w);
   XUNLOCK(im->x.disp);
+
+  XITK_FREE(w);
 }
 
 /*
@@ -581,9 +615,9 @@ static void _xitk_window_destroy_window(xitk_widget_t *w, void *data) {
 /*
  * Create a window error, containing an error message.
  */
-void xitk_window_dialog_ok_with_width(ImlibData *im, char *title,
-				      xitk_state_callback_t cb, void *userdata, 
-				      int window_width, int align, char *message, ...) {
+void xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *button_label,
+					      xitk_state_callback_t cb, void *userdata, 
+					      int window_width, int align, char *message, ...) {
   xitk_dialog_t              *wd;
   xitk_labelbutton_widget_t   lb;
   int                         windoww = window_width, windowh;
@@ -655,7 +689,7 @@ void xitk_window_dialog_ok_with_width(ImlibData *im, char *title,
 
   XITK_WIDGET_INIT(&lb, im);
   lb.button_type       = CLICK_BUTTON;
-  lb.label             = _("OK");
+  lb.label             = button_label;
   lb.align             = LABEL_ALIGN_CENTER;
   lb.callback          = _xitk_window_destroy_window;
   lb.state_callback    = NULL;
@@ -701,7 +735,7 @@ void xitk_window_dialog_ok_with_width(ImlibData *im, char *title,
   XMapRaised(im->x.disp, (xitk_window_get_window(wd->xwin)));
   XUNLOCK(im->x.disp);
 
-  wd->key = xitk_register_event_handler("xitk_ok", 
+  wd->key = xitk_register_event_handler("xitk_1btn", 
 					(xitk_window_get_window(wd->xwin)),
 					_window_handle_event,
 					NULL,
@@ -711,14 +745,49 @@ void xitk_window_dialog_ok_with_width(ImlibData *im, char *title,
 
 }
 
+void xitk_window_dialog_ok_with_width(ImlibData *im, char *title,
+				      xitk_state_callback_t cb, void *userdata, 
+				      int window_width, int align, char *message, ...) {
+
+  va_list   args;
+  char     *buf;
+  int       n, size = 100;
+  
+  if((buf = xitk_xmalloc(size)) == NULL) 
+    return;
+  
+  while(1) {
+    
+    va_start(args, message);
+    n = vsnprintf(buf, size, message, args);
+    va_end(args);
+    
+    if(n > -1 && n < size)
+      break;
+    
+    if(n > -1)
+      size = n + 1;
+    else
+      size *= 2;
+    
+    if((buf = realloc(buf, size)) == NULL)
+      return;
+  }
+  
+  xitk_window_dialog_one_button_with_width(im, title, _("OK"), cb, userdata, window_width,
+					   align, buf);
+  XITK_FREE(buf);
+}
+
 /*
  * Create an interactive window, containing 'yes', 'no', 'cancel' buttons.
  */
-void xitk_window_dialog_yesno_with_width(ImlibData *im, char *title,
-					 xitk_state_callback_t ycb, 
-					 xitk_state_callback_t ncb, 
-					 void *userdata, 
-					 int window_width, int align, char *message, ...) {
+void xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
+					       char *button1_label, char *button2_label,
+					       xitk_state_callback_t cb1, 
+					       xitk_state_callback_t cb2, 
+					       void *userdata, 
+					       int window_width, int align, char *message, ...) {
   xitk_dialog_t              *wd;
   xitk_labelbutton_widget_t   lb;
   int                         windoww = window_width, windowh;
@@ -764,8 +833,8 @@ void xitk_window_dialog_yesno_with_width(ImlibData *im, char *title,
 
   wd->imlibdata = im;
   wd->type = DIALOG_TYPE_YESNO;
-  wd->yescallback = ycb;
-  wd->nocallback = ncb;
+  wd->yescallback = cb1;
+  wd->nocallback = cb2;
   wd->userdata = userdata;
   wd->xwin = xitk_window_create_dialog_window(im, ((title != NULL) ? title : _("Question?")), 
 					      0, 0, windoww, windowh);  
@@ -791,7 +860,7 @@ void xitk_window_dialog_yesno_with_width(ImlibData *im, char *title,
 
   XITK_WIDGET_INIT(&lb, im);
   lb.button_type       = CLICK_BUTTON;
-  lb.label             = _("Yes");
+  lb.label             = button1_label;
   lb.align             = LABEL_ALIGN_CENTER;
   lb.callback          = _xitk_window_destroy_window;
   lb.state_callback    = NULL;
@@ -805,7 +874,7 @@ void xitk_window_dialog_yesno_with_width(ImlibData *im, char *title,
 					   "Black", "Black", "White", DEFAULT_FONT_12)));
 
   lb.button_type       = CLICK_BUTTON;
-  lb.label             = _("No");
+  lb.label             = button2_label;
   lb.align             = LABEL_ALIGN_CENTER;
   lb.callback          = _xitk_window_destroy_window;
   lb.state_callback    = NULL;
@@ -852,7 +921,7 @@ void xitk_window_dialog_yesno_with_width(ImlibData *im, char *title,
   XMapRaised(im->x.disp, (xitk_window_get_window(wd->xwin)));
   XUNLOCK(im->x.disp);
 
-  wd->key = xitk_register_event_handler("xitk_yesno", 
+  wd->key = xitk_register_event_handler("xitk_2btns", 
 					(xitk_window_get_window(wd->xwin)),
 					_window_handle_event,
 					NULL,
@@ -861,15 +930,56 @@ void xitk_window_dialog_yesno_with_width(ImlibData *im, char *title,
 					(void *)wd);
 }
 
+/*
+ *
+ */
+void xitk_window_dialog_yesno_with_width(ImlibData *im, char *title,
+					 xitk_state_callback_t ycb, 
+					 xitk_state_callback_t ncb, 
+					 void *userdata, 
+					 int window_width, int align, char *message, ...) {
+  va_list   args;
+  char     *buf;
+  int       n, size = 100;
+  
+  if((buf = xitk_xmalloc(size)) == NULL) 
+    return;
+  
+  while(1) {
+    
+    va_start(args, message);
+    n = vsnprintf(buf, size, message, args);
+    va_end(args);
+    
+    if(n > -1 && n < size)
+      break;
+    
+    if(n > -1)
+      size = n + 1;
+    else
+      size *= 2;
+    
+    if((buf = realloc(buf, size)) == NULL)
+      return;
+  }
+  
+  xitk_window_dialog_two_buttons_with_width(im, title, _("Yes"), _("No"), 
+					    ycb, ncb, userdata, window_width, align, buf);
+  XITK_FREE(buf);
+}
+
 /*
  * Create an interactive window, containing 'yes', 'no', 'cancel' buttons.
  */
-void xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *title,
-					       xitk_state_callback_t ycb, 
-					       xitk_state_callback_t ncb, 
-					       xitk_state_callback_t ccb, 
-					       void *userdata, 
-					       int window_width, int align, char *message, ...) {
+void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
+						 char *button1_label,
+						 char *button2_label,
+						 char *button3_label,
+						 xitk_state_callback_t cb1, 
+						 xitk_state_callback_t cb2, 
+						 xitk_state_callback_t cb3, 
+						 void *userdata, 
+						 int window_width, int align, char *message, ...) {
   xitk_dialog_t              *wd;
   xitk_labelbutton_widget_t   lb;
   int                         windoww = window_width, windowh;
@@ -915,9 +1025,9 @@ void xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *title,
 
   wd->imlibdata = im;
   wd->type = DIALOG_TYPE_YESNOCANCEL;
-  wd->yescallback = ycb;
-  wd->nocallback = ncb;
-  wd->cancelcallback = ccb;
+  wd->yescallback = cb1;
+  wd->nocallback = cb2;
+  wd->cancelcallback = cb3;
   wd->userdata = userdata;
   wd->xwin = xitk_window_create_dialog_window(im, ((title != NULL) ? title : _("Question?")), 
 					      0, 0, windoww, windowh);  
@@ -944,7 +1054,7 @@ void xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *title,
 
   XITK_WIDGET_INIT(&lb, im);
   lb.button_type       = CLICK_BUTTON;
-  lb.label             = _("Yes");
+  lb.label             = button1_label;
   lb.align             = LABEL_ALIGN_CENTER;
   lb.callback          = _xitk_window_destroy_window;
   lb.state_callback    = NULL;
@@ -958,7 +1068,7 @@ void xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *title,
 					   "Black", "Black", "White", DEFAULT_FONT_12)));
 
   lb.button_type       = CLICK_BUTTON;
-  lb.label             = _("No");
+  lb.label             = button2_label;
   lb.align             = LABEL_ALIGN_CENTER;
   lb.callback          = _xitk_window_destroy_window;
   lb.state_callback    = NULL;
@@ -972,7 +1082,7 @@ void xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *title,
 					   "Black", "Black", "White", DEFAULT_FONT_12)));
 
   lb.button_type       = CLICK_BUTTON;
-  lb.label             = _("Cancel");
+  lb.label             = button3_label;
   lb.align             = LABEL_ALIGN_CENTER;
   lb.callback          = _xitk_window_destroy_window;
   lb.state_callback    = NULL;
@@ -1019,7 +1129,7 @@ void xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *title,
   XMapRaised(im->x.disp, (xitk_window_get_window(wd->xwin)));
   XUNLOCK(im->x.disp);
 
-  wd->key = xitk_register_event_handler("xitk_yesnocancel", 
+  wd->key = xitk_register_event_handler("xitk_3btns", 
 					(xitk_window_get_window(wd->xwin)),
 					_window_handle_event,
 					NULL,
@@ -1027,3 +1137,39 @@ void xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *title,
 					wd->widget_list,
 					(void *)wd);
 }
+
+void xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *title,
+					       xitk_state_callback_t ycb, 
+					       xitk_state_callback_t ncb, 
+					       xitk_state_callback_t ccb, 
+					       void *userdata, 
+					       int window_width, int align, char *message, ...) {
+  va_list   args;
+  char     *buf;
+  int       n, size = 100;
+  
+  if((buf = xitk_xmalloc(size)) == NULL) 
+    return;
+  
+  while(1) {
+    
+    va_start(args, message);
+    n = vsnprintf(buf, size, message, args);
+    va_end(args);
+    
+    if(n > -1 && n < size)
+      break;
+    
+    if(n > -1)
+      size = n + 1;
+    else
+      size *= 2;
+    
+    if((buf = realloc(buf, size)) == NULL)
+      return;
+  }
+  
+  xitk_window_dialog_three_buttons_with_width(im, title, _("Yes"), _("No"), _("Cancel"),
+					      ycb, ncb, ccb, userdata, window_width, align, buf);
+  XITK_FREE(buf);
+}
