@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,11 +17,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xitk.c,v 1.29 2002/01/09 00:29:56 f1rmb Exp $
+ * $Id: xitk.c,v 1.35 2002/04/21 22:17:12 f1rmb Exp $
  *
  */
 #define _XITK_C_
 
+/* required to enable POSIX variant of getpwuid_r on solaris */
+#define _POSIX_PTHREAD_SEMANTICS 1
+
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -90,16 +94,16 @@ typedef struct {
   Window                      window;
   Atom                        XA_XITK;
 
-  xitk_move_t                  move;
+  xitk_move_t                 move;
 
   struct {
-    int                         x;
-    int                         y;
+    int                       x;
+    int                       y;
   } old_pos;
 
   struct {
-    int                         x;
-    int                         y;
+    int                       x;
+    int                       y;
   } new_pos;
 
   int                         width;
@@ -182,7 +186,7 @@ int xitk_system(int dont_run_as_root, char *command) {
 /*
  * A thread-safe usecond sleep
  */
-void xitk_usec_sleep(unsigned usec) {
+void xitk_usec_sleep(unsigned long usec) {
 #if HAVE_NANOSLEEP
   /* nanosleep is prefered on solaris, because it's mt-safe */
   struct timespec ts;
@@ -211,13 +215,13 @@ static void xitk_signal_handler(int sig) {
   case SIGTERM:
   case SIGQUIT:
     if(cur_pid == xitk_pid) {
-      __gfx_t   *fx;
+      __gfx_t  *fx;
       
       XITK_WARNING("XITK killed with signal %d\n", sig);
 
       MUTLOCK();
       gXitk->running = 0;
-
+      
       /* 
        * Tada.... and now Ladies and Gentlemen, the dirty hack
        */
@@ -288,14 +292,13 @@ void xitk_change_window_for_event_handler (xitk_register_key_t key, Window windo
 
     if(fx->key == key) {
 
-      XLOCK(gXitk->display);
-
       fx->window = window;
 
-      if(fx->xdnd && (window != None))
-	xitk_make_window_dnd_aware(fx->xdnd, window);
+      if(fx->xdnd && (window != None)) {
+	if(!xitk_make_window_dnd_aware(fx->xdnd, window))
+	  xitk_unset_dnd_callback(fx->xdnd);
+      }
       
-      XUNLOCK(gXitk->display);
       MUTUNLOCK();
       return;
     }
@@ -346,7 +349,7 @@ xitk_register_key_t xitk_register_event_handler(char *name, Window window,
     XLOCK(gXitk->display);
     err = XGetWindowAttributes(gXitk->display, fx->window, &wattr);
     XUNLOCK(gXitk->display);
-
+    
     if(err != BadDrawable && err != BadWindow) {
       fx->new_pos.x = wattr.x;
       fx->new_pos.y = wattr.y;
@@ -377,8 +380,8 @@ xitk_register_key_t xitk_register_event_handler(char *name, Window window,
     fx->xdnd = (xitk_dnd_t *) xitk_xmalloc(sizeof(xitk_dnd_t));
     
     xitk_init_dnd(gXitk->display, fx->xdnd);
-    xitk_set_dnd_callback(fx->xdnd, dnd_cb);
-    xitk_make_window_dnd_aware(fx->xdnd, fx->window);
+    if(xitk_make_window_dnd_aware(fx->xdnd, fx->window))
+      xitk_set_dnd_callback(fx->xdnd, dnd_cb);
   }
   else
   fx->xdnd = NULL;
@@ -432,7 +435,6 @@ void xitk_unregister_event_handler(xitk_register_key_t *key) {
     fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
 
   }
-  
 
   MUTUNLOCK();
 }
@@ -530,18 +532,22 @@ void xitk_xevent_notify(XEvent *event) {
 
 	    XMoveWindow(gXitk->display, fx->window,
 			fx->new_pos.x, fx->new_pos.y);
-
 	    err = XGetWindowAttributes(gXitk->display, fx->window, &wattr);
+
+	    XUNLOCK(gXitk->display);
+
 	    if(err != BadDrawable && err != BadWindow) {
 	      if(wattr.your_event_mask & PointerMotionMask) {
-
+		
+		XLOCK(gXitk->display);
 		while(XCheckWindowEvent(gXitk->display, fx->window, 
 					PointerMotionMask, event) == True);
+		XUNLOCK(gXitk->display);
 	      }
 	    }
 
-	    XFlush(gXitk->display);
-
+	    XLOCK(gXitk->display);
+	    XSync(gXitk->display, False);
 	    XUNLOCK(gXitk->display);
 
 	  }
@@ -585,6 +591,8 @@ void xitk_xevent_notify(XEvent *event) {
 
 	      XLOCK(gXitk->display);
 	      err = XGetWindowAttributes(gXitk->display, fx->window, &wattr);
+	      XUNLOCK(gXitk->display);
+
 	      if(err != BadDrawable && err != BadWindow) {
 		
 		fx->old_pos.x = event->xmotion.x_root - event->xbutton.x;
@@ -592,8 +600,6 @@ void xitk_xevent_notify(XEvent *event) {
 
 	      }
 	      
-	      XUNLOCK(gXitk->display);
-
 	      fx->move.offset_x = event->xbutton.x;
 	      fx->move.offset_y = event->xbutton.y;
 
@@ -638,12 +644,13 @@ void xitk_xevent_notify(XEvent *event) {
 	}
 	break;
 
+	case SelectionNotify:
 	case ClientMessage:
 	  if(fx->xdnd)
 	    xitk_process_client_dnd_message(fx->xdnd, event);
 	  break;
 	}
-
+	
 	if(fx->xevent_callback) {
 	  fx->xevent_callback(event, fx->user_data);
 	}
@@ -829,6 +836,9 @@ int xitk_get_focus_color(void) {
 int xitk_get_select_color(void) {
   return xitk_config_get_select_color(gXitk->config);
 }
+unsigned long xitk_get_timer_label_animation(void) {
+  return xitk_config_get_timer_label_animation(gXitk->config);
+}
 
 /*
  *
@@ -858,7 +868,7 @@ const char *xitk_get_homedir(void) {
   char *buffer = NULL;
   int bufsize = 128;
 
-  buffer = (char *) xmalloc(bufsize);
+  buffer = (char *) xitk_xmalloc(bufsize);
   
   if((ret = getpwuid_r(getuid(), &pwd, buffer, bufsize, &pw)) < 0) {
 #else
