@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xitk.h,v 1.43 2002/01/15 22:22:36 f1rmb Exp $
+ * $Id: xitk.h,v 1.54 2002/04/19 20:59:49 f1rmb Exp $
  *
  * Unique public header file for xitk TK.
  *
@@ -34,6 +34,7 @@
 #include <limits.h>
 #include <X11/Xlib.h>
 #include <pthread.h>
+#include <stdarg.h>
 #include "Imlib-light/Imlib.h"
 
 #ifdef NEED_MRLBROWSER
@@ -54,6 +55,11 @@
 #define XITK_PATH_MAX           PATH_MAX
 #endif
 
+#define INPUT_MOTION (ExposureMask | ButtonPressMask | ButtonReleaseMask |    \
+                      KeyPressMask | KeyReleaseMask | ButtonMotionMask |      \
+                      StructureNotifyMask | PropertyChangeMask |              \
+                      LeaveWindowMask | EnterWindowMask | PointerMotionMask)
+
 #define MWM_HINTS_DECORATIONS   (1L << 1)
 #define PROP_MWM_HINTS_ELEMENTS 5
 typedef struct _mwmhints {
@@ -189,7 +195,7 @@ typedef struct xitk_widget_s {
   widget_get_skin_t               get_skin;
 
   pthread_t                       tips_thread;
-  int                             tips_timeout;
+  unsigned long                   tips_timeout;
   char                           *tips_string;
 
   void                           *private_data;
@@ -545,10 +551,12 @@ void xitk_xevent_notify(XEvent *event);
  * *** Sliders ***
  */
 
-/*  To handle a vertical slider */
+/*  vertical slider type */
 #define XITK_VSLIDER 1
-/*  To handle an horizontal slider */
+/*  horizontal slider type */
 #define XITK_HSLIDER 2
+/*  rotate button *slider* type */
+#define XITK_RSLIDER 3
 
 typedef struct {
   int                       magic;
@@ -813,11 +821,12 @@ void xitk_image_change_image(ImlibData *im,
  */
 void xitk_init_dnd(Display *display, xitk_dnd_t *);
 
-void xitk_make_window_dnd_aware(xitk_dnd_t *, Window);
+int xitk_make_window_dnd_aware(xitk_dnd_t *, Window);
 
-Bool xitk_process_client_dnd_message(xitk_dnd_t *, XEvent *);
+int xitk_process_client_dnd_message(xitk_dnd_t *, XEvent *);
 
 void xitk_set_dnd_callback(xitk_dnd_t *, void *);
+void xitk_unset_dnd_callback(xitk_dnd_t *);
 
 
 /*
@@ -1198,6 +1207,11 @@ int xitk_mrlbrowser_get_window_info(xitk_widget_t *w, window_info_t *inf);
  */
 Window xitk_mrlbrowser_get_window_id(xitk_widget_t *w);
 
+/**
+ *
+ */
+void xitk_mrlbrowser_set_tips_timeout(xitk_widget_t *w, int enabled, unsigned long timeout);
+
 #endif
 
 typedef struct {
@@ -1284,6 +1298,16 @@ int xitk_skin_check_version(xitk_skin_config_t *, int);
  */
 void xitk_skin_unload_config(xitk_skin_config_t *);
 
+/*
+ *
+ */
+int xitk_skin_get_visibility(xitk_skin_config_t *, const char *);
+
+/*
+ *
+ */
+int xitk_skin_get_enability(xitk_skin_config_t *, const char *);
+
 /*
  *
  */
@@ -1332,7 +1356,12 @@ char *xitk_skin_get_label_fontname(xitk_skin_config_t *, const char *);
 /*
  *
  */
-int xitk_skin_get_label_printable(xitk_skin_config_t *skonfig, const char *str);
+int xitk_skin_get_label_printable(xitk_skin_config_t *, const char *);
+
+/*
+ *
+ */
+int xitk_skin_get_label_staticity(xitk_skin_config_t *, const char *);
 
 /*
  *
@@ -1342,12 +1371,27 @@ char *xitk_skin_get_skin_filename(xitk_skin_config_t *, const char *);
 /*
  *
  */
-char *xitk_skin_get_slider_skin_filename(xitk_skin_config_t *skonfig, const char *str);
+char *xitk_skin_get_slider_skin_filename(xitk_skin_config_t *, const char *);
 
 /*
  *
  */
-int xitk_skin_get_slider_type(xitk_skin_config_t *skonfig, const char *str);
+int xitk_skin_get_slider_type(xitk_skin_config_t *, const char *);
+
+/*
+ *
+ */
+int xitk_skin_get_slider_radius(xitk_skin_config_t *, const char *);
+
+/*
+ *
+ */
+void xitk_skin_lock(xitk_skin_config_t *skonfig);
+
+/*
+ *
+ */
+void xitk_skin_unlock(xitk_skin_config_t *skonfig);
 
 
 /**
@@ -1607,6 +1651,9 @@ void draw_outter_frame(ImlibData *im, Pixmap p, char *title, char *fontname,
 
 void draw_tab(ImlibData *im, xitk_image_t *p);
 
+void draw_paddle_rotate(ImlibData *im, xitk_image_t *p);
+void draw_rotate_button(ImlibData *im, xitk_image_t *p);
+
 #define XITK_WINDOW_ANSWER_UNKNOWN 0
 #define XITK_WINDOW_ANSWER_OK      1
 #define XITK_WINDOW_ANSWER_YES     2
@@ -1695,11 +1742,23 @@ void xitk_set_window_title(Display *display, Window window, char *title);
  */
 void xitk_window_set_window_title(ImlibData *im, xitk_window_t *w, char *title);
 
+/*
+ *
+ */
+int xitk_is_window_size(Display *display, Window window, int width, int height);
+
 #define ALIGN_LEFT    1
 #define ALIGN_CENTER  2
 #define ALIGN_RIGHT   3
 #define ALIGN_DEFAULT (ALIGN_LEFT)
 
+/*
+ *
+ */
+void xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *button_label,
+					      xitk_state_callback_t cb, void *userdata, 
+					      int window_width, int align, char *message, ...);
+
 /**
  *
  */
@@ -1713,6 +1772,7 @@ void xitk_window_dialog_ok_with_width(ImlibData *im, char *title,
 void xitk_window_dialog_ok(ImlibData *im, char *title,
 			   xitk_state_callback_t cb, void *userdata, int align, char *message, ...);
 
+#ifdef	__GNUC__
 #define xitk_window_dialog_ok(im, title, cb, userdata, align, message, args...) {                \
   if(((im) == NULL) || ((message) == NULL))                                                      \
     return;                                                                                      \
@@ -1728,7 +1788,33 @@ void xitk_window_dialog_ok(ImlibData *im, char *title,
     return;                                                                                            \
   xitk_window_dialog_ok_with_width(im, "Information", NULL, NULL, 400, ALIGN_CENTER, message, ##args); \
 }
+#else
+#define xitk_window_dialog_ok(im, title, cb, userdata, align, ...) {                               \
+  if(((im) == NULL))                                                                               \
+    return;                                                                                        \
+  xitk_window_dialog_ok_with_width(im, title, cb, userdata, 400, align, __VA_ARGS__);              \
+}
+#define xitk_window_dialog_error(im, ...) {                                                        \
+  if(((im) == NULL))                                                                               \
+    return;                                                                                        \
+  xitk_window_dialog_ok_with_width(im, "Error", NULL, NULL, 400, ALIGN_CENTER, __VA_ARGS__);       \
+}
+#define xitk_window_dialog_info(im, ...) {                                                         \
+  if(((im) == NULL))                                                                               \
+    return;                                                                                        \
+  xitk_window_dialog_ok_with_width(im, "Information", NULL, NULL, 400, ALIGN_CENTER, __VA_ARGS__); \
+}
+#endif
 
+/*
+ *
+ */
+void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
+						 xitk_state_callback_t cb1, 
+						 xitk_state_callback_t cb2, 
+						 xitk_state_callback_t cb3, 
+						 void *userdata, 
+						 int window_width, int align, char *message, ...);
 /**
  *
  */
@@ -1748,11 +1834,29 @@ void xitk_window_dialog_yesnocancel(ImlibData *im, char *title,
 				    xitk_state_callback_t ccb, 
 				    void *userdata, int align, char *message, ...);
 
+#ifdef __GNUC__
 #define xitk_window_dialog_yesnocancel(im, title, ycb, ncb, ccb, userdata, align, message, args...) {         \
   if(((im) == NULL) || ((message) == NULL))                                                                   \
     return;                                                                                                   \
   xitk_window_dialog_yesnocancel_with_width(im, title, ycb, ncb, ccb, userdata, 400, align, message, ##args); \
 }
+#else
+#define xitk_window_dialog_yesnocancel(im, title, ycb, ncb, ccb, userdata, align, ...) {         \
+  if(((im) == NULL))                                                                   \
+    return;                                                                                                   \
+  xitk_window_dialog_yesnocancel_with_width(im, title, ycb, ncb, ccb, userdata, 400, align, __VA_ARGS__); \
+}
+#endif
+
+/*
+ *
+ */
+void xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
+					       char *button1_label, char *button2_label,
+					       xitk_state_callback_t cb1, 
+					       xitk_state_callback_t cb2, 
+					       void *userdata, 
+					       int window_width, int align, char *message, ...);
 
 /**
  *
@@ -1770,11 +1874,19 @@ void xitk_window_dialog_yesno(ImlibData *im, char *title,
 			      xitk_state_callback_t ycb, 
 			      xitk_state_callback_t ncb, 
 			      void *userdata, int align, char *message, ...);
+#ifdef __GNUC__
 #define xitk_window_dialog_yesno(im, title, ycb, ncb, userdata, align, message, args...) {         \
   if(((im) == NULL) || ((message) == NULL))                                                        \
     return;                                                                                        \
   xitk_window_dialog_yesno_with_width(im, title, ycb, ncb, userdata, 400, align, message, ##args); \
 }
+#else
+#define xitk_window_dialog_yesno(im, title, ycb, ncb, userdata, align, ...) {                      \
+  if(((im) == NULL))                                                                               \
+    return;                                                                                        \
+  xitk_window_dialog_yesno_with_width(im, title, ycb, ncb, userdata, 400, align, __VA_ARGS__);     \
+}
+#endif
 
 
 typedef struct {
